
/*
	file...: main.c
	purpose: generate generic shift restraint file from sequence

	author.: Robert Boyko
	date...: Feb 17, 1999	
	place..: University of Alberta / Department of Biochemistry
*/

#include "defs.h"

/* procedure calls */
void init_makeshift();
extern void read_seq_file(), do_makeshift(), read_bmrb_shift_file();
extern int read_spec_file();

char dataFname[MAX_FNAME_LEN];
char bmrbFname[MAX_FNAME_LEN];
SEQ_REC mySeq;
AMINO_ACID_REC aaTable[CODES];


/*************************************************************/
/* main program
*/
int main(argc, argv)
int argc;
char *argv[];
{

	init_makeshift(argc, argv);
	do_makeshift();
	return(0);

}	/* main */


/*************************************************************/
/* analyzing arguments to the program.
*/
void init_makeshift(argc, argv)
int argc;
char *argv[];
{
	int rc;
	char sVal[MAX_LINE_LEN];
	FILE *fp;
		 
	if (argc == 0) {
		show_err(TRUE, "init_makeshift", "Usage: makeshift seqFile defaultShiftsFile");
	}

	strcpy(specFname, argv[1]);
	if (NULL == (fp = (FILE *)fopen(specFname, "r"))) {
		sprintf(sVal, "Cannot open atom spec file: %s\n", specFname);
		show_err(TRUE, "init_makeshift", sVal);
	}
	rc = read_spec_file(fp, aaTable);
	if (rc == ERROR_FLAG) {
		sprintf(sVal, "No amino acids read, check spec file: %s\n", specFname);
		show_err(TRUE, "init_makeshift", sVal);
	}
	fclose(fp);

	strcpy(bmrbFname, argv[2]);
	if (NULL == (fp = (FILE *)fopen(bmrbFname, "r"))) {
		sprintf(sVal, "Cannot open bmrb default shifts file: %s\n", bmrbFname);
		show_err(TRUE, "init_makeshift", sVal);
	}
	read_bmrb_shift_file(fp, aaTable);
	fclose(fp);

	strcpy(dataFname, argv[3]);
	if (NULL == (fp = (FILE *)fopen(dataFname, "r"))) {
		sprintf(sVal, "Cannot open sequence file: %s\n", dataFname);
		show_err(TRUE, "init_makeshift", sVal);
	}
	read_seq_file(fp, &mySeq);
	fclose(fp);

}	/* init_makeshift */
