
/*
	orb header file
*/

#include "util/std.h" 

/********************/
/* Definitions      */
/********************/

/* program version and argument flags */
#define VERSION				"0.9"

/* name of default parameters file */
#define ORB_PARMS			"orb.parms"
#define DEFAULT_LOG_FILE	"orb.log"

#define WARN_FLAG			-1
#define ERROR_FLAG			-4
#define ERROR_STRING		"ERROR"
#define MAX_ERR_NUM			8
#define INP_SEQ_SCH			'>'
#define COMMENT				'#'

/* command line flags for specifying the parameter and output files */
#define PARMS_FLAG			"-i"
#define VERB_FLAG			"-v"
#define RESULTS_FLAG		"-o"
#define CSI_FLAG			"-c"
#define PPM_FLAG			"-p"

/* amino acid restrictions */
#define MAX_ATOM_NAME_LEN	5
#define MAX_RES_NAME_LEN	5
#define MAX_ATOM_SCOR		100
#define MAX_RES_ATOM_NUM	60	/* Be careful about lowering this value */

/* define secondary structure */
#define STRUCT_NUM			4
#define BETA				0
#define COIL				1
#define HELX				2
#define NO_SS				3
#define CH_B				'B'
#define CH_C				'C'
#define CH_H				'H'
#define CH_X				'X'
#define AA_SVALID(c) 		(((c == CH_B) || (c == CH_C) \
							|| (c == CH_H) \
							|| (c == CH_X))? TRUE: FALSE)

/* parameter file restrictions */
#define BNORM_STATES		3
#define DSW_NO_ENTRY		0
#define DSW_PSEUDO1			1
#define DSW_PSEUDO2			2
#define DSW_CREATE			(DSW_PSEUDO1 + DSW_PSEUDO2)
#define DSW_READ			(DSW_CREATE + 1)	

/* amino acid shift restrictions */
#define NO_SHIFT_VAL		999.99
#define NO_SHIFT_STR		"999.99"

/* ppm file keywords */
#define PPM_SEQ_CODE		"Sequence"

/* sequence alignment keywords (specific for xalign) */
#define TITLE_KEY			"Sequences"	
#define PCT_TABLE_KEY		"Pairwise"
#define STATS_KEY			"Statistics"
#define ALIGN_KEY			"Multiple"
#define S_TITLE_KEY			"Struct"	
#define CONSENSUS_KEY		"Summary"

/* amino acid weighting matrix restrictions */
#define WT_MAT_INIT			0
#define WT_MAT_DIM			32
#define WT_MAT_SIZE			(WT_MAT_DIM * WT_MAT_DIM)
#define MIN_WT_MAT_VAL		WT_MAT_INIT
#define MAX_WT_MAT_VAL		50

/* alignment restrictions */
#define MAX_SEQ_NUM			30
#define MAX_SEQ_LEN			300
#define MIN_SEQ_LEN			3
#define MAX_ALIGN_LEN		(MAX_SEQ_LEN * 2)	
#define RES_PER_LINE		50

/* homology calculation parameters */
#define HOMOL_WIND_SIZE		15

#define OPEN_RES_ID			'['

/* check for comment line */
#define COMMENT_LINE(c)		(((c == '!') || (c == '#'))? TRUE: FALSE)


/********************/
/* define structures */
/*********************/

struct atomRec {
	double dswShift[STRUCT_NUM];
	double dswStddev[STRUCT_NUM];
	char rName[MAX_ATOM_NAME_LEN];
};

struct aminoAcidRec {
	int atomNum; 
	struct atomRec atom[MAX_RES_ATOM_NUM];
	char rName[MAX_RES_NAME_LEN];
};

struct molMatRec {
	int atom1[MAX_RES_ATOM_NUM];
	int atom2[MAX_RES_ATOM_NUM];
	int scor[MAX_RES_ATOM_NUM];
};

struct alignResRec {
	char pRes;
	char sRes;
	int idNum;
	int homLocScor;
	int homStructScor;
	int resAbsScor;
	double shift1[MAX_RES_ATOM_NUM];
	double shift2[MAX_RES_ATOM_NUM];
};

struct alignSeqRec {
	char title[MAX_LINE_LEN];
	char id[MAX_ID_LEN+1];
	int numRes;
	int startIx, endIx;
	int homGlobScor;
	double refVal;
	double userShiftBias;
	char refAtom[MAX_ATOM_NAME_LEN];
	struct alignResRec residue[MAX_ALIGN_LEN];
};

struct homolRec {
	char *seqId;
	char *atom;
	int molScor;
	int ssaType;
	int selectShift;
	double ppm[2];
	double shift[2];
	double homWt;
	double wtScor;
};

struct predictRec {
	char *atom;
	double dswVal;
	double dswStd;
	double dswWt;
	double predict;
	double predStd;
	int homNum;
	struct homolRec hom[MAX_SEQ_NUM];
};

typedef struct atomRec ATOM_REC;
typedef struct aminoAcidRec AMINO_ACID_REC;
typedef struct molMatRec MOL_MAT_REC;
typedef struct predictRec PREDICT_REC;
typedef struct homolRec HOMOL_REC;
 
typedef struct alignSeqRec ALIGN_SEQ_REC;
typedef struct alignResRec ALIGN_RES_REC;
typedef struct alignAtomRec ALIGN_ATOM_REC;

/********************/
/* global variables */
/********************/

extern FILE		*outfp, *verbfp, *csifp, *ppmfp;

extern char		parmsFname[MAX_FNAME_LEN];
extern char		dataFname[MAX_FNAME_LEN];
extern char		resultsFname[MAX_FNAME_LEN];
extern char		csiFname[MAX_FNAME_LEN];
extern char		ppmFname[MAX_FNAME_LEN];
extern char		specFname[MAX_FNAME_LEN];
extern char		logFname[MAX_FNAME_LEN];
extern char		verbFname[MAX_FNAME_LEN];
extern char		molMatFname[MAX_FNAME_LEN];
extern char		wtHomolFname[MAX_FNAME_LEN];
extern char		wtStructFname[MAX_FNAME_LEN];
extern char		dswFname[MAX_FNAME_LEN];
extern char		noShiftStr[MAX_LINE_LEN];

extern char		name3letter[CODES][MAX_RES_NAME_LEN];
extern int		alignLen;
extern int		numSeq;
extern int		dswFileNum;
extern int		seqSelect;

extern ALIGN_SEQ_REC seqArr[MAX_SEQ_NUM];
extern AMINO_ACID_REC aaTable[CODES];
extern MOL_MAT_REC molMat[CODES][CODES];
extern int		molMatNum[CODES][CODES];
extern int		wtHomolMat[WT_MAT_SIZE];
extern int		wtStructMat[WT_MAT_SIZE];
extern int		printLogFlag;

extern double	csGlobMin, csLocMin, csMolMin, csStructMin;
extern double	csLocWind, csStructWind, csDbase;
extern double	wtStddev, stddevLb, stddevFactor;
