
/*
	file: ramino.c
	purpose: reads data to setup the amino acid table. 
*/

#include "defs.h" 

/* global variables */
char specFname[MAX_FNAME_LEN];
char name3letter[CODES][MAX_RES_NAME_LEN];

/* procedure calls */
static int read_spec_file(), read_spec_amino();
static int read_spec_code(), read_spec_atoms();
extern void show_err();


/*************************************************************/
/* read amino acid specification file 
*/
void read_spec_data(parmsfp)
FILE *parmsfp;
{
	int rc;
	FILE *fp;
	char sVal[MAX_LINE_LEN];

	if (read_prompt(parmsfp, PARMS_PROMPT) == FALSE) {
		sprintf(sVal, "Cannot find amino acid specification file\n");
		show_err(TRUE, "read_spec_data", sVal);
	}

	fscanf(parmsfp, "%s\n", specFname);
	fp = fopen(specFname, "r");
	if (fp == NULL) {
		sprintf(sVal, "Cannot open amino acid spec file: %s\n", specFname);
		show_err(TRUE, "read_spec_data", sVal);
	}

	fprintf(stdout, "Reading amino acid specification file: %s\n", specFname);
	rc = read_spec_file(fp, aaTable);
	if (rc == 0) {
		sprintf(sVal, "No amino acids read, check spec file: %s\n", specFname);
		show_err(TRUE, "read_spec_data", sVal);
	}

	fclose(fp);

}	/* read_spec_data */


/*************************************************************/
/* Read amino acid specification file 
*/
static int read_spec_file(fp, data)
FILE *fp;
AMINO_ACID_REC *data;
{
	int i, ix, count;
	char aStr[MAX_LINE_LEN], resName[MAX_LINE_LEN];

	count = 0;
	for (i=0; i<CODES; i++)
		data[i].atomNum = 0;

	while (TRUE) {

		if (EOF == fscanf(fp, "%s", aStr)) {
			break;
		}

		if (0 == strcmp(aStr, "AMINO_ACID")) {

			if (TRUE == read_spec_amino(fp, resName)) {
				ix = read_spec_code(fp);
				if (ix >= 0) {
					strcpy(name3letter[ix], resName);
					data[ix].atomNum = read_spec_atoms(fp, data, ix);
					count++;
				}
			}
		}
	}

	return(count);

}	/* read_spec_file */


/*************************************************************/
/* Read amino acid  3 letter code 
*/
static int read_spec_amino(fp, aStr)
FILE *fp;
char *aStr;
{
	char sVal[MAX_LINE_LEN];

	if (EOF == fscanf(fp, "%s", aStr)) {
		sprintf(sVal, "Unexpected EOF after AMINO_ACID.\n");
		show_err(TRUE, "read_spec_amino", sVal);
	}

	if (0 == strcmp(aStr, "BLNK"))
		return(FALSE);

	if (strlen(aStr) >= (MAX_RES_NAME_LEN - 1)) {
		sprintf(sVal, "Did not find 3 letter code: '%s'\n", aStr);
		show_err(TRUE, "read_spec_amino", sVal);
	}

	return(TRUE);

}	/* read_spec_amino */


/*************************************************************/
/* Read one letter code 
*/
static int read_spec_code(fp)
FILE *fp;
{
	char aStr[MAX_LINE_LEN], sVal[MAX_LINE_LEN];

/* find keyword */

	while (TRUE) {
		if (EOF == fscanf(fp, "%s", aStr)) {
			sprintf(sVal, "Unexpecting end of file at CHAR_CODE\n");
			show_err(TRUE, "read_spec_code", sVal);
		}
		if (0 == strcmp(aStr, "CHAR_CODE"))
			break;
	}

	if (EOF == fscanf(fp, "%s", aStr)) {
		sprintf(sVal, "Unexpecting end of file after CHAR_CODE\n");
		show_err(TRUE, "read_spec_code", sVal);
	}

	if (strlen(aStr) != 1) {
		sprintf(sVal, "Expecting 1 letter code but found: '%s'\n", aStr);
		show_err(TRUE, "read_spec_code", sVal);
	}

	if (AA_CVALID(aStr[0]) == FALSE) {
		sprintf(sVal, "Illegal 1 letter code found: '%s'\n", aStr);
		show_err(TRUE, "read_spec_code", sVal);
	}

	return(AA_IX(aStr[0]));

}	/* read_spec_code */


/*************************************************************/
/* Read atom names 
*/
static int read_spec_atoms(fp, data, ix)
FILE *fp;
AMINO_ACID_REC *data;
int ix;
{
	int count, len, j, k;
	char aStr[MAX_LINE_LEN], sVal[MAX_LINE_LEN];

	count = 0;

/* find keyword */

	while (TRUE) {
		if (EOF == fscanf(fp, "%s", aStr)) {
			sprintf(sVal, "Unexpecting end of file in read_spec_atoms\n");
			show_err(TRUE, "read_spec_atoms", sVal);
		}
		if (0 == strcmp(aStr, "VALID_ATOMS"))
			break;
	}

/* read each atom name */

	while (TRUE) {
		if (EOF == fscanf(fp, "%s", aStr)) {
			sprintf(sVal, "Unexpecting end of file while reading atoms\n");
			show_err(TRUE, "read_spec_atoms", sVal);
		}
		if (strlen(aStr) < 6) {
			if (0 != strcmp(aStr, "-")) {
				if (NULL != strchr(aStr, '-')) {
					len = strlen(aStr) - 1;
				} else {
					len = strlen(aStr);
				}
				if (count >= MAX_RES_ATOM_NUM) {
					sprintf(sVal, "Max atoms exceeded for: %s\n", aStr);
					show_err(TRUE, "read_spec_atoms", sVal);
				}
				strncpy(data[ix].atom[count].rName, aStr, len);
				count++;
			}
		} else {
			data[ix].atomNum = count;
			for (j=0; j<MAX_RES_ATOM_NUM; j++) {
				for (k=0; k<STRUCT_NUM; k++) {
					data[ix].atom[j].dswShift[k] = NO_SHIFT_VAL;
					data[ix].atom[j].dswStddev[k] = 0.0;
				}
			}
			return(count);
		}
	}

}	/* read_spec_atoms */
