
/*
	file: rshift.c
	read input ppm files (shifts of other proteins)	
*/

#include "defs.h"

/* procedure calls */
static FILE *read_ppm_fname();
static void init_ppm_shift();
static int proc_ppm_fname(), read_ppm_data();
static int scan_shift(), scan_star_shift(), proc_shift();
extern int do_seq_choice_menu(), check_rshift_atom(), check_rshift_shifts();
extern int pres_3to1();
extern void show_err(), exit();
extern double atof();


/*************************************************************/
/* read chemical shift files (ppm format) 
*/
void read_shift_files()
{
	register int i;
	char fname[MAX_FNAME_LEN], sVal[MAX_LINE_LEN];

	fprintf(stdout, "\n");
	fprintf(stdout, " Now is the time that you can enter chemical shift\n");
	fprintf(stdout, " files for your homologous sequences. Remember that\n");
	fprintf(stdout, " the program expects PPM format files.\n");

	sprintf(sVal, " Do you have your shifts files ready (y/n)?\n");
	if (FALSE == yes_no_query(sVal)) {
		fprintf(stdout, "\nBye\n");
		exit(0);
	}

	for (i=0; i<numSeq; i++) {

		init_ppm_shift(&seqArr[i]);
		if (i != seqSelect) {
			sprintf(sVal, " Do you have shifts for the sequence below %s %s",
				"(y/n)?\n   ", seqArr[i].title);
			if (TRUE == yes_no_query(sVal))
				if (FALSE == proc_ppm_fname(i, fname))
					i--;
		}
	}

}	/* read_shift_files */


/*************************************************************/
/* process chemical shift files (ppm format) 
*/
static int proc_ppm_fname(ix, fname)
int ix;
char *fname;
{
	FILE *fp;
	int errs, rc;

	fp = read_ppm_fname(fname);

	if (fp != NULL) {
		
		fprintf(stdout, "\n");
		fprintf(stdout, " Enter a referencing offset for the shifts and the\n");
		fprintf(stdout, " atoms affected (either H, N, C, *) (eg, -1.3 N)\n");
		fprintf(stdout, " Enter '0.0 *' for no reference changes.\n");
		fprintf(stdout, PROG_PROMPT);
		fscanf(stdin, "%lf %s", &seqArr[ix].refVal, seqArr[ix].refAtom);
		fprintf(stdout, "\n");
		fprintf(stdout, " Enter a multiplier which indicates the relative\n");
		fprintf(stdout, " importance of these shifts to other shift files.\n");
		fprintf(stdout, " You should enter '1.0' unless you think that orb\n");
		fprintf(stdout, " will undervalue/overvalue particular shift data.\n");
		fprintf(stdout, PROG_PROMPT);
		fscanf(stdin, "%lf", &seqArr[ix].userShiftBias);
		fprintf(stdout, "\n");
		fprintf(stdout, "Reading and Analyzing shifts for %s\n", fname);

		errs = read_ppm_data(fp, &seqArr[ix]);
		if (errs > 0) {
			fprintf(stdout, "Number of errors: %d\n", errs);
		}
	}

	else
		return(FALSE);

	return(TRUE);

}	/* proc_ppm_fname */


/*************************************************************/
/* read chemical shift file (ppm format) 
*/
static FILE *read_ppm_fname(fname)
char *fname;
{
	FILE *fp;
	char sVal[MAX_LINE_LEN];

	fprintf(stdout, "\n Enter your shifts file.\n");
	fprintf(stdout, PROG_PROMPT);
	fscanf(stdin, "%s", fname);
	fp = fopen(fname, "r");

	if (fp == NULL) {
		sprintf(sVal, "Error, Illegal filename: '%s'\n", fname);
		show_err(FALSE, "read_ppm_fname", sVal);
	}

	return(fp);

}	/* read_ppm_fname */


/*************************************************************/
/* init shift data structure for a sequence
*/
static void init_ppm_shift(seqPtr)
ALIGN_SEQ_REC *seqPtr;
{
	register int i, j;

	for (i=0; i<alignLen; i++) {
		for (j=0; j<MAX_RES_ATOM_NUM; j++) {
			seqPtr->residue[i].shift1[j] = NO_SHIFT_VAL;
			seqPtr->residue[i].shift2[j] = NO_SHIFT_VAL;
		}
	}

}   /* init_ppm_shift */


/*************************************************************/
/* process one ppm file 
*/
static int read_ppm_data(fp, seqPtr)
FILE *fp;
ALIGN_SEQ_REC *seqPtr;
{
	int errCount, rc;
	char sVal[MAX_LINE_LEN], line[MAX_LINE_LEN];

	errCount = 0;

	while (TRUE) {

		if (NULL == fgets(line, MAX_LINE_LEN, fp))
			break;

		if (COMMENT_LINE(line[0]) == FALSE) {
			if ((rc = sscanf(line, "%s", sVal)) > 0) {
				rc = scan_shift(line, seqPtr);
				if (rc == ERROR_FLAG)
					errCount++;
			}
		}
	}

	return(errCount);

}	/* read_ppm_data */

/*************************************************************/
/* read ppm file shift values
*/
static int scan_shift(line, seqPtr)
char *line;
ALIGN_SEQ_REC *seqPtr;
{
	int i, molNum, idNum, nd, rc, ipch;
	char mch, pch, wAtom[MAX_LINE_LEN];
	char sVal[MAX_LINE_LEN], res3[MAX_LINE_LEN], atom[MAX_LINE_LEN];
	char shift1[MAX_LINE_LEN], shift2[MAX_LINE_LEN];

/* read arguments */

	nd = sscanf(line, "%d:%3s_%d:%s", &molNum, res3, &idNum, atom);

/* must read at least 4 arguments */

	if (nd < 4) {
		sprintf(sVal, "Error, unrecognized PPM format: %s", line);
		fprintf(stdout, "%s", sVal);
		return(ERROR_FLAG);
	}

/* check for legal amino acid */

	upper_case(res3);
	ipch = pres_3to1(res3);
	if (ipch == ERROR_FLAG) {
		sprintf(sVal, "Error, unrecognized amino acid: %s", line);
		fprintf(stdout, "%s", sVal);
		return(ERROR_FLAG);
	}

/* check for residue id number correspondence */

	pch = AA_CH(ipch);
	for (i=0; i<alignLen; i++) {
		if (seqPtr->residue[i].idNum == idNum) {
			mch = seqPtr->residue[i].pRes;
			if (mch != pch) {
				sprintf(sVal, "Error, check Id number with alignment file:");
				fprintf(stdout, "%s %s", sVal, line);
				return(ERROR_FLAG);
			}
			break;
		}
	}

	if (i == alignLen) {
		sprintf(sVal, "Error, check Id number with alignment file:");
		fprintf(stdout, "%s %s", sVal, line);
		return(ERROR_FLAG);
	}

/* check the atom name */

	upper_case(atom);
	rc = check_rshift_atom(ipch, atom, wAtom);
	if (rc == ERROR_FLAG) {
		sprintf(sVal, "Error, unrecognized atom: %s", line);
		fprintf(stdout, "%s", sVal);
		return(ERROR_FLAG);
	}

/* check for valid shift fields */

	rc = proc_shift(line, seqPtr, wAtom, ipch, i, shift1, shift2);
	return(rc);

}	/* scan_shift */


/*************************************************************/
/* read and store ppm file shift values
*/
static int proc_shift(line, seqPtr, wAtom, ipch, ix, shift1, shift2)
int ix, ipch;
ALIGN_SEQ_REC *seqPtr;
char *line, *wAtom, *shift1, *shift2;
{
	char comment[MAX_LINE_LEN];
	double s1, s2;
	int aix, rc;

	strcpy(shift1, noShiftStr);
	strcpy(shift2, noShiftStr);
	strcpy(comment, "");
	rc = check_rshift_shifts(line, shift1, shift2, comment);

	if (rc == 0) {

		aix = atom_to_index(ipch, wAtom);

		s1 = atof(shift1);
		s2 = atof(shift2);

		if (s2 < s1) {
			seqPtr->residue[ix].shift1[aix] = s2;
			seqPtr->residue[ix].shift2[aix] = s1;
		} else {
			seqPtr->residue[ix].shift1[aix] = s1;
			seqPtr->residue[ix].shift2[aix] = s2;
		}
	}

	return(rc);

}	/* proc_shift */
