
/*
	file: parms.c
	purpose: reading fields in the parameter file. 
*/

#include "defs.h" 

/* global vars */
char	wtHomolFname[MAX_FNAME_LEN];
char	wtStructFname[MAX_FNAME_LEN];
int		wtHomolMat[WT_MAT_SIZE];
int		wtStructMat[WT_MAT_SIZE];

/* procedure calls */
static void read_wt_mat_file();
extern void show_err();


/*************************************************************/
/* read primary sequence homology scoring matrix 
*/
void read_wt_homol(parmsfp)
FILE *parmsfp;
{
	FILE *fp;
	char sVal[MAX_LINE_LEN];

	if (read_prompt(parmsfp, PARMS_PROMPT) == FALSE) {
		sprintf(sVal, "Cannot find sequence homology file\n");
		show_err(TRUE, "read_wt_homol", sVal);
	}

	fscanf(parmsfp, "%s\n", wtHomolFname);
	fp = fopen(wtHomolFname, "r");
	if (fp == NULL) {
		sprintf(sVal, "Cannot open homology matrix: %s\n", wtHomolFname);
		show_err(TRUE, "read_wt_homol", sVal);
	}

	fprintf(stdout, "Reading primary sequence similarity file: %s\n",
		wtHomolFname);
	read_wt_mat_file(fp, wtHomolMat);
	fclose(fp);

}	/* read_wt_homol */


/*************************************************************/
/* read structure homology scoring matrix 
*/
void read_wt_struct(parmsfp)
FILE *parmsfp;
{
	FILE *fp;
	char sVal[MAX_LINE_LEN];

	if (read_prompt(parmsfp, PARMS_PROMPT) == FALSE) {
		sprintf(sVal, "Cannot find structure homology file\n");
		show_err(TRUE, "read_wt_struct", sVal);
	}

	fscanf(parmsfp, "%s\n", wtStructFname);
	fp = fopen(wtStructFname, "r");
	if (fp == NULL) {
		sprintf(sVal, "Cannot open structure homology: %s\n", wtStructFname);
		show_err(TRUE, "read_wt_struct", sVal);
	}

	fprintf(stdout, "Reading secondary structure similarity file: %s\n",
		wtStructFname);
	read_wt_mat_file(fp, wtStructMat);
	fclose(fp);

}	/* read_wt_struct */


/*************************************************************/
/* Reads generic similarity matrix.
*/
static void read_wt_mat_file(fp, data)
FILE *fp;
int	*data;
{
	register int j, count, ch, ix;
	int nd, score, ixWt[WT_MAT_DIM];
	char *rc, *s1, line[MAX_LINE_LEN], sVal[MAX_LINE_LEN];
	char aStr[MAX_LINE_LEN];

	count = 0;
	for (j=0; j<WT_MAT_SIZE; j++)
		data[j] = WT_MAT_INIT;

	rc = fgets(line, MAX_LINE_LEN-1, fp);
	while (rc != NULL) {
		strcpy(aStr, line);
		if (FALSE == COMMENT_LINE(line[0])) {
			s1 = strtok(line, " ");
			if (strlen(s1) != 1) {
				sprintf(sVal, "Illegal label length: %s\n", line);
				show_err(TRUE, "read_wt_mat_file", sVal);
			}
			ch = AA_UC(s1[0]);
			if (FALSE == AA_CVALID(ch)) {
				sprintf(sVal, "Illegal label character: %s\n", aStr);
				show_err(TRUE, "read_wt_mat_file", sVal);
			}
			ix = AA_IX(ch);
			if ((ix*ix) > WT_MAT_SIZE) {
				sprintf(sVal, "Space restrictions exceeded: %s\n", aStr);
				show_err(TRUE, "read_wt_mat_file", sVal);
			}
			ixWt[count] = ix;

			for (j=0; j<=count; j++) {
				s1 = strtok((char *)NULL, " ");
				if (s1 == NULL) {
					sprintf(sVal, "Trouble reading matrix at %s\n", aStr);
					show_err(TRUE, "read_wt_mat_file", sVal);
				}
				sscanf(s1, "%d", &score);
				if (score < MIN_WT_MAT_VAL) {
					sprintf(sVal, "Min score violated at %s\n", aStr);
					show_err(TRUE, "read_wt_mat_file", sVal);
				}
				if (score > MAX_WT_MAT_VAL) {
					sprintf(sVal, "Max score violated at %s\n", aStr);
					show_err(TRUE, "read_wt_mat_file", sVal);
				}
				data[(ixWt[count] * WT_MAT_DIM) + ixWt[j]] = score;
				data[(ixWt[j] * WT_MAT_DIM) + ixWt[count]] = score;
			}
			count++;
			s1 = strtok((char *)NULL, " ");
			if (s1 != NULL) {
				nd = sscanf(s1, "%d", &score);
				if (nd > 0) {
					sprintf(sVal, "Input check, line too long at %s\n", aStr);
					show_err(TRUE, "read_wt_mat_file", sVal);
				}
			}
		}
		rc = fgets(line, MAX_LINE_LEN-1, fp);
	}

}	/* read_wt_mat_file */
