
/*
	file: ushift.c
	ppm shift file check utilities
*/

#include "defs.h"

/*************************************************************/
/* check if legal atom name 
*/
int check_rshift_atom(rix, myAtom, wName)
int rix;
char *myAtom, *wName;
{
	register int i, j, num, rLen, mLen;
	char *rsName;

	num = aaTable[rix].atomNum;
	mLen = strlen(myAtom);
	for (i=0; i<num; i++) {
		rsName = aaTable[rix].atom[i].rName;
		rLen = strlen(rsName);
		if (rLen == mLen) {
			for (j=0; j<rLen; j++) {
				if (myAtom[j] != rsName[j])
					if ((myAtom[j] != '*') || (rsName[j] != '#'))
						break;
			}
			if (j == rLen) {
				strcpy(wName, rsName);
				return(0);
			}
		}
	}

	return(ERROR_FLAG);

}	/* check_rshift_atom */


/*************************************************************/
/* check for valid shift fields 
*/
int check_rshift_shifts(line, shift1, shift2, comment)
char *line, *shift1, *shift2, *comment;
{
	int nd, rc;
	char str1[MAX_LINE_LEN], str2[MAX_LINE_LEN], com1[MAX_LINE_LEN];
	char sVal[MAX_LINE_LEN];
	double aShift, bShift;

	nd = sscanf(line, "%*s %s %s %s", str1, str2, com1);

	if (nd < 1) {
		sprintf(sVal, "Warning, no shift value found for: %s", line);
		fprintf(stdout, "%s", sVal);
		return(WARN_FLAG);
	}

/* check shift 1 value */ 

	rc = sscanf(str1, "%lf%s", &aShift, sVal);
	if (rc == 0) {
		if (NULL != strchr(str1, '*')) {
			sprintf(sVal, "Warning, shift1 value assumed unknown at: %s", line);
			fprintf(stdout, "%s", sVal);
			strcpy(comment, "");
			if (nd > 5) {
				strcpy(comment, strstr(line, str2));
			}
			return(WARN_FLAG);
		}
		if (COMMENT_LINE(str1[0]) == TRUE) {
			sprintf(sVal, "Warning, shift1 value unknown: %s", line);
			fprintf(stdout, "%s", sVal);
			strcpy(comment, strstr(line, str1));
			return(WARN_FLAG);
		}
		sprintf(sVal, "Error, shift1 value not readable at: %s", line);
		fprintf(stdout, "%s", sVal);
		return(ERROR_FLAG);
	}

/* shift 1 not valid */

	if (rc >= 2) {
		sprintf(sVal, "Error, shift1 value not readable at: %s", line);
		fprintf(stdout, "%s", sVal);
		return(ERROR_FLAG);
	}

/* shift 1 value is ok */

	strcpy(shift1, str1);
	if (nd == 1) {
		return(0);
	}

/* check shift 2 value */

	rc = sscanf(str2, "%lf%s", &bShift, sVal);
	if (rc == 0) {
		if (NULL != strchr(str1, '*')) {
			sprintf(sVal, "Warning, shift2 value assumed unknown at: %s", line);
			fprintf(stdout, "%s", sVal);
			strcpy(comment, "");
			if (nd > 2) {
				strcpy(comment, strstr(line, com1));
			}
			return(WARN_FLAG);
		}
		if (COMMENT_LINE(str2[0]) == TRUE) {
			strcpy(comment, strstr(line, str2));
			return(0);
		}
		sprintf(sVal, "Error, cannot recognize characters following shift1: %s",
			line);
		fprintf(stdout, "%s", sVal);
		return(ERROR_FLAG);
	}

/* invalid shift 2 */

	if (rc >= 2) {
		sprintf(sVal, "Error, shift2 value not readable at: %s", line);
		fprintf(stdout, "%s", sVal);
		return(ERROR_FLAG);
	}

/* shift 2 is ok */
	
	strcpy(shift2, str2);
	if (nd == 2) {
		return(0);
	}

/* check for trailing comment fields */

	if (COMMENT_LINE(com1[0]) == TRUE) {
		strcpy(comment, strstr(line, com1));
		return(0);
	}
	sprintf(sVal, "Error, cannot recognize characters following shift2: %s",
		line);
	fprintf(stdout, "%s", sVal);
	return(ERROR_FLAG);

}	/* check_rshift_shifts */
