
/*
	file: std.h
	purpose: general definitions for SEQSEE-like programs.
*/

#include <stdio.h>
#include <string.h>


/********************/
/* Definitions      */
/********************/


/* self explanatory definitions */
#define TRUE				1
#define FALSE				0
#define MIN_INT				(1 << 32 - 1)
#define MAX_INT				(~MIN_INT)
#define MAX_LINE_LEN		256
#define MAX_FNAME_LEN		256

/* prompt string for input queries */
#define PROG_PROMPT			"\n>> "
#define PARMS_PROMPT		">>"

/* Max lengths for sequence title and id codes */
#define MAX_TITLE_LEN		256
#define MAX_ID_LEN			20	

/* first and last character codes */
#define FIRST_CODE			'A'
#define LAST_CODE			'Z'

/* range for amino acid one letter code, eg 'Z' - 'A' */
#define CODES				(LAST_CODE - FIRST_CODE + 1)


/********************/
/* Macros           */
/********************/


/* return max or min of two numbers */
#define MAX(x,y)			(x>y? x: y)
#define MIN(x,y)			(x<y? x: y)

/* convert chars from upper to lower case and vice versa */
#define AA_UC(c)			(((c>='a') && (c<='z'))? c+'A'-'a': c)
#define AA_LC(c)			(((c>='A') && (c<='Z'))? c-'A'+'a': c)

/* convert from character code to array index */
#define AA_IX(c)			(c - FIRST_CODE)

/* convert from array index to character code */
#define AA_CH(x)			(x + FIRST_CODE)

/* determine if character code is valid */
#define AA_CVALID(c) (((c >= FIRST_CODE) && (c <= LAST_CODE))? TRUE: FALSE)

/* determine if index entry for character code is valid */
#define AA_IVALID(x) (((x >= 0) && (x < CODES))? TRUE: FALSE)

/* determine if amino acid code is one of the 20 amino acids known */
#define AA_CCODE(c)			((AA_CVALID(c)) && \
							((c!='B') && (c!='J') && (c!='O') && (c!='U') \
							&& (c!='X') && (c!='Z'))? TRUE: FALSE)
