#
# This is a TCL script for the front end for the Orb program.  
#
# usage: wish orb.tcl
#
################################################################################

########################################################################
#
# main orb startup script 
#
proc orb_main { installdir workdir } {
	global orbVar 

    # check our arguments
	if { [ catch { set orbVar(INSTALL_DIR) $installdir } err] } {
		error "Usage: orb_main installdir workdir"
	}
	if { [ catch { set orbVar(WORK_DIR) $workdir } err] } {
		error "Usage: orb_main installdir workdir"
	}
	if ![ file isdirectory $orbVar(INSTALL_DIR) ] {
		error "Cannot find orb installation directory '$orbVar(INSTALL_DIR)'."
	}
	if ![ file isdirectory $orbVar(WORK_DIR) ] {
		error "Cannot find workspace directory '$orbVar(WORK_DIR)'."
	}

	# setup our environment
	init_orb
	read_orb_defaults $orbVar(file,DEFAULTS) $orbVar(SYS_DEFAULTS)
	set_orb_parms 0

	# define main window
	set f $orbVar(MAIN_WINDOW)
	toplevel $f
	wm geometry $f $orbVar(geometry,main) 
	wm title $f "orb - $orbVar(INSTALL_DIR)"
	set f $orbVar(MAIN_WINDOW).main
	frame $f
	pack $f
	panel_orb_menubar $f

	# draw contents of startup window
	panel_orb_startup $f.win
	tkwait window $f
}

########################################################################
#
# init variables
#
proc init_orb {} {
	global orbVar 

	set orbVar(MAIN_WINDOW)		".orb"
	set orbVar(WEB_HOME)		"http://www.pence.ca/software/orb"

	# location of binary files called by this tcl script
	set orbVar(BIN_DIR)  		"$orbVar(INSTALL_DIR)/bin"
	set orbVar(ORB_PROG) 		"$orbVar(BIN_DIR)/orb_x"
	set orbVar(STAR_PROG) 		"$orbVar(BIN_DIR)/star2ppm"
	set orbVar(XALIGN_PROG) 	"$orbVar(BIN_DIR)/xalign_x"

	# location of lib files needed by orb
	set orbVar(LIB_DIR)   		"$orbVar(INSTALL_DIR)/lib/orb"
	set orbVar(SYS_DEFAULTS)  	"$orbVar(LIB_DIR)/orb.defaults"
	set orbVar(SYS_PARMS)		"$orbVar(LIB_DIR)/orb.parms"
	set orbVar(STAR_TABLE)		"$orbVar(LIB_DIR)/star2ppm.table"

	# other constants
	set orbVar(ALIGN_KEY)		"Num"
	set orbVar(OFFSET_ATOMS)	"all ha hn hb h* n ca cb c*"
	set orbVar(PARMS_KEY)		">>"

	set orbVar(DOC,AUTHORS)		"$orbVar(LIB_DIR)/doc.authors"
	set orbVar(DOC,PPM)			"$orbVar(LIB_DIR)/doc.ppm"
	set orbVar(DOC,XALIGN)		"$orbVar(LIB_DIR)/doc.xalign"
	set orbVar(DOC,SEQFILE)		"$orbVar(LIB_DIR)/doc.seqfile"

	set orbVar(file,atomNaming)	"$orbVar(STAR_TABLE)"
	set orbVar(starFilePattern) "str"

	set orbVar(geometry,main)	"750x500+50+50"
	set orbVar(geometry,view)	"+100+100"
	set orbVar(geometry,result)	"+150+150"
	set orbVar(geometry,opt1)	"+200+200"
	set orbVar(geometry,weight)	"+250+250"

	# Input Files created by orb
	set orbVar(file,DEFAULTS) 	"$orbVar(WORK_DIR)/orb.defaults"
	set orbVar(file,PARMS)		"$orbVar(WORK_DIR)/orb.parms"
	set orbVar(file,RUN)		"$orbVar(WORK_DIR)/orb.run"

	set orbVar(out,ppm) 		"$orbVar(WORK_DIR)/orb.shifts.ppm"
	set orbVar(out,eppm) 		"$orbVar(WORK_DIR)/orb.shifts.eppm"
	set orbVar(out,csi) 		"$orbVar(WORK_DIR)/orb.shifts.csi"
	set orbVar(out,verb) 		"$orbVar(WORK_DIR)/orb.log.predict"
	set orbVar(out,log) 		"$orbVar(WORK_DIR)/orb.log.run"

	set orbVar(dir,shifts)		""
	set orbVar(file,sequence) 	""
	set orbVar(file,align) 		"$orbVar(WORK_DIR)/xalign.out"
}

########################################################################
#
# main orb shutdown routine 
#
proc exit_orb {} {
	global orbVar 
	destroy $orbVar(MAIN_WINDOW)
}

########################################################################
#
# create menubar
#
proc panel_orb_menubar { f } {
	set mb $f.mbar
	frame $mb -relief raised -bd 2
	frame $f.dummy
	pack $mb $f -side top -fill x
	panel_orb_menus $mb
}

#
# create menu items
#
proc panel_orb_menus {f} {
	global orbVar

	menubutton $f.file -padx 10 -text File -underline 0 -menu $f.file.menu
	menubutton $f.help -text Help -underline 0 -menu $f.help.menu
	
	menu $f.file.menu
	$f.file.menu add command -label "About this software..." \
		-command "o_panel_file_view orbVar(DOC,AUTHORS)"
	$f.file.menu add separator
	$f.file.menu add command -label "Exit" -command "exit_orb" 
	
	# Help menu item
	menu $f.help.menu
	$f.help.menu add command -label "Help at www.pence.ca/software/orb"
	
	pack $f.file -side left
	pack $f.help -side right
	tk_menuBar $f $f.file $f.help
	focus $f
}

########################################################################
#
# orb startup window routine 
#
proc panel_orb_startup { f } {
	global orbVar 

	pack [frame $f]
	set orbVar(aMode) 1 

	label $f.title -text "Welcome to orb!" 
	pack $f.title
	message $f.intro -text \
		"This software allows you to predict the\
		chemical shift values for a given sequence. First indicate how\
		you expect the prediction to be made." 
	pack $f.intro

	frame $f.select
	pack $f.select -padx 30 -pady 20
	radiobutton $f.select.seq \
		-text "Use sequence (and secondary structure) information only" \
		-variable orbVar(aMode) -value 1
	radiobutton $f.select.hom \
		-text "I can provide homologous sequence chemical shift data files"\
		-variable orbVar(aMode) -value 2
	pack $f.select.seq $f.select.hom -side top -anchor w -pady 2
	$f.select.hom invoke

	# buttons section
	frame $f.buttons
	pack $f.buttons -pady 20
	button $f.buttons.cont -text "Continue" -command "do_seq_hom $f"
    button $f.buttons.quit -text Exit -command "exit_orb"
    pack $f.buttons.cont $f.buttons.quit -side left
}

#
# decide which window to display next
#
proc do_seq_hom { f } {
	global orbVar
	destroy $f
	if {$orbVar(aMode) == 1} {
		panel_orb_seq $f
	} else  {
		panel_orb_hom $f
	}
}

########################################################################
#
# window where orb is run with sequence only 
#
proc panel_orb_seq { f } {
	global orbVar

	pack [frame $f]
	label $f.title -text "Orb Sequence Input" 
	pack $f.title

	message $f.intro -text \
		"Orb is supporting the nmrview and PIR sequence file\
		formats. Click on the HELP button for more information on these\
		formats. Otherwise, enter your sequence file via the keyboard or use\
		the Browse button to navigate your file system."
	pack $f.intro

	o_panel_entry $f.seqs       "Sequence File:" \
		14 orbVar(file,sequence) 48 { } \
		-text Browse -command "o_panel_file_browse orbVar(file,sequence)"
	pack $f.seqs -pady 30

	# buttons section
	frame $f.buttons
	pack $f.buttons -pady 20
	button $f.buttons.help -text "Help " \
		-command "o_panel_file_view orbVar(DOC,SEQFILE)"
	button $f.buttons.cont -text "Continue" \
		-command "destroy $f; panel_orb_seq_output $f"
    button $f.buttons.quit -text Exit -command "exit_orb"
    pack $f.buttons.help $f.buttons.cont $f.buttons.quit -side left
}

########################################################################
#
# get output files and execute orb with sequence only 
#
proc panel_orb_seq_output { f } {
	global orbVar

	# default output file names
	set orbVar(out,verb) "/tmp/orb.verb.tmp"

	pack [frame $f]
	label $f.title -text "Orb Output Files" 
	pack $f.title

	message $f.intro -text \
		"Orb will save its predictions in three different file formats.\
		You will have the opportunity to view each of these files as well\
		as the log file after hitting the PREDICT button."
	pack $f.intro

	o_panel_entry $f.ppm "Predicted Shifts (ppm):" \
		24 orbVar(out,ppm) 48 { }
	pack $f.ppm

	o_panel_entry $f.eppm "Predicted Shifts (eppm):" \
		24 orbVar(out,eppm) 48 { }
	pack $f.eppm

	o_panel_entry $f.csi "Predicted Shifts (csi):" \
		24 orbVar(out,csi) 48 { }
	pack $f.csi

	o_panel_entry $f.log "Log File:" \
		24 orbVar(out,log) 48 { }
	pack $f.log

	message $f.exemsg -width 500 -text "" 
	pack $f.exemsg -pady 15

	# buttons section
	frame $f.buttons
	pack $f.buttons -pady 20
	button $f.buttons.cont -text "Predict" -command "execute_orb_seq"
    button $f.buttons.quit -text Exit -command "exit_orb"
    pack $f.buttons.cont $f.buttons.quit -side left
}

#
# user clicks execute button for orb single sequence
#
proc execute_orb_seq { } {
	global orbVar 

	if [catch {open $orbVar(file,RUN) w} fp] {
		error "Cannot open run file: $orbVar(file,RUN)"
	}
	puts $fp "1"
	puts $fp $orbVar(file,sequence)
	puts $fp $orbVar(out,ppm)
	puts $fp $orbVar(out,eppm)
	puts $fp $orbVar(out,verb)
	puts $fp $orbVar(out,csi)
	close $fp

	set output "$orbVar(out,ppm) $orbVar(out,eppm) $orbVar(out,csi) $orbVar(out,log)"
	execute_orb_x $output
}

########################################################################
########################################################################

########################################################################
#
# window which starts process for using orb with homologous shifts
#
proc panel_orb_hom { f } {
	global orbVar

	pack [frame $f]
	label $f.title -text "Organizing Homologous Chemical Shift Data" 
	pack $f.title
	set orbVar(bMode) 1

	message $f.intro -text \
		"The first stipulation by orb is that all the possibly useful\
		homologous\
		chemical shift data files are located in a single directory.\
		The second criteria is that these shift files are formatted\
		in ppm format with the '.ppm' or '.PPM' filename extension.\n\nYou\
		can use our utility program to convert\
		a directory of shift files in star format to corresponding\
		ppm format files."
	pack $f.intro 

	frame $f.select
	pack $f.select -padx 30 -pady 20
	radiobutton $f.select.cont \
		-text "These conditions are already met" \
		-variable orbVar(bMode) -value 1
	radiobutton $f.select.convert \
		-text "Convert my shift files to ppm format" \
		-variable orbVar(bMode) -value 2 
	radiobutton $f.select.spec \
		-text "Tell me about the specifications for ppm file format" \
		-variable orbVar(bMode) -value 3 \
		-command "$f.select.cont invoke; o_panel_file_view orbVar(DOC,PPM)"
	pack $f.select.cont $f.select.convert $f.select.spec \
		-side top -anchor w -pady 2
	$f.select.cont invoke

	# buttons section
	frame $f.buttons
	pack $f.buttons -pady 20 
	button $f.buttons.cont -text "Continue" -command "do_hom_next $f"
    button $f.buttons.quit -text Exit -command "exit_orb"
    pack $f.buttons.cont $f.buttons.quit -side left
}

#
# decide which window to display next
#
proc do_hom_next { f } {
	global orbVar
	destroy $f
	if {$orbVar(bMode) == 1} {
		panel_orb_hom_align $f
	}
	if {$orbVar(bMode) == 2} {
		panel_orb_star2ppm $f	
	}
}

########################################################################
#
# window to run star2ppm software 
#
proc panel_orb_star2ppm { f } {
	global orbVar

	pack [frame $f] 
	label $f.title -text "Convert shift data from star to ppm format" 
	pack $f.title

	message $f.intro -text \
		"The default for this software is to find all *.str files in\
		the specified directory\
		and create corresponding *.ppm files.\ Please take a moment\
		to ensure that your star files follow this file naming convention\
		or enter a different File search pattern below.\
		Since atom naming conventions are\
		non-standard, you may wish to inspect our Translation file."
	pack $f.intro

	o_panel_entry $f.input     "Input directory of star files:" \
		32 orbVar(dir,shifts) 40 { } \
		-text Browse -command "o_panel_file_browse orbVar(dir,shifts)"
	pack $f.input 

	o_panel_entry $f.shiftsdir "File search pattern:" \
		32 orbVar(starFilePattern) 8 { }
	pack $f.shiftsdir 

	o_panel_entry $f.translate "Translation file for atom names:" \
		32 orbVar(file,atomNaming) 40 { }\
		-text "View" -command "o_panel_file_view orbVar(file,atomNaming)"
	pack $f.translate 

	message $f.exemsg -width 500 -text "" 
	pack $f.exemsg -pady 15

	# buttons section
	frame $f.buttons
	pack $f.buttons -pady 20
	button $f.buttons.exe -text "Predict" -command "do_star2ppm $f"
	button $f.buttons.cont -text "Continue" -state disabled \
		-command "destroy $f; panel_orb_hom_align $f"
    button $f.buttons.quit -text Exit -command "exit_orb"
    pack $f.buttons.exe $f.buttons.cont $f.buttons.quit -side left
}

#
# run star2ppm software 
#
proc do_star2ppm { f } {
	global orbVar

	if ![file isdirectory $orbVar(dir,shifts) ] {
		error "Cannot access output directory: '$orbVar(dir,shifts)'"
	}
	if ![file readable $orbVar(file,atomNaming) ] {
		error "Cannot read translation file: '$orbVar(file,atomNaming)'"
	}
	set orbVar(star,log) "$orbVar(dir,shifts)/star2ppm.log" 
	set unixcmd "rm -f $orbVar(star,log)"
	if [ catch { eval exec $unixcmd } err] {
		error "Problems with command: $unixcmd \n"
	}

	set input [glob -nocomplain $orbVar(dir,shifts)/*.$orbVar(starFilePattern)]
	set rc 0
	set outFiles "$orbVar(star,log)"
	foreach fn $input {
		set root [file rootname $fn]
		set output $root.ppm

		# execute star2ppm c program
		set unixcmd "$orbVar(STAR_PROG) $fn $orbVar(file,atomNaming) $output"
		if [catch {eval exec $unixcmd >> $orbVar(star,log)} err] {
			set rc 1
		} else {
			set outFiles "$outFiles $output" 
		}
	}

	if { $rc == 0 } {
		set count [llength $input]
		$f.exemsg configure -text "$count files successfully converted."
	} else {
		$f.exemsg configure -text "Software had problems, please view log file"
	}
	$f.buttons.cont configure -state normal
	o_panel_show_results $outFiles
}

########################################################################
#
# window to get alignment file input
#
proc panel_orb_hom_align { f } {
	global orbVar

	pack [frame $f] 
	label $f.title -text "Correlating Homologous Shift Data with your Target Sequence" 
	pack $f.title
	set orbVar(cMode) 1

	message $f.intro -text \
		"Next, you need to create an alignment file which tells orb the\
		corresponding amino acids within a family of sequences. Also,\
		each sequence name in the family must be chosen to exactly\
		match the corresponding filename in the homologous\
		shifts directory. Don't forget to include the sequence you\
		want to predict in your alignment file.\n\nIt is recommended\
		you use the xalign software to generate the alignment file."
	pack $f.intro

	frame $f.select
	pack $f.select -padx 30 -pady 20
	radiobutton $f.select.cont \
		-text "I already have my alignment file" \
		-variable orbVar(cMode) -value 1
	radiobutton $f.select.xalign \
		-text "Create alignment file using xalign" \
		-variable orbVar(cMode) -value 2 
	radiobutton $f.select.spec \
		-text "Tell me about the specifications for the alignment file format" \
		-variable orbVar(cMode) -value 3 \
		-command "o_panel_file_view orbVar(DOC,XALIGN)"
	pack $f.select.cont $f.select.xalign $f.select.spec \
		-side top -anchor w -pady 2
	$f.select.cont invoke

	# buttons section
	frame $f.buttons
	pack $f.buttons -pady 20 
	button $f.buttons.cont -text "Continue" -command "do_hom_align $f"
    button $f.buttons.quit -text Exit -command "exit_orb"
    pack $f.buttons.cont $f.buttons.quit -side left
}

#
# either user has alignment file or needs to create it
#
proc do_hom_align { f } {
	global orbVar
	if {$orbVar(cMode) == 1} {
		destroy $f
		panel_orb_hom_inputs $f
	}
	if {$orbVar(cMode) == 2} {
		set orbVar(cMode) 1
		xalign_main $orbVar(INSTALL_DIR) $orbVar(WORK_DIR)
	}
}

########################################################################
#
# window to get data files to run orb with homologous shifts
#
proc panel_orb_hom_inputs { f } {
	global orbVar

	pack [frame $f]
	label $f.title -text "Orb Input Data" 
	pack $f.title

	message $f.intro -text \
		"You should now be ready to type the pathname for each\
		required input or you may prefer\
		to use the BROWSE button to navigate your file system.\n\nUse\
		the VERIFY DATA button to make sure orb is able to\
		correlate the sequences in the alignment file to shift files\
		in the shift files directory." 
	pack $f.intro

	o_panel_entry $f.seqs       "Alignment File:" \
		17 orbVar(file,align) 44 { } \
		-text Browse -command "o_panel_file_browse orbVar(file,align)"
	pack $f.seqs

	o_panel_entry $f.shiftsdir  "Shifts directory:" \
		17 orbVar(dir,shifts) 44 { }\
		-text Browse -command "o_panel_file_browse orbVar(dir,shifts)"
	pack $f.shiftsdir

	message $f.vermsg -width 500 -text "" 
	pack $f.vermsg -pady 15

	# buttons section
	frame $f.buttons
	pack $f.buttons -pady 20
	button $f.buttons.verify -text "Verify data" -command "check_hom_inputs $f"
	button $f.buttons.cont -text "Continue" -state disabled \
		-command "destroy $f; panel_orb_hom_output $f"
    button $f.buttons.quit -text Exit -command "exit_orb"
    pack $f.buttons.verify $f.buttons.cont $f.buttons.quit -side left
}

#
# User pressed the 'Verify data' button 
#
proc check_hom_inputs { f } {
	global orbVar seqDat

	# a delay makes it clear that an update occurred
	$f.vermsg configure -text ""
	update
	after 200

	if ![file readable $orbVar(file,align) ] {
		error "Cannot read alignment input file: '$orbVar(file,align)'"
	}
	if ![file isdirectory $orbVar(dir,shifts) ] {
		error "Cannot access shifts directory: '$orbVar(dir,shifts)'"
	}
	read_orb_align $orbVar(file,align)
	check_seq_data $orbVar(dir,shifts)

	set files ""
	set count 0
	for {set i 0} {$i < $seqDat(seqNum)} {incr i} {
		if [info exists seqDat($i,bias)] { 
			set files "$files $seqDat($i,key)"
			incr count	
		}
	}
	$f.vermsg configure \
		-text "$count homologous shift files found: $files"
	$f.buttons.cont configure -state normal
}

#
# correlate alignment sequences with shift data files
#
proc check_seq_data { dir } {
	global orbVar seqDat

	for {set i 0} {$i < $seqDat(seqNum)} {incr i} {
		set key $seqDat($i,key)
		set fsearch $dir/$key

		set fname [ glob -nocomplain ${fsearch}.ppm ]
		if { [llength $fname] == 0 } {
			set fname [ glob -nocomplain ${fsearch}.PPM ]
		}
		if { [llength $fname] > 0 } {
			if [file readable $fname] {
				set seqDat($i,bias) 1.0
				set seqDat($i,fn) $fname	
				set seqDat($i,offsetVal)	0.0
				set seqDat($i,offsetAtom) [lindex $orbVar(OFFSET_ATOMS) 0]
			}
		}
	}
}

########################################################################
#
# allow user to configure orb settings. 
#
proc panel_orb_hom_output { f } {
	global orbVar seqDat

	pack [frame $f]
	label $f.title -text "Sequence to Predict and Output Files"
	pack $f.title

	message $f.intro -text \
		"Click on the 'Sequence to Predict' menu button below and select the\
		sequence for orb to predict.\
		Orb will save its predictions in three different file formats.\
		The Verbose output will allow you to see exactly how orb calculates\
		each shift prediction. The Log file is useful for debugging\
		errors.\n\nPress PREDICT to generate the predictions, OPTIONS allows\
		you to change the default parameters which the software uses."
	pack $f.intro

	frame $f.select
	pack $f.select -padx 30 -pady 20
	label $f.select.label -text "Sequence to Predict:"
	pack $f.select.label -side left
	menubutton $f.select.seq -text $seqDat($seqDat(current),key) \
		-menu $f.select.seq.menu -underline 0
	pack $f.select.seq -side left
	menu $f.select.seq.menu
	for {set np 0} {$np < $seqDat(seqNum)} {incr np} {
		$f.select.seq.menu add command -label $seqDat($np,key) \
			-command "bind_orb_seq_select $f.select.seq $seqDat($np,key) $np" 
	}

	o_panel_entry $f.ppm "Predicted Shifts (ppm):" \
		24 orbVar(out,ppm) 48 { }
	pack $f.ppm

	o_panel_entry $f.eppm "Predicted Shifts (eppm):" \
		24 orbVar(out,eppm) 48 { }
	pack $f.eppm

	o_panel_entry $f.csi "Predicted Shifts (csi):" \
		24 orbVar(out,csi) 48 { }
	pack $f.csi

	o_panel_entry $f.verb "Verbose Output:" \
		24 orbVar(out,verb) 48 { }
	pack $f.verb
	
	o_panel_entry $f.log "Log File:" \
		24 orbVar(out,log) 48 { }
	pack $f.log

	# buttons section
	frame $f.buttons
	pack $f.buttons -pady 20
	button $f.buttons.parms -text "Options" \
		-command "panel_orb_hom_options"
	button $f.buttons.exe -text "Predict" \
		-command "execute_orb_hom"
    button $f.buttons.quit -text Exit -command "exit_orb"
    pack $f.buttons.parms $f.buttons.exe $f.buttons.quit -side left
}

#
# user selects a sequence to predict
#
proc bind_orb_seq_select {f name ix} {
	global seqDat 
	$f configure -text $name
	set seqDat(current) $ix
}

#
# user clicks execute button to run orb with homologous shifts 
#
proc execute_orb_hom { } {
	global orbVar seqDat

	if [catch {open $orbVar(file,RUN) w} fp] {
		error "Cannot open run file: $orbVar(file,RUN)"
	}
	puts $fp "2"
	puts $fp $orbVar(file,align)
	puts $fp [expr $seqDat(current) + 1]
	puts $fp "y"
    for {set np 0} {$np < $seqDat(seqNum)} {incr np} {
  		if { $np != $seqDat(current) } {
			if ![info exists seqDat($np,bias)] {
        		puts $fp "n"
				continue
			}
			set fname $seqDat($np,fn)
    		if ![file readable $fname] {
        		puts $fp "n"
				continue
			}
        	puts $fp "y"
        	puts $fp $fname
			if {$seqDat($np,offsetAtom) == "all"} {
        		puts $fp "$seqDat($np,offsetVal) *"
			} else {
        		puts $fp "$seqDat($np,offsetVal) $seqDat($np,offsetAtom)"
			}
        	puts $fp $seqDat($np,bias)
		}
	}
	puts $fp $orbVar(out,ppm)
	puts $fp $orbVar(out,eppm)
	puts $fp $orbVar(out,verb)
	puts $fp $orbVar(out,csi)
	close $fp

	set output "$orbVar(out,ppm) $orbVar(out,eppm) $orbVar(out,csi) $orbVar(out,verb) $orbVar(out,log)"
	execute_orb_x $output
}

########################################################################
#
# panel to configure orb.parms (calculation parameters) 
#
proc panel_orb_hom_options { } {
	global orbVar seqDat

	set f $orbVar(MAIN_WINDOW).options
	toplevel $f
	wm geometry $f $orbVar(geometry,opt1)
	wm title $f "Options"

	label $f.title -text "Orb Calculation Options"
	pack $f.title

	pack [frame $f.file]
	label $f.file.head -text "Input files "
	pack $f.file.head -pady 5
	set title(0) 	"Amino Acids Definition:"
	set title(1) 	"BMRB/Wishart Stats:"
	set title(2) 	"Amino Acid Homology:"
	set title(3)	"Secondary Structure Homology:"
	set title(4) 	"Molecular Similarity:"
	for {set i 0} {$i < 5} {incr i} {
		o_panel_entry $f.file.$i "$title($i)" \
			35 "orbVar(parms,$i)" 40 { } -text "View" \
			-command "o_panel_file_view orbVar(parms,$i)"
		pack $f.file.$i -padx 30
	}

	pack [frame $f.score]
	label $f.score.head -text "Minimum similarity percent score required"
	pack $f.score.head -pady 5
	set title(5) 	"Molecular Homology:"
	set title(6) 	"Global Homology:"
	set title(7) 	"Local Amino Acid Homology:"
	set title(8) 	"Local Secondary Structure Homology:"
	for {set i 5} {$i < 9} {incr i} {
		o_panel_entry $f.score.$i "$title($i)" \
			35 "orbVar(parms,$i)" 10 { } 
		pack $f.score.$i 
	}

	pack [frame $f.size] 
	label $f.size.head -text "Window sizes for defining local homology"
	pack $f.size.head -pady 5
	set title(9) 	"Amino Acid Homology:"
	set title(10) 	"Secondary Structure Homology:"
	for {set i 9} {$i < 11} {incr i} {
		o_panel_entry $f.size.$i "$title($i)" \
			35 "orbVar(parms,$i)" 10 { } 
		pack $f.size.$i
	}

	frame $f.buttons
	pack $f.buttons -pady 20
	button $f.buttons.exe -text "Predict" \
		-command "bind_orb_save_options $f; execute_orb_hom"
	button $f.buttons.done -text "Quit" -command "destroy $f"
	button $f.buttons.reset -text "Reset" -command "set_orb_parms 1"

    for {set np 0} {$np < $seqDat(seqNum)} {incr np} {
		if [info exists seqDat($np,bias)] {
			break;
		}
	}
	if { $np < $seqDat(seqNum) } {
		button $f.buttons.more -text "More Options..." \
			-command "panel_orb_shift_options"
    	pack $f.buttons.exe $f.buttons.more $f.buttons.reset $f.buttons.done -side left
	} else {
    	pack $f.buttons.exe $f.buttons.reset $f.buttons.done -side left
	}
}

#
# save current field values in the options window
#
proc bind_orb_save_options { f } {
	global orbVar
	for {set i 0} {$i < 5} {incr i} {
		set orbVar(parms,$i) [ o_panel_get $f.file.$i ] 
	}
	for {set i 5} {$i < 9} {incr i} {
		set orbVar(parms,$i) [ o_panel_get $f.score.$i ] 
	}
	set orbVar(parms,9) [ o_panel_get $f.size.9 ] 
	set orbVar(parms,10) [ o_panel_get $f.size.10 ] 
	write_orb_parms $orbVar(file,PARMS)
}

###############################################################################
#
# Options window for sequences in orb 
#
proc panel_orb_shift_options {} {
	global seqDat orbVar

	set f $orbVar(MAIN_WINDOW).weighting
	toplevel $f
	wm geometry $f $orbVar(geometry,weight)
	wm title $f "weighting"

	label $f.title -text "Weighting of Chemical Shift Data"
	pack $f.title

	message $f.intro -text \
	"Although orb is designed to calculate reasonable weighted averages\
	of the available chemical shift data, the user may wish to manually\
	increase or decrease the importance of certain shift data.\
	Select the appropriate shift data, type a new MULTIPLIER value,\
	and finally press the return key to save the change. Note that\
	entering a value of 0.0 will tell orb to disregard the shift data\
	selected."
	pack $f.intro

	pack [frame $f.fr] -pady 20
	menubutton $f.fr.seq -text "Select Data" \
		-menu $f.fr.seq.menu -underline 0
	pack $f.fr.seq -side left
	menu $f.fr.seq.menu
	for {set np 0} {$np < $seqDat(seqNum)} {incr np} {
		if [info exists seqDat($np,bias)] {
			$f.fr.seq.menu add command \
				-label "$seqDat($np,key)" \
				-command "bind_orb_option_file $f $np"
		}
	}
	$f.fr.seq.menu add command \
		-label "BMRB/Wishart" \
		-command "bind_orb_option_file $f $np"

	set start ""
	o_panel_entry $f.fr.mult "Multiplier:" \
		15 start 10 { } 
	pack $f.fr.mult -side left

	frame $f.buttons
	pack $f.buttons -pady 20
	button $f.buttons.reset -text "Predict" \
		-command "write_orb_parms $orbVar(file,PARMS); execute_orb_hom"
	button $f.buttons.done -text "Quit" -command "destroy $f"
    pack $f.buttons.reset $f.buttons.done -side left
}

#
# update title and multiplier when user selects a shift data file
#
proc bind_orb_option_file { f np } {
	global seqDat orbVar
	if {$np == $seqDat(seqNum)} {
		$f.fr.seq configure -text "BMRB/Wishart"
		$f.fr.mult.b.entry configure -textvar orbVar(parms,11)
	} else {
		$f.fr.seq configure -text "$seqDat($np,key)"
		$f.fr.mult.b.entry configure -textvar seqDat($np,bias)
	}
}

########################################################################
########################################################################

########################################################################
#
# read the user parameters file 
#
proc set_orb_parms { updateFlag } {
	global orbVar 

	# file is not readable
	if { ![file readable $orbVar(file,PARMS)] } {
		set updateFlag 1
	}

	# we must read system parms and write copy for the user
	if { $updateFlag == 1 } {
		read_orb_parms $orbVar(SYS_PARMS)
		write_orb_parms $orbVar(file,PARMS)
	}

	# now we can read parameters file
	read_orb_parms $orbVar(file,PARMS)
}

########################################################################
#
# read the defaults file (look in user's directory and system location)
#
proc read_orb_defaults { f_user f_sys } {

	# first read system defaults
	if ![file readable $f_sys ] {
		puts stderr "Unable to open installed defaults file: $f_sys"
		exit
	}

	if [catch {option readfile $f_sys interactive} err] {
		puts stderr "Problem reading option in $f_sys: $err"
		exit
	}

	# then read user defaults
	if [file readable $f_user ] {
		if [catch {option readfile $f_user interactive} err] {
			puts stderr "Warning, problem reading option in $f_user: $err"
		}
	}
}

########################################################################
#
# Read orb.parms file as specified by the argument 
#
proc read_orb_parms { fn } {
	global orbVar

	# cannot open file
	if [catch {open $fn r} fp] {
		error "Cannot open orb.parms file: $fn"
	}

	set count 0
	while {[gets $fp line] >= 0} {
		set nd [scan $line "%s %s" key value]
		if {$nd == 2} {
			if { $key == $orbVar(PARMS_KEY) } {
				if {$count < 5 && $fn == $orbVar(SYS_PARMS)} {
					set orbVar(parms,$count) $orbVar(LIB_DIR)/$value 
				} else {
					set orbVar(parms,$count) $value 
				}
				incr count
			}
		}
	}
	close $fp
}

########################################################################
#
# reads alignment data file
#
proc read_orb_align { fn } {
	global seqDat orbVar

	# cannot open file
	if [catch {open $fn r} fp] {
		error "Cannot open alignment file: $fn"
	}

	# read file to start of sequences
    while {[gets $fp line] >= 0} {
		set nd [scan $line "%s" key]
		if {$nd == 1} {
			if {[string compare $key $orbVar(ALIGN_KEY)] == 0} {
				break
			}
		}
	}

	# read each sequence ID
	set count 0
    while {[gets $fp line] >= 0} {
		set nd [scan $line "%d %s" num akey]
		if {$nd == 2} {
			set key [file rootname $akey]
			set seqDat($count,key) $key
			incr count
		} else {
			break
		}
	}
	set seqDat(seqNum)	$count
	set seqDat(current)	0
}

###############################################################################
#
# write orb.parms file 
#
proc write_orb_parms { fn } {
	global orbVar

	# cannot open file
	if [catch {open $fn w} fp] {
		error "Cannot write orb parameters: $fn"
	}

	puts $fp "\nOrb parameters file\n"
	for {set i 0} {$i < 12} {incr i} {
		puts $fp "$orbVar(PARMS_KEY) $orbVar(parms,$i)"
	}
	close $fp
}

#######################################################################
#######################################################################

#
# purpose: to call a tk routine to navigate user's file system 
#
proc o_panel_file_browse { fn } {
	global orbVar 

	set isDir [string match "*dir,*" $fn]
	if { !$isDir } {
		set chosen [tk_getOpenFile]
	} else {
		set chosen [tk_chooseDirectory]
	}
	if { $chosen != "" } { 
		set currentpwd [pwd] 
		set cmd "regsub {$currentpwd\/} $chosen \"\" shortchosen"
		eval $cmd
		set $fn $shortchosen
	}
}

#
# this routine creates a window to display a file 
#
proc o_panel_file_view {fn} {
	global orbVar 
	upvar $fn fname 

	if [catch {open $fname r} fp] {
		error "Cannot read file: $fname"
	}

	set f $orbVar(MAIN_WINDOW).view
	toplevel $f 
	wm geometry $f $orbVar(geometry,view)
	wm title $f "View file:  $fname"

	o_panel_textwin $f.results 80 30
	pack $f.results -pady 10 -side top -fill both -expand true

	button $f.quit -text "Dismiss" -command "destroy $f"
	pack $f.quit -pady 5
	o_load_file $fname $f.results.text
}

#   
# create a generic widget for file text display
#       
proc o_panel_textwin { f width height } {
	frame $f
	pack $f
    text $f.text -relief raised -bd 2 -bg pink -fg black\
		-width $width -height $height \
		-setgrid true -wrap none \
		-xscrollcommand [list $f.xscroll set] \
		-yscrollcommand [list $f.yscroll set]
	scrollbar $f.xscroll -orient horizontal \
		-command [list $f.text xview]
	scrollbar $f.yscroll -orient vertical \
		-command [list $f.text yview]
	pack $f.xscroll -side bottom -fill x
	pack $f.yscroll -side right -fill y
	pack $f.text -side left -fill both -expand true
}

#
# generic orb entry widget
#
proc o_panel_entry { f title tWidth tVar eWidth command args } {
    frame $f
#	pack $f -pady 0 -padx 5 -fill both -expand true
	pack $f -fill both -expand true
    frame $f.a
    frame $f.b
    frame $f.c
	pack $f.a $f.b $f.c -side left
	label $f.a.label -text $title -anchor e -width $tWidth
	pack $f.a.label -side right -padx 5 -fill x -expand true
	entry $f.b.entry -bd 2 -textvar $tVar -width $eWidth -relief sunken
	bind $f.b.entry <Return> $command
	pack $f.b.entry -side left -fill x -expand true
	if { $args != "" } {
		eval {button $f.c.buttons} $args
		pack $f.c.buttons -side left -padx 5
	}
}

#
# generic orb entry widget, get string in item
#
proc o_panel_get { f } {
	return [$f.b.entry get]
}

#
# loads a file onto a canvas - code taken form "TCL and the TK toolkit"
#
proc o_load_file { fname wind } {
	if [catch {open $fname r} fp] {
		error "Cannot read file: $fname"
	}
	$wind delete 1.0 end
	while {![eof $fp]} {
   		$wind insert end [read $fp 10000]
	}
	close $fp
}

#
# display orb results
#
proc o_panel_show_results { files } {
    global orbVar

	set f $orbVar(MAIN_WINDOW).results
	if ![winfo exists $f] {
		toplevel $f 
		wm geometry $f $orbVar(geometry,result)
		wm title $f "results"
		set orbVar(results,select) 0
	} else {
		destroy $f.fr1 $f.results $f.quit
	}

	frame $f.fr1
	pack $f.fr1 -pady 20
	
	set count 0
	foreach fn $files {
		set name [file tail $fn]
		radiobutton $f.fr1.$count -text $name -value $count \
			-variable orbVar(results,select)
		pack $f.fr1.$count -side left
		bind $f.fr1.$count <Button-1> "o_load_file $fn $f.results.text"
		incr count
	}

	o_panel_textwin $f.results 80 30
	set fn [ lindex $files $orbVar(results,select) ]
	o_load_file $fn $f.results.text 
	$f.fr1.$orbVar(results,select) invoke

	pack $f.results -pady 10 -side top -fill both -expand true
	button $f.quit -text "Dismiss" -command "destroy $f"
	pack $f.quit -pady 5
}

###############################################################################
###############################################################################

#
# call orb_x c program to do orb calculations
#
proc execute_orb_x { output } {
	global orbVar

	set unixcmd "$orbVar(ORB_PROG) -i $orbVar(file,PARMS) < $orbVar(file,RUN)"
	if [catch {eval exec $unixcmd > $orbVar(out,log)} err] {
		if { [file size $orbVar(out,log)] > 0 } {
			o_panel_file_view orbVar(out,log)
		} else {
			error "Problems running orb_x program. $err"
		}
	} else {
		o_panel_show_results $output
	}
}
