
/*
	This file contains routines required to perform udsw
*/

#include "main.h"

/* procedure calls */
static void proc_atoms();
static FILE *open_data_file(), *open_output_file();
static int read_sequence(), read_convert(), valid_atom_name();
DB_RES_REC dswTable[CODES];
AMINO_ACID_REC convertTable[CODES];

extern void upper_case(), show_err();
extern void write_dsw_shifts(), read_dsw_shifts();
extern char pres_3to1();


/*************************************************************/
/* convert shift file 
*/
void do_udsw()
{
	int seqNum, cSize;
	char sequence[MAX_SEQ_NUM];
	FILE *outfp, *shiftsfp, *dswfp;

	shiftsfp = open_data_file(bmrbFname, "data.bmrb");
	fprintf(stdout, "\n BMRB Shifts Stats File: %s\n", bmrbFname);

	dswfp = open_data_file(dswFname, "data.dsw");
	fprintf(stdout, "\n DSW Shifts Stats File: %s\n", dswFname);
	read_dsw_shifts(dswfp);

	outfp = open_output_file(resultsFname);
	fprintf(stdout, "\n Output File: %s\n\n", resultsFname);

	cSize = read_convert();
	fprintf(stdout, "\n Conversion Table: %s\n", convertFname);
	fprintf(stdout, " Number of table entries: %d\n", cSize);
	fprintf(stdout, " Make sure that this shift table is up to date.\n");
	fprintf(stdout, " \n");

	proc_atoms(shiftsfp, outfp);
	write_dsw_shifts(outfp);
	fprintf(stdout, " Done\n\n");

}	/* do_udsw */


/*************************************************************/
/* opens user data file.
*/
static FILE *open_data_file(fname, ftype)
char *fname, *ftype;
{
	FILE *fp;

	fp = fopen(fname, "r");
	while (fp == NULL) {
		fprintf(stdout, "\n\n");
		fprintf(stdout, " Please enter your %s file.\n", ftype);
		fprintf(stdout, "\n");
		fscanf(stdin, "%s", fname);
		fp = fopen(fname, "r");
	}

	return(fp);

}	/* open_data_file */


/*************************************************************/
/* opens output file.
*/
static FILE *open_output_file(fname)
char *fname;
{
	FILE *fp;

	fp = fopen(fname, "w");
	while (fp == NULL) {
		fprintf(stdout, "\n\n");
		fprintf(stdout, " Enter your output file.\n");
		fprintf(stdout, "\n");
		fscanf(stdin, "%s", fname);
		fp = fopen(fname, "w");
	}

	return(fp);

}	/* open_output_file */

/*************************************************************/
/* Read shifts conversion file 
*/
static int read_convert()
{
	char res[MAX_LINE_LEN], sVal[MAX_LINE_LEN], old[MAX_LINE_LEN];
	char n1[MAX_LINE_LEN], n2[MAX_LINE_LEN], n3[MAX_LINE_LEN];
	int nd, i, j, num, amb, ix;
	FILE *fp;

	fp = fopen(convertFname, "r");
	while (fp == NULL) {
		fprintf(stdout, "\n\n");
		fprintf(stdout, " I did not find a atom name conversion file in\n");
		fprintf(stdout, " this directory. Please enter a valid file\n");
		fprintf(stdout, "\n");
		fscanf(stdin, "%s", convertFname);
		fp = fopen(convertFname, "r");
	}

	num = 0;
	for (i=0; i<CODES; i++) {
		convertTable[i].atomNum = 0;
	}

	while (TRUE) {

		if (NULL == fgets(sVal, MAX_LINE_LEN, fp))
			break;

		nd = sscanf(sVal, "%s %d %s %s %s %s", res, &amb, old, n1, n2, n3);

		if (nd == 0) {
			fprintf(stdout, "found\n");
		}

		if ((nd >= 3) && (strlen(res) == 1)) {
			ix = 'Z' - res[0];
			if ((ix >= 0) && (ix < CODES)) {
				j = convertTable[ix].atomNum;
				convertTable[ix].atom[j].amb = amb;
				strcpy(convertTable[ix].atom[j].oldAtom, old);
				if (nd == 6) {
					strcpy(convertTable[ix].atom[j].newAtom[0], n1);
					strcpy(convertTable[ix].atom[j].newAtom[1], n2);
					strcpy(convertTable[ix].atom[j].newAtom[2], n3);
				} else if (nd == 5) {
					strcpy(convertTable[ix].atom[j].newAtom[0], n1);
					strcpy(convertTable[ix].atom[j].newAtom[1], n2);
					strcpy(convertTable[ix].atom[j].newAtom[2], "");
				} else if (nd == 4) {
					strcpy(convertTable[ix].atom[j].newAtom[0], n1);
					strcpy(convertTable[ix].atom[j].newAtom[1], "");
					strcpy(convertTable[ix].atom[j].newAtom[2], "");
				} else {
					strcpy(convertTable[ix].atom[j].newAtom[0], "");
					strcpy(convertTable[ix].atom[j].newAtom[1], "");
					strcpy(convertTable[ix].atom[j].newAtom[2], "");
				}
				convertTable[ix].atomNum++;
				num++;
			}
		}
	}
	return(num);
}	/* read_convert */


/*************************************************************/
/* Read and process shifts from data file 
*/
static void proc_atoms(fp, outfp)
FILE *fp, *outfp;
{
	char res3[MAX_LINE_LEN], line[MAX_LINE_LEN];
	char sVal[MAX_LINE_LEN], *aNew, atom[MAX_LINE_LEN];
	int i, j, m, nd, ch, id, ix, num, dnum, amb, stereo;
	double shift, stddev;

	num = 0;
	while (TRUE) {

        nd = fscanf(fp, "%s %s %*s %*s %*s %*s %lf %lf",
			res3, sVal, &shift, &stddev);

		if (nd == 4) {
			upper_case(res3);
			ix = pres_3to1(res3);

			if (dswTable[ix].atomNum == 0) {
				strcpy(dswTable[ix].rName, res3);
			}

			num = convertTable[ix].atomNum;
			for (i=0; i<num; i++) {
				if (strcmp(sVal, convertTable[ix].atom[i].oldAtom) == 0) {
					if (strlen(convertTable[ix].atom[i].newAtom[0]) == 0) {
						fprintf(stdout, "Warning, cannot convert %s %s\n",
							res3, sVal);
						break;
					}
					dnum = dswTable[ix].atomNum;
					for (m=0; m<3; m++) {
						aNew = convertTable[ix].atom[i].newAtom[m];
						if (strlen(aNew) == 0) {
							break;
						}
						for (j=0; j<dnum; j++) {
							if (0 == strcmp(aNew, dswTable[ix].atom[j].rName)) {
								break;
							}
						}
						if (j == dnum) {
							dswTable[ix].atomNum++;
							strcpy(dswTable[ix].atom[j].rName, aNew);
							dswTable[ix].atom[j].dswShift[COIL] = shift;
							dswTable[ix].atom[j].dswShift[BETA] = shift;
							dswTable[ix].atom[j].dswShift[HELX] = shift;
							dswTable[ix].atom[j].dswStddev[COIL] = stddev;
							dswTable[ix].atom[j].dswStddev[BETA] = stddev;
							dswTable[ix].atom[j].dswStddev[HELX] = stddev;
						}
						fprintf(stdout, "changed '%s' '%s' %lf \n", res3, sVal, shift);
						dswTable[ix].atom[j].dswShift[NO_SS] = shift;
						dswTable[ix].atom[j].dswStddev[NO_SS] = stddev;
					}
					break;
				}
			}
			if (i == num) {
				fprintf(stdout, "Error, cannot convert atom '%s' '%s' \n",
					res3, sVal);
			}
		}
		if (NULL == fgets(sVal, MAX_LINE_LEN, fp))
			break;
	}
}	/* proc_atoms */
