
/*
	file: dsw.c
	purpose: read/write  wishart's shifts database 
*/

#include "main.h" 

/* global vars */
char	dswFname[MAX_FNAME_LEN];

/* procedure calls */
extern void show_err();
extern int pres_3to1();



/*************************************************************/
/* Read Wishart default shifts file 
*/
void read_dsw_shifts(fp)
FILE *fp;
{
	int i, j, nd, ix, num;
	double coil, helx, beta, ss;
	double sdCoil, sdHelx, sdBeta, sdss;
	char res[MAX_LINE_LEN], atom[MAX_LINE_LEN];
	char aStr[MAX_LINE_LEN], sVal[MAX_LINE_LEN];

	while (TRUE) {
		if (NULL == fgets(aStr, MAX_LINE_LEN, fp)) {
			break;
		}
		if (aStr[0] != COMMENT) {
			nd = sscanf(aStr, 
				"%3s:%s %*s %lf%lf %*s %lf%lf %*s %lf%lf %*s %lf%lf",
				res, atom, &coil, &sdCoil, &beta, &sdBeta, &helx, &sdHelx,
				&ss, &sdss);
			if (nd != 10) {
				sprintf(sVal, "Bad file format: %s\n", aStr);
				show_err(stdout, FALSE, "read_dsw_shifts", sVal);
				continue;
			}

			ix = pres_3to1(res);
			if (ix < 0) {
				sprintf(sVal, "Illegal amino acid: %s\n", res);
				show_err(stdout, TRUE, "read_dsw_shifts", sVal);
			}

			strcpy(dswTable[ix].rName, res);
			num = dswTable[ix].atomNum;
			for (j=0; j<num; j++) {
				if (0 == strcmp(atom, dswTable[ix].atom[j].rName)) {
					sprintf(sVal, "Duplicate entry: %s %s\n", res, atom);
					show_err(stdout, FALSE, "read_dsw_shifts", sVal);
				}
			}

			strcpy(dswTable[ix].atom[j].rName, atom);
			dswTable[ix].atom[j].dswShift[COIL] = coil;
			dswTable[ix].atom[j].dswStddev[COIL] = sdCoil;
			dswTable[ix].atom[j].dswShift[BETA] = beta;
			dswTable[ix].atom[j].dswStddev[BETA] = sdBeta;
			dswTable[ix].atom[j].dswShift[HELX] = helx;
			dswTable[ix].atom[j].dswStddev[HELX] = sdHelx;
			dswTable[ix].atom[j].dswShift[NO_SS] = ss;
			dswTable[ix].atom[j].dswStddev[NO_SS] = sdss;
			dswTable[ix].atomNum++;
		}
	}
}	/* read_dsw_shifts */

/*************************************************************/
/* write Wishart default shifts file 
*/
void write_dsw_shifts(fp)
FILE *fp;
{
	int ix, j, k, num;

	fprintf(fp, "# \n"); 
	fprintf(fp, "# Shift Statistics - David Wishart structure shift values +\n"); 
	fprintf(fp, "#    averaged values from BMRB for last column\n"); 
	fprintf(fp, "# \n"); 
	fprintf(fp, "# \n"); 
	fprintf(fp,"#          |      COIL     |      BETA     |      HELIX    |    BMRB       |\n");
	fprintf(fp,"#----------|---------------|---------------|---------------|---------------|\n");

	fprintf(fp, "# \n"); 
	for (ix=25; ix>=0; ix--) {
		num = dswTable[ix].atomNum;
		for (j=0; j<num; j++) {
			fprintf(fp, "%3s:%-6s |", 
				dswTable[ix].rName, dswTable[ix].atom[j].rName);
			for (k=0; k<STRUCT_NUM; k++) {
				fprintf(fp, "%7.2f%7.2f |",
					dswTable[ix].atom[j].dswShift[k],
					dswTable[ix].atom[j].dswStddev[k]);
			}
			fprintf(fp, "\n");
		}
	}
}
