/*
	main.h file
*/

#include <stdio.h>
#include <string.h>

#define TRUE				1
#define FALSE				0
#define MAX_LINE_LEN		256
#define MAX_FNAME_LEN		256

#define MAX_RES_NAME_LEN    5
#define MAX_SEQ_NUM			1000
#define CODES				26
#define MAX_RES_ATOM_NUM    50
#define MAX_ATOM_NAME_LEN	6
#define COMMENT				'#'

#define STRUCT_NUM          4
#define BETA                0
#define COIL                1
#define HELX                2
#define NO_SS               3

/* convert chars from upper to lower case and vice versa */

#define AA_UC(c)			(((c>='a') && (c<='z'))? c+'A'-'a': c)
#define AA_LC(c)			(((c>='A') && (c<='Z'))? c-'A'+'a': c)


/* first and last character codes */
#define FIRST_CODE          'A'
#define LAST_CODE           'Z'

/* range for amino acid one letter code, eg 'Z' - 'A' */
#define CODES               (LAST_CODE - FIRST_CODE + 1)

/* return max or min of two numbers */
#define MAX(x,y)			(x>y? x: y)
#define MIN(x,y)			(x<y? x: y)

/********************/
/* define structures */
/*********************/

struct dbAtomRec {
	double dswShift[STRUCT_NUM];
	double dswStddev[STRUCT_NUM];
	char rName[MAX_ATOM_NAME_LEN];
};

struct dbResRec {
	int atomNum;
    struct dbAtomRec atom[MAX_RES_ATOM_NUM];
	char rName[MAX_RES_NAME_LEN];
};

struct atomRec {
	char oldAtom[MAX_ATOM_NAME_LEN];
	char newAtom[3][MAX_ATOM_NAME_LEN];
	int amb;
};

struct aminoAcidRec {
	int atomNum;
	struct atomRec atom[MAX_RES_ATOM_NUM];
};


typedef struct atomRec ATOM_REC;
typedef struct aminoAcidRec AMINO_ACID_REC;
typedef struct dbAtomRec DB_ATOM_REC;
typedef struct dbResRec DB_RES_REC;


/* global variables */

extern DB_RES_REC dswTable[CODES];
extern AMINO_ACID_REC convertTable[CODES];

extern char    bmrbFname[MAX_FNAME_LEN];
extern char    dswFname[MAX_FNAME_LEN];
extern char    convertFname[MAX_FNAME_LEN];
extern char    resultsFname[MAX_FNAME_LEN];
extern int	   RC;
