

typedef struct {
  char *ref;
  char *accession;
  char *residues;
  char *journal;
  char *title;
} reference;

typedef struct {
  char class;
  char *entry;
  char *title;
  char *formal_name;
  char *common_name;
  char no_refs;
  reference *refs;
  char *sequence;
  short length;
  int molecular_weight;
} dbentry;

static char *rep_words[] = {"-transporting",
"1-aminocyclopropane-1-carboxylate",
"adenovirus",
"aeruginosa",
"african",
"agrobacterium",
"Alcaligenes",
"alpha-amylase",
"alpha",
"american",
"Anabaena",
"antigen",
"",
"Arabidopsis",
"aries",
"Aspergillus",
"aureus",
"Azotobacter",
"acillus",
"barley",
"beta",
"bovine",
"Bradyrhizobium",
"brucei",
"Caenorhabditis",
"californica",
"Candida",
"capricolum",
"capsulatus",
"carboxylase",
"cattle",
"",
"cerevisiae",
"chain",
"chicken",
"Chironomus",
"Chlamydia",
"Chlamydomonas",
"chloroplast",
"class",
"clawed",
"clone",
"Clostridium",
"coelicolor",
"coli",
"collagen",
"common",
"component",
"Copenhagen",
"Corynebacterium",
"cress",
"cuniculus",
"cytochrome-c",
"cytochrome",
"cytomegalovirus",
"decarboxylase",
"dehydrogenase",
"denitrificans",
"Desulfovibrio",
"Dictyostelium",
"discoideum",
"disease",
"dna-binding",
"dna-directed",
"domestica",
"domestic",
"Drosophila",
"elegans",
"Emericella",
"Enterococcus",
"enterocolitica",
"envelope",
"Epstein-Barr",
"Erwinia",
"Escherichia",
"esculentum",
"factor",
"falciparum",
"familiaris",
"fly",
"fragment",
"fragments",
"frog",
"fruit",
"allus",
"garden",
"gene",
"glutathione",
"glyceraldehyde-3-phosphate",
"glycine",
"glycoprotein",
"gonorrhoeae",
"growth",
"gtp-binding",
"Haemophilus",
"Halobacterium",
"hamster",
"heavy",
"hemagglutinin",
"hemoglobin",
"hepatitis",
"herpesvirus",
"histocompatibility",
"histone",
"homeotic",
"homolog",
"Homo",
"house",
"human",
"hypothetical",
"immunodeficiency",
"infectious",
"influenza",
"inhibitor",
"isolate",
"Japanese",
"kappa",
"kinase",
"Klebsiella",
"Kluyveromyces",
"lactis",
"Lactobacillus",
"Lactococcus",
"laevis",
"leguminosarum",
"Lycopersicon",
"maize",
"man",
"Marchantia",
"marxianus",
"Mastadenovirus",
"melanogaster",
"meliloti",
"membrane",
"meningitidis",
"Methanobacterium",
"Methanococcus",
"mitochondrial",
"mitochondrion",
"mosaic",
"mouse-ear",
"mouse",
"musculus",
"Mus",
"Mycobacterium",
"Mycoplasma",
"Neisseria",
"Neurospora",
"Nicotiana",
"nidulans",
"nonstructural",
"norvegicus",
"Norway",
"nuclear",
"Oncorhynchus",
"orientalis",
"Oryctolagus",
"oxidase",
"papillomavirus",
"peptide",
"phage",
"phosphatase",
"phospholipase",
"phosphoprotein",
"phosphoribosyltransferase",
"photosystem",
"pig",
"plasmid",
"Plasmodium",
"pneumoniae",
"polyhedrosis",
"polymerase",
"polymorpha",
"polyprotein",
"pombe",
"potato",
"precursor",
"primigenius",
"probable",
"protein-tyrosine-phosphatase",
"protein-tyrosine",
"proteinase",
"protein",
"Pseudomonas",
"putida",
"rabbit",
"Rattus",
"rat",
"receptor",
"reductase",
"region",
"regulatory",
"reinhardtii",
"hizobium",
"Rhodobacter",
"ribosomal",
"ribulose-bisphosphate",
"rice",
"rna-directed",
"rotavirus",
"Saccharomyces",
"Salmonella",
"sapiens",
"sativa",
"Schizosaccharomyces",
"scrofa",
"sequence",
"sheep",
"silkworm",
"site-specific",
"soybean",
"sphaeroides",
"spinach",
"Staphylococcus",
"stearothermophilus",
"strain",
"Streptococcus",
"Streptomyces",
"Strongylocentrotus",
"subtilis",
"sulfolobus",
"surface",
"Synechococcus",
"Synechocystis",
"synthase",
"t-cell",
"tabacum",
"taurus",
"Tetrahymena",
"thaliana",
"thermoautotrophicum",
"thuringiensis",
"tobacco",
"tomato",
"trachomatis",
"transcription",
"transferase",
"transforming",
"translation",
"transport",
"transposon",
"Triticum",
"Trypanosoma",
"tumefaciens",
"type",
"typhimurium",
"ubiquinol--cytochrome-c",
"ubiquinone",
"vaccinia",
"variola",
"Vibrio",
"vinelandii",
"virus",
"vulgaris",
"wheat",
"Xenopus",
"yeast"};

static char *journals[] = {"AIDS Res. Hum. Retroviruses (19",
"APMIS (19",
"Acc. Chem. Res. (19",
"Acta Biochim. Biophys. Acad. Sci. Hung. (19",
"Acta Biochim. Biophys. Sin. (19",
"Acta Chem. Scand. (19",
"Acta Chem. Scand. B Org. Chem. Biochem. (19",
"Acta Crystallogr. (19",
"Acta Dermatol. Venereol. (19",
"Acta Endocrinol. (19",
"Acta Genet. Sin. (19",
"Acta Med. Scand. Suppl. (19",
"",
"Acta Microbiol. Sin. (19",
"Acta Pharm. Suec. (19",
"Actinomycetol. (19",
"Adv. Enzyme Regul. (19",
"Adv. Enzymol. (19",
"Adv. Exp. Med. Biol. (19",
"Adv. Prostaglandin Thromboxane Leukotriene Res. (19",
"Adv. Protein Chem. (19",
"Adv. Second Messenger Phosphoprotein Res. (19",
"Agents Actions (19",
"Agric. Biol. Chem. (19",
"Allergy (19",
"Am. J. Clin. Nutr. (19",
"Am. J. Hum. Genet. (19",
"Am. J. Hypertens. (19",
"Am. J. Med. (19",
"Am. J. Med. Genet. (19",
"Am. J. Obstet. Gynecol. (19",
"",
"Am. J. Pathol. (19",
"Am. J. Phys. Anthropol. (19",
"Am. J. Physiol. (19",
"Am. J. Trop. Med. Hyg. (19",
"Am. J. Vet. Res. (19",
"Am. Rev. Respir. Dis. (19",
"Am. Soc. Hematol. Abstr. (19",
"Anal. Biochem. (19",
"Anat. Embryol. (19",
"Anim. Genet. (19",
"Ann. Genet. Sel. Anim. (19",
"Ann. Hum. Genet. (19",
"Ann. Immunol. (Inst. Paste",
"Ann. Inst. Pasteur Virol. (19",
"Ann. N. Y. Acad. Sci. (19",
"Ann. Neurol. (19",
"Ann. Phytopathol. Soc. Jpn (19",
"Ann. Phytopathol. Soc. Jpn. (19",
"Annu. Rev. Biochem. (19",
"Anticancer Res. (19",
"Antimicrob. Agents Chemother. (19",
"Antonie Van Leeuwenhoek (19",
"Appl. Environ. Microbiol. (19",
"Appl. Microbiol. Biotechnol. (19",
"Appl. Theor. Electrophor. (19",
"Arch. Biochem. Biophys. (19",
"Arch. Int. Physiol. Biochim. (19",
"Arch. Microbiol. (19",
"Arch. Oral Biol. (19",
"Arch. Virol. (19",
"Ark. Kemi (19",
"Arterioscler. Thromb. (19",
"Arthritis Rheum. (19",
"Atherosclerosis (19",
"Aust. J. Biol. Sci. (19",
"Aust. J. Plant Physiol. (19",
"Barley Genet. Newsl. (19",
"Basic Life Sci. (19",
"Behring Inst. Mitt. (19",
"Bio\\/Technology (19",
"BioTechniques (19",
"Biochem. Biophys. Res. Commun. (19",
"Biochem. Cell Biol. (19",
"Biochem. Genet. (19",
"Biochem. Int. (19",
"Biochem. J. (19",
"Biochem. Med. Metab. Biol. (19",
"Biochem. Mol. Biol. Int. (19",
"Biochem. Pharmacol. (19",
"Biochem. Physiol. Pflanz. (19",
"Biochem. Soc. Trans. (19",
"Biochemistry (19",
"Biochemistry (N.",
"Biochim. Biophys. Acta (19",
"Biochimie (19",
"Biokhimiia (19",
"Biol. Cell (19",
"Biol. Chem. Hoppe-Seyler (19",
"Biol. Pharm. Bull. (19",
"Biol. Reprod. (19",
"Biomed. Biochim. Acta (19",
"Biomed. Environ. Mass Spectrom. (19",
"Biomed. Res. (19",
"Biomed. Sci. (19",
"Bioorg. Chem. (19",
"Bioorg. Khim. (19",
"Biophys. J. (19",
"Biophys. Struct. Mech. (19",
"Biopolymers (19",
"Biosci. Biotechnol. Biochem. (19",
"Biosci. Rep. (19",
"Biotechnol. Appl. Biochem. (19",
"Biotechnol. Lett. (19",
"Biotechnol. Prog. (19",
"Biotechnology (19",
"Blood (19",
"Blood Cells (19",
"Bot. Acta (19",
"Br. J. Cancer (19",
"Br. J. Pharmacol. (19",
"Br. J. Pharmacol. Chemother. (19",
"Brain Res. (19",
"Brain Res. Bull. (19",
"Brain Res. Dev. Brain Res. (19",
"Brain Res. Mol. Brain Res. (19",
"Bull. Soc. Chim. Biol. (19",
"Bull. World Health Organ. (19",
"C. R. Acad. Sci. (19",
"C. R. Acad. Sci. D Sci. Nat. (19",
"C. R. Acad. Sci. III Sci. Vie (19",
"C. R. Seances Soc. Biol. Fil. (19",
"Calcif. Tissue Int. (19",
"Can. J. Biochem. (19",
"Can. J. Biochem. Cell Biol. (19",
"Can. J. For. Res. (19",
"Can. J. Microbiol. (19",
"Can. J. Physiol. Pharmacol. (19",
"Can. J. Zool. (19",
"Cancer Commun. (19",
"Cancer Lett. (19",
"Cancer Res. (19",
"Carbohydr. Res. (19",
"Carcinogenesis (19",
"Carlsberg Res. Commun. (19",
"Cell (19",
"Cell Differ. Dev. (19",
"Cell Growth Differ. (19",
"Cell Motil. Cytoskeleton (19",
"Cell Motil. Cytoskeleton. (19",
"Cell Regul. (19",
"Cell Struct. Funct. (19",
"Cell. Immunol. (19",
"Cell. Mol. Biol. (19",
"Cell. Mol. Neurobiol. (19",
"Cell. Signal. (19",
"Cereal Chem. (19",
"Chem. Ber. (19",
"Chem. Commun. (19",
"Chem. Ind. (19",
"Chem. Lett. (19",
"Chem. Pharm. Bull. (19",
"Chem. Phys. Lipids (19",
"Chimia (19",
"Chinese Biochem. J. (19",
"Chinese J. Microbiol. Immunol. (19",
"Chinese J. Virol. (19",
"Chromosoma (19",
"Circ. Res. (19",
"Clin. Biochem. (19",
"Clin. Chem. (19",
"Clin. Chim. Acta (19",
"Clin. Exp. Immunol. (19",
"Clin. Sci. (19",
"Cold Spring Harb. Symp. Quant. Biol. (19",
"Collect. Czech. Chem. Commun. (19",
"Comp. Biochem. Physiol. (19",
"Comp. Biochem. Physiol. B Comp. Biochem. (19",
"Connect. Tissue Res. (19",
"Curr. Eye Res. (19",
"Curr. Genet. (19",
"Curr. Microbiol. (19",
"Curr. Top. Microbiol. Immunol. (19",
"Curr. Top. Plant Biochem. Physiol. (19",
"Cytogenet. Cell Genet. (19",
"Cytokine (19",
"DNA (19",
"DNA Cell Biol. (19",
"DNA Res. (19",
"DNA Seq. (19",
"Denpun Kagaku (19",
"Dev. Biol. (19",
"Dev. Comp. Immunol. (19",
"Dev. Genet. (19",
"Dev. Growth Differ. (19",
"Dev. Neurosci. (19",
"Development (19",
"Diabetes (19",
"Diabetologia (19",
"Differentiation (19",
"Digestion (19",
"Dokl. Akad. Nauk SSSR (19",
"Dokl. Biochem. (19",
"Drug Metab. Dispos. (19",
"EMBO J. (19",
"Electrophoresis (19",
"Endocr. Res. (19",
"Endocrinol. Exp. (19",
"Endocrinology (19",
"Environ. Health Perspect. (19",
"Enzyme (19",
"Enzyme Microb. Technol. (19",
"Eur. J. Biochem. (19",
"Eur. J. Cell Biol. (19",
"Eur. J. Haematol. (19",
"Eur. J. Immunol. (19",
"Eur. J. Pharmacol. (19",
"Evolution (19",
"Exp. Cell Res. (19",
"Exp. Eye Res. (19",
"Exp. Hematol. (19",
"Exp. Parasitol. (19",
"Experientia (19",
"FASEB J. (19",
"FEBS Lett. (19",
"FEMS Microbiol. Lett. (19",
"Fed. Proc. (19",
"Fibrinolysis (19",
"Gastroenterology (19",
"Gen. Comp. Endocrinol. (19",
"Gene (19",
"Gene Expression (19",
"Genes Dev. (19",
"Genet. Res. (19",
"Genetica (19",
"Genetics (19",
"Genome (19",
"Genomics (19",
"Glia (19",
"Glycobiology (19",
"Glycoconjugate J. (19",
"Growth Factors (19",
"Gut (19",
"Helv. Chim. Acta (19",
"Hemoglobin (19",
"Hepatology (19",
"Hereditas (19",
"Histochemistry (19",
"Hoppe-Seyler's Z. Physiol. Chem. (19",
"Horm. Metab. Res. (19",
"Hum. Genet. (19",
"Hum. Immunol. (19",
"Hum. Mol. Genet. (19",
"Hypertension (19",
"ICN UCLA Symp. Mol. Cell. Biol. (19",
"IRCS Med. Sci. (19",
"Immunobiology (19",
"Immunochemistry (19",
"Immunogenetics (19",
"Immunol. Cell Biol. (19",
"Immunol. Invest. (19",
"Immunol. Lett. (19",
"Immunology (19",
"Infect. Immun. (19",
"",
"Insect Biochem. (19",
"Int. Arch. Allergy Appl. Immunol. (19",
"Int. Congr. Ser. (19",
"Int. Immunol. (19",
"Int. J. Biochem. (19",
"Int. J. Biol. Macromol. (19",
"Int. J. Cancer (19",
"Int. J. Parasitol. (19",
"Int. J. Pept. Protein Res. (19",
"Int. J. Protein Res. (19",
"Int. J. Syst. Bacteriol. (19",
"Intervirology (19",
"",
"Invest. Ophthalmol. Vis. Sci. (19",
"Ital. J. Biochem. (19",
"Izv. Akad. Nauk SSSR Ser. Khim. (19",
"J. Agric. Food Chem. (19",
"J. Allergy Clin. Immunol. (19",
"J. Am. Chem. Soc. (19",
"J. Antibiot. (19",
"J. Antimicrob. Chemother. (19",
"J. Bacteriol. (19",
"J. Biochem. (19",
"J. Biol. Chem. (19",
"J. Bone Miner. Res. (19",
"J. Cancer Res. Clin. Oncol. (19",
"J. Cardiovasc. Pharmacol. (19",
"J. Cell Biol. (19",
"J. Cell Sci. (19",
"J. Cell Sci. Suppl. (19",
"J. Cell. Biochem. (19",
"",
"J. Cell. Physiol. (19",
"J. Cereal Sci. (19",
"J. Chem. Soc. Chem. Commun. (19",
"J. Chem. Soc. D Chem. Commun. (19",
"J. Chem. Soc. Pak. (19",
"J. Chromatogr. (19",
"J. Clin. Endocrinol. Metab. (19",
"J. Clin. Invest. (19",
"J. Clin. Microbiol. (19",
"J. Comp. Physiol. B (19",
"J. Dairy Res. (19",
"J. Dairy Sci. (19",
"J. Dent. Res. (19",
"J. Endocrinol. (19",
"J. Evol. Biochem. Physiol. (19",
"J. Exp. Bot. (19",
"J. Exp. Med. (19",
"J. Exp. Pathol. (19",
"J. Exp. Zool. (19",
"J. Fac. Med. Baghdad (19",
"J. Ferment. Bioeng. (19",
"J. Gen. Appl. Microbiol. (19",
"J. Gen. Microbiol. (19",
"J. Gen. Virol. (19",
"J. Genet. and Breed. (19",
"J. Hered. (19",
"J. Immunogenet. (19",
"J. Immunol. (19",
"J. Immunol. Methods (19",
"J. Infect. Dis. (19",
"J. Inherit. Metab. Dis. (19",
"J. Inorg. Biochem. (19",
"J. Interferon Res. (19",
"J. Invertebr. Pathol. (19",
"J. Invest. Dermatol. (19",
"J. Lab. Clin. Med. (19",
"J. Leukoc. Biol. (19",
"J. Lipid Res. (19",
"J. Med. Microbiol. (19",
"J. Med. Virol. (19",
"J. Membr. Biol. (19",
"J. Mol. Appl. Genet. (19",
"J. Mol. Biol. (19",
"J. Mol. Cell. Cardiol. (19",
"J. Mol. Endocrinol. (19",
"J. Mol. Evol. (19",
"J. Natl. Cancer Inst. (19",
"J. Neural Transmission (19",
"J. Neurochem. (19",
"J. Neuroimmunol. (19",
"J. Neurosci. (19",
"J. Neurosci. Res. (19",
"J. Nutr. (19",
"J. Photochem. Photobiol. B (19",
"J. Phycol. (19",
"J. Protein Chem. (19",
"J. Reprod. Fertil. (19",
"J. S. Afr. Chem. Inst. (19",
"J. Sci. Food Agric. (19",
"J. Steroid Biochem. (19",
"J. Supramol. Struct. (19",
"J. Virol. (19",
"Johns Hopkins Med. J. Suppl. (19",
"Jpn. J. Cancer Res. (19",
"Jpn. J. Clin. Oncol. (19",
"Jpn. J. Genet. (19",
"Jpn. J. Med. Sci. Biol. (19",
"Jpn. J. Pharmacol. (19",
"Justus Liebigs Ann. Chem. (19",
"Kagaku To Seibutsu (19",
"Kexue Tongbao (19",
"Kidney Int. (19",
"Lab. Invest. (19",
"Lancet (19",
"Leukemia (19",
"Life Sci. (19",
"Lipids (19",
"Lymphokine Cytokine Res. (19",
"Matrix (19",
"Mech. Dev. (19",
"Med. Microbiol. Immunol. (19",
"Meth. Enzymol. (19",
"Microb. Pathog. (19",
"Microbiol. Immunol. (19",
"Milchwissenschaft (19",
"Mol. Biochem. Parasitol. (19",
"Mol. Biol. (19",
"Mol. Biol. (Mosk) (19",
"Mol. Biol. Cell (19",
"Mol. Biol. Evol. (19",
"Mol. Biol. Med. (19",
"Mol. Biol. Rep. (19",
"Mol. Carcinog. (19",
"Mol. Cell. Biochem. (19",
"Mol. Cell. Biol. (19",
"Mol. Cell. Endocrinol. (19",
"Mol. Cell. Neurosci. (19",
"Mol. Endocrinol. (19",
"Mol. Gen. Genet. (19",
"Mol. Immunol. (19",
"Mol. Microbiol. (19",
"Mol. Pharmacol. (19",
"Mol. Phylogenet. Evol. (19",
"Mol. Plant Microb. Interact. (19",
"Mol. Reprod. Dev. (19",
"Monatsh. Chem. (19",
"Muscle Nerve (19",
"Mutat. Res. (19",
"Nagasaki Igakkai Zasshi (19",
"Nat. Cult. (19",
"Nat. Immun. Cell Growth Regul. (19",
"Natural Toxins (19",
"Nature (19",
"Nature Genet. (19",
"Nature New Biol. (19",
"Nature Struct. Biol. (19",
"Naturwissenschaften (19",
"Neth. Milk Dairy J. (19",
"Neurochem. Int. (19",
"Neurochem. Res. (19",
"Neurology (19",
"Neuron (19",
"Neuropeptides (19",
"Neuropharmacology (19",
"Neurosci. Lett. (19",
"Neuroscience (19",
"New Biol. (19",
"Nichiidaishi (19",
"Nippon Nogeikagaku Kaishi (19",
"Nucleic Acids Res. (19",
"Oncogene (19",
"Oncogene Res. (19",
"Ophthalmic Res. (19",
"Org. Mass Spectrom. (19",
"Pancreas (19",
"Parasite Immunol. (19",
"Parasitol. Res. (19",
"Pediatr. Res. (19",
"Pept. Res. (19",
"Peptides (19",
"Pharmacol. Biochem. Behavior (19",
"Pharmacology (19",
"Philos. Trans. R. Soc. Lond. (19",
"Philos. Trans. R. Soc. Lond. B (19",
"Photochem. Photobiol. (19",
"Photosyn. Res. (19",
"Physiol. Chem. Phys. (19",
"Physiol. Mol. Plant Pathol. (19",
"Physiol. Plantarum (19",
"Physiol. Veg. (19",
"Phytochemistry (19",
"Pigment Cell Res. (19",
"Plant Cell (19",
"Plant Cell Physiol. (19",
"Plant J. (19",
"Plant Mol. Biol. (19",
"Plant Physiol. (19",
"Plant Sci. (19",
"Plant Sci. Lett. (19",
"Planta (19",
"Plasmid (19",
"Poult. Sci. (19",
"Preparative Biochemistry (19",
"Proc. Jpn. Acad. (19",
"Proc. Jpn. Acad. B Phys. Biol. Sci. (19",
"Proc. Natl. Acad. Sci. U.S.A. (19",
"Proc. R. Soc. Lond. B Biol. Sci. (19",
"Proc. Soc. Exp. Biol. Med. (19",
"Prog. Nucleic Acid Res. Mol. Biol. (19",
"Prog. Photosyn. Res. (19",
"Protein Eng. (19",
"Protein Expr. Purif. (19",
"Protein Sci. (19",
"Protein Seq. Data Anal. (19",
"Proteins (19",
"Pure Appl. Chem. (19",
"Recent Prog. Horm. Res. (19",
"Regul. Pept. (19",
"Regul. Pept. Suppl. (19",
"Reprod. Fertil. Dev. (19",
"Res. Immunol. (19",
"Res. Microbiol. (19",
"Res. Virol. (19",
"Rev. Fr. Transfus. Immunohematol. (19",
"Rev. Infect. Dis. (19",
"S. Afr. J. Chem. (19",
"Sapporo Igaku Zasshi (19",
"Scand. J. Clin. Lab. Invest. (19",
"Scand. J. Immunol. (19",
"Sci. Sin. (19",
"Sci. Sin. B Chem. Biol. Agric. Med. Earth Sci. (19",
"Science (19",
"Science in China (series",
"Second Messengers Phosphoproteins (19",
"Seikagaku (19",
"Somat. Cell Mol. Genet. (19",
"Southeast Asian J. Trop. Med. Public Health (19",
"Sov. Genet. (19",
"Sov. J. Bioorg. Chem. (19",
"Syst. Appl. Microbiol. (19",
"Syst. Zool. (19",
"Tetrahedron (19",
"Tetrahedron Lett. (19",
"Tex. Rep. Biol. Med. (19",
"Text. Res. J. (19",
"Theor. Appl. Genet. (19",
"Thromb. Haemost. (19",
"Thromb. Res. (19",
"Tissue Antigens (19",
"Toxicol. Appl. Pharmacol. (19",
"Toxicon (19",
"Trans. Assoc. Am. Physicians (19",
"Tree Physiol. (19",
"Trends Biochem. Sci. (19",
"Trends Genet. (19",
"Ups. J. Med. Sci. (19",
"Vaccine (19",
"Virchows Arch. B Cell Pathol. (19",
"Virology (19",
"Virus Genes (19",
"Virus Res. (19",
"Vision Res. (19",
"Viva Origino (19",
"",
"Vopr. Med. Khim. (19",
"Vox Sang. (19",
"Yeast (19",
"Z. Naturforsch. B (19",
"Z. Naturforsch. C Biosci. (19",
"Z. Vererbungsl. (19",
"Zbl. Bakt. Hyg. I. Abt. Orig. C (19",
"Zool. Res. (19",
"Zool. Sci. (19"};

#define NO_JOURNALS 522

