
/*
	SEQSEE header file
*/

#include <stdio.h>
#include <string.h>

#define TRUE				1
#define FALSE				0
#define ERROR_FLAG			-1

#define MAX_LINE_LEN		256
#define MAX_FNAME_LEN		256
#define MAX_ACC_LEN			8	

#define MIN_INT				(1 << 32 - 1)
#define MAX_INT				(~MIN_INT)
#define MIN_DOUBLE			((double)MIN_INT)
#define MAX_DOUBLE			((double)MAX_INT)

/* first and last character codes */
#define FIRST_CODE			'A'
#define LAST_CODE			'Z'

/* range for amino acid one letter code, eg 'Z' - 'A' */
#define CODES				(LAST_CODE - FIRST_CODE + 1)

/* total different amino acids */
#define AA_NUM				20

/* prompt string for input queries */
#define PROG_PROMPT			"\n>> "
#define DATA_PROMPT			'>'

/* define input formats for sequences which we support */
#define SEQ_PIR             1
#define SEQ_SPROT           2
#define SEQ_PEP             3
#define SEQ_PIRCOMP         4
#define SEQ_RBO             5
#define SEQ_KBD             6

/* define scoring matrices */
#define COMMENT				'#'
#define ANCHOR_NUM			5
#define WT_MAT_DIM			(CODES + 1 + ANCHOR_NUM)
#define WT_MAT_SIZE			(WT_MAT_DIM * WT_MAT_DIM)
#define WT_MAT_INIT			0
#define MIN_WT_MAT_VAL		-50	
#define MAX_WT_MAT_VAL		50

/* limits on sequence length */
#define MAX_RES				4000
#define MIN_RES				3

/* define secondary structure types */
#define CH_BETA				'B'
#define CH_COIL				'C'
#define CH_HELX				'H'
#define CH_TURN				'T'
#define CH_UNKNOWN			'X'
#define BETA				0
#define COIL				1
#define HELX				2
#define TURN				3

/* define grouping types */
#define GROUP_NOTHING		'-'
#define GROUPING_CHAR       '*'
#define ANCHOR_SCHAR        '1'
#define ANCHOR_ECHAR        (ANCHOR_SCHAR + ANCHOR_NUM - 1)

/* Max files in sequence database */
#define MAX_DB_FILES	 	20	

/* these may be out of date */
#define BAD_RES_ERR         -10
#define BAD_CHAR_ERR        -20
#define MAX_RES_ERR         -30
#define MIN_RES_ERR         -40
#define END_SEQ             '/'
#define PIR_SCH             '>'
#define END_OF_SEQ          '$'



/* macros */
#define MAX(x,y)			(x>y? x: y)
#define MIN(x,y)			(x<y? x: y)
#define AA_IX(c)			(c - 'A')
#define AA_CH(c)			(c + 'A')
#define AA_UC(c)			(((c>='a') && (c<='z'))? c+'A'-'a': c)
#define AA_LC(c)			(((c>='A') && (c<='Z'))? c+'a'-'A': c)
#define AA_IVALID(x)		(((x >= 0) && (x <= 25))? TRUE: FALSE)
#define AA_CVALID(c)		(((c >= 'A') && (c <= 'Z'))? TRUE: FALSE)
#define AA_ICODE(x)			(((x!=1) && (x!=9) && (x!=14) && (x!=20) \
								&& (x!=23) && (x!=25))? TRUE: FALSE)
#define AA_CCODE(c)         ((AA_CVALID(c)) && \
							((c!='B') && (c!='J') && (c!='O') && (c!='U') \
							&& (c!='X') && (c!='Z'))? TRUE: FALSE)
#define AA_CNUM(c)			(((c >= '0') && (c <= '9'))? TRUE: FALSE)
#define RAND(c)				(((c * 32719) + 3) % 32749)

#define AA_SVALID(c)        (((c == CH_BETA) || (c == CH_COIL) \
							|| (c == CH_HELX) || (c == CH_TURN) \
							|| (c == CH_UNKNOWN))? TRUE: FALSE)



/* common structure types */

struct seqRec {
	char	*acc;
	char	*title;
	int		sFlag;
	int		uFlag;
	char	*pRes;
	char	*sRes;
	char	*uBuf;
	char	*organism;
	char	*ref;
	int		len;
	int		*resId;
};
typedef struct seqRec SEQ_REC;
