
/*
	peakcon header file
*/

#include <stdio.h>
#include <string.h>

/* definitions */
#define TRUE				1
#define FALSE				0
#define MAX_LINE_LEN		256
#define MAX_FNAME_LEN		256
#define MIN_INT				(1 << 32 - 1)
#define MAX_INT				(~MIN_INT)

/* program version and argument flags */
#define VERSION				"Version 2.0 - April/99"

/* character meanings */
#define RULE_KEY			"RULE"
#define OUTPUT_KEY			"OUTPUT"
#define FORMAT_KEY			"FORMAT"
#define FIELD_SEP			":"
#define COMMENT				'#'
#define NO_VAL				"-99"

/* Codes for different types of items to print on output */
#define P_RULEID			0
#define P_PROB				1
#define P_STRING			2
#define P_MATCH				3

/* constraints on peak pick files */
#define MAX_DATA_FILE_NUM	10		/* max number of files */
#define MAX_DATA_NUM		2500	/* max data entries */
#define MAX_DATA_DIM		8		/* max number of dimensions */
#define MAX_DATA_FIELD_NUM	100		/* max number of data fields to parse */

/* constraints on relations between the peak pick files */
#define MAX_RULE_NUM		15		/* max number of relations */
#define MAX_ATOM_NUM		60		/* max number of different atoms */
#define MAX_ATOM_NAME_LEN	4		/* max length of atom names */
#define MAX_PRINT_TERMS		40
#define INIT_SHIFT			-1

/* macros */
#define AA_UC(c)			(((c>='a') && (c<='z'))? c+'A'-'a': c)
#define MIN(x,y)            (x<y? x: y)


/* define structures */

/* each data entry of the peak pick file */
typedef struct peakRec PEAK_REC;
struct peakRec {
	int IDnum;
	double coord[MAX_DATA_DIM];
	int assignNum;
};

/* defines one peak pick file */
typedef struct dataFileRec DATA_FILE_REC;
struct dataFileRec {
	char fName[MAX_FNAME_LEN];
	int nDim;
	int nData;
	int labelIx;
	int dataIx[MAX_DATA_DIM];
	double fferr[MAX_DATA_DIM];
	double f2ferr[MAX_DATA_DIM];
	PEAK_REC peak[MAX_DATA_NUM];
};

/* defines one term in a connectivity relation */
typedef struct relTermRec REL_TERM_REC;
struct relTermRec {
	int atomIx;
	double lb;
	double ub;
};

/* defines one connectivity relationship record */
typedef struct relationRec RELATION_REC;
struct relationRec {
	int dataFileIndex;
	int assignState;
	REL_TERM_REC relTerm[MAX_DATA_DIM];
};

/* defines assignment record  */
typedef struct assignRec ASSIGN_REC;
struct assignRec {
	double prob;
	int IDnum;
	double pVal[MAX_ATOM_NUM];
};

typedef struct printRec PRINT_REC;
struct printRec {
	int ruleID;
	int atomIx;
	int pType;
	double lb, ub;
	char printFormat[MAX_LINE_LEN];
	char printBuf[MAX_LINE_LEN];
};


/* global variables */

extern char rulesFname[MAX_FNAME_LEN];
extern char resultsFname[MAX_FNAME_LEN];

extern int dataFileNum;
extern int atomNum;
extern int ruleNum;
extern int printNum;

extern DATA_FILE_REC dataFile[MAX_DATA_FILE_NUM];
extern RELATION_REC connectRule[MAX_RULE_NUM];
extern PRINT_REC printData[MAX_PRINT_TERMS];
