
/*
	file: rules.c
	procedures to read rules file
*/

#include "defs.h"

/* local variables */

static char atomNames[MAX_ATOM_NUM][MAX_ATOM_NAME_LEN];
static char fileNames[MAX_DATA_FILE_NUM][MAX_FNAME_LEN];
static int fileNum;

/* global vars */

int atomNum;
int ruleNum;
int printNum;
PRINT_REC printData[MAX_PRINT_TERMS];
DATA_FILE_REC dataFile[MAX_DATA_FILE_NUM];
RELATION_REC connectRule[MAX_RULE_NUM];

/* procedure calls */

static int get_atom_index();
static void read_rule(), read_output(), read_format(), read_relation();
extern void show_err();
extern char *strtok();
extern int atoi();


/*************************************************************/
/* read rules file 
*/
void read_rules_file(fp)
FILE *fp;
{
	register int i;
	char rLine[MAX_LINE_LEN], sVal[MAX_LINE_LEN];
	char *rc, *s1;
	int keyLenRule, keyLenFormat, keyLenOutput;

	fprintf(stdout, "\nReading rules file: %s\n\n", rulesFname);

	atomNum = 0;
	fileNum = 0;
	ruleNum = 0;

	keyLenRule = strlen(RULE_KEY);
	keyLenFormat = strlen(FORMAT_KEY);
	keyLenOutput = strlen(OUTPUT_KEY);

	rc = fgets(rLine, MAX_LINE_LEN-1, fp);
	while (rc != NULL) {

		s1 = strtok(rLine, " ");
		if (strncmp(s1, RULE_KEY, keyLenRule) == 0)
			read_rule();
		else if (strncmp(s1, FORMAT_KEY, keyLenFormat) == 0)
			read_format();
		else if (strncmp(s1, OUTPUT_KEY, keyLenOutput) == 0)
			read_output();

		rc = fgets(rLine, MAX_LINE_LEN-1, fp);
	}

/* check if we read in everything ok */
	if ((ruleNum <= 0) || (atomNum <= 0) || (fileNum <= 0)) {
		sprintf(sVal, "Problems with rulesfile: atoms=%d, files=%d rules=%d.",
			atomNum, fileNum, ruleNum);
		show_err(TRUE, "read_rules_file", sVal);
	}

/* consider command argument data files */
	if ((dataFileNum > 0) && (fileNum != dataFileNum)) {
		sprintf(sVal, "Number of datafile arguments does not match rules file");
		show_err(TRUE, "read_rules_file", sVal);
	}

/* assume correct file names are in the rules file */
	if (dataFileNum == 0) {
		for (i=0; i<fileNum; i++) {
			strcpy(dataFile[i].fName, fileNames[i]);
		}
		dataFileNum = fileNum;
	}

	fprintf(stdout, "\n Number of rules: %d\n", ruleNum);
	fprintf(stdout, " Number of distinct atoms: %d\n", atomNum);

}	/* read_rules_file */


/*************************************************************/
/* read and process each line with a "FORMAT" key 
*/
static void read_format()
{
	int num, nd;
	char *s1, sVal[MAX_LINE_LEN];
	char fData[MAX_LINE_LEN], fIntra[MAX_LINE_LEN], fInter[MAX_LINE_LEN];

	s1 = strtok((char *)NULL, " ");
	if (s1 == NULL) {
		sprintf(sVal, " File name missing in FORMAT statement");
		show_err(TRUE, "read_format", sVal);
	}
	strcpy(fileNames[fileNum], s1); 
	fprintf(stdout, " %s %12s", FORMAT_KEY, s1);

/* parse label column section */
	s1 = strtok((char *)NULL, " ");
	dataFile[fileNum].labelIx = atoi(s1);
	fprintf(stdout, " %d ",  dataFile[fileNum].labelIx);

/* read the other sections */
	s1 = strtok((char *)NULL, " ");
	strcpy(fData, s1);
	s1 = strtok((char *)NULL, " ");
	strcpy(fIntra, s1);
	s1 = strtok((char *)NULL, " ");
	strcpy(fInter, s1);

/* parse the data columns term */
	s1 = strtok(fData, FIELD_SEP);
	num = 0;
	while (s1 != NULL) {
		dataFile[fileNum].dataIx[num] = atoi(s1);
		fprintf(stdout, "%d ", dataFile[fileNum].dataIx[num]);
		s1 = strtok((char *)NULL, FIELD_SEP);
		num++;
	}
	dataFile[fileNum].nDim = num;

/* parse the intrafile error term */
	s1 = strtok(fIntra, FIELD_SEP);
	num = 0;
	while (s1 != NULL) {
		nd = sscanf(s1, "%lf", &dataFile[fileNum].fferr[num]);
		if (nd == 0) {
			sprintf(sVal, " Expecting error term but found %s instead.", s1);
			show_err(TRUE, "read_format", sVal);
		}
		fprintf(stdout, "%g ", dataFile[fileNum].fferr[num]);
		s1 = strtok((char *)NULL, FIELD_SEP);
		num++;
	}

/* parse the interfile error term */
	s1 = strtok(fInter, FIELD_SEP);
	num = 0;
	while (s1 != NULL) {
		nd = sscanf(s1, "%lf", &dataFile[fileNum].f2ferr[num]);
		if (nd == 0) {
			sprintf(sVal, " Expecting error term but found %s instead.", s1);
			show_err(TRUE, "read_format", sVal);
		}
		fprintf(stdout, "%g ", dataFile[fileNum].f2ferr[num]);
		s1 = strtok((char *)NULL, FIELD_SEP);
		num++;
	}

	fprintf(stdout, "\n");
	fileNum++;

}	/* read_format */


/*************************************************************/
/* read and process each line with a "RULE" key 
*/
static void read_rule()
{
	register int fn;
	char *s1, sVal[MAX_LINE_LEN];

	s1 = strtok((char *)NULL, " ");
	fprintf(stdout, " %s %s", RULE_KEY, s1);

/* analyze the filename */
	for (fn=0; fn<fileNum; fn++) {
		if (0 == strcmp(s1, fileNames[fn]))
			break;
	}

	if (fn == fileNum) {
		sprintf(sVal, " FORMAT statement missing for: %s\n", s1);
		show_err(TRUE, "read_rule", sVal);
	}
	connectRule[ruleNum].dataFileIndex = fn;

/* read assign state variable */
	s1 = strtok((char *)NULL, " ");
	sscanf(s1, "%d", &connectRule[ruleNum].assignState);
	fprintf(stdout, " %d", connectRule[ruleNum].assignState);

	if (connectRule[ruleNum].assignState < 0) {
		sprintf(sVal, " Bad assign state: Relation #%d, assignState %d\n",
			ruleNum, connectRule[ruleNum].assignState);
		show_err(TRUE, "read_rule", sVal);
	}

/* read connectivity relation */
	read_relation(fn);
	ruleNum++;

}	/* read_rule */


/*************************************************************/
/* reads relationship fields for one record. 
*/
static void read_relation(fn)
int fn;
{
	register int i, ix;
	int dim, nd;
	char *s1, rTerms[MAX_DATA_DIM][MAX_LINE_LEN], sVal[MAX_LINE_LEN];
	char lb[MAX_LINE_LEN], ub[MAX_LINE_LEN];

	char *termsep = " \t\n";
	char *fieldsep = ":,";

/* read each atom term */
	dim = 0;
	s1 = strtok((char *)NULL, termsep);
	while (s1 != NULL) {
		strcpy(rTerms[dim], s1);
		fprintf(stdout, " '%s'", rTerms[dim]);
		dim++;
		s1 = strtok((char *)NULL, termsep);
	}
	fprintf(stdout, "\n");

/* does number of terms equal formatting info */
	if (dim != dataFile[fn].nDim) {
		sprintf(sVal, "Number of terms(%d) does not match formatting(%d).",
			dim, dataFile[fn].nDim);
		show_err(TRUE, "read_relation", sVal);
	}

/* parse each atom term */
	for (i=0; i<dim; i++) {
		s1 = strtok(rTerms[i], fieldsep);
		ix = get_atom_index(s1, atomNames, atomNum);

/* add new atom to our list */
		if (ix >= atomNum) {
			strcpy(atomNames[ix], s1);
			atomNum++;
		}

/* are there any boundary conditions on the atom? */
		connectRule[ruleNum].relTerm[i].lb = MIN_INT;
		connectRule[ruleNum].relTerm[i].ub = MAX_INT;
		s1 = strtok((char *)NULL, fieldsep);
		if (s1 != NULL) {
			sscanf(s1, "%s", lb);
			s1 = strtok((char *)NULL, fieldsep);
			if (s1 == NULL) {
				sprintf(sVal, "Bad constraints construct at %s\n", rTerms[i]);
				show_err(TRUE, "read_relation", sVal);
			}
			sscanf(s1, "%s", ub);
			to_upper(lb, lb);
			to_upper(ub, ub);

			/* find minimum boundary */
			if ((strncmp(lb,"INF",3) == 0) || (strncmp(lb,"-INF",4) == 0)) {
				connectRule[ruleNum].relTerm[i].lb = MIN_INT;
			} else {
				nd = sscanf(lb, "%lf", &connectRule[ruleNum].relTerm[i].lb);
				if (nd == 0) {
					sprintf(sVal, "Constraints construct error, first term at %s\n", rTerms[i]);
					show_err(TRUE, "read_relation", sVal);
				}
			}

			/* find maximum boundary */
			if ((strncmp(ub,"INF",3) == 0) || (strncmp(ub,"+INF",4) == 0)) {
				connectRule[ruleNum].relTerm[i].ub = MAX_INT;
			} else {
				nd = sscanf(ub, "%lf", &connectRule[ruleNum].relTerm[i].ub);
				if (nd == 0) {
					sprintf(sVal, "Constraints construct error, second term at %s\n", rTerms[i]);
					show_err(TRUE, "read_relation", sVal);
				}
			}
		}

		connectRule[ruleNum].relTerm[i].atomIx = ix;
	}

}	/* read_relation */


/*************************************************************/
/* read in user's choice for how output should look 
*/
static void read_output()
{
	char *s1, *s2, sVal[MAX_LINE_LEN];
	int ruleID;

/* we allow limited number of print terms */
	if (printNum >= MAX_PRINT_TERMS) {
		sprintf(sVal, "Too many terms in output: %d.", MAX_PRINT_TERMS);
		show_err(TRUE, "read_output", sVal);
	}

/* read first print term */
	s1 = strtok((char *)NULL, " \n\t");
	if (s1 == NULL) {
		sprintf(sVal, "Expecting print term but none found.");
		show_err(TRUE, "read_output", sVal);
	}

/* read optional second print term, a formatting term */
/* if no formatting term, the default is string format */
	s2 = strtok((char *)NULL, " \n\t");
	if (s2 == NULL) {
		strcpy(printData[printNum].printFormat, "%s");
		fprintf(stdout, " %s %s\n", OUTPUT_KEY, s1);
	} else {
		strcpy(printData[printNum].printFormat, s2);
		fprintf(stdout, " %s %s %s\n", OUTPUT_KEY, s1, s2);
	}

/* get index of rule item if we are printing a rule item */
	printData[printNum].atomIx = get_atom_index(s1, atomNames, atomNum);

/* if following statement true, this is not a rule item */
	if (printData[printNum].atomIx == atomNum) {

/* see if this is a PROB keyword statement */
		if (0 == strncmp(s1, "PROB", 4)) {
			printData[printNum].pType = P_PROB;

/* see if this is a RULEn keyword statement */
		} else {
			if (0 == strncmp(s1, "RULE", 4)) {
				printData[printNum].pType = P_RULEID;
				ruleID = atoi(&s1[4]);
				if ((ruleID > ruleNum) || (ruleID <= 0)) {
					sprintf(sVal, "Bad RULE num on output spec: %d", ruleID);
					show_err(TRUE, "rules_output", sVal);
				}
				printData[printNum].ruleID = ruleID;

/* otherwise assume this is an arbitrary string to print */
			} else {
				printData[printNum].pType = P_STRING;
				strcpy(printData[printNum].printBuf, s1);
			}
		}
	} else {
		printData[printNum].pType = P_MATCH;
	}

	printNum++;

}   /* read_output */


/*************************************************************/
/* return index of a string in a list of strings 
*/
static int get_atom_index(query, list, listNum)
char *query;
char list[MAX_ATOM_NUM][MAX_ATOM_NAME_LEN];
int listNum;
{
	register int ix;

/* find index for the name in members */
	for (ix=0; ix<listNum; ix++) {
		if (0 == strcmp(query, list[ix])) {
			return(ix);
		}
	}

	return(ix);
  
}	/* get_atom_index */
