
######################################################################
#
# chains subroutines 
#
######################################################################

######################################################################
#
# user clicks on a connection in the chains section 
#
proc chains_panel_click { f list y } {
	global snbVar snbConst myChain

	set cur $myChain(current)
	set numConnect $myChain($cur,nConnect)
	if { $numConnect == 0 } { return }

	# find which connection item the user selected
	set snbVar(panel,chainSelect) [$snbVar(panel,chainList) nearest $y]

	# calculate new connections 
	calc_connections $snbVar(panel,chainSelect)

	# update current selection in the chains panel (eg, add cursor)
	chains_panel_update

	# update panel buttons
	if { $snbVar(panel,chainSelect) == 0 || $snbVar(panel,chainSelect) > $numConnect } {
		panel_state_buttons ""
		return
	}
	if { $snbVar(panel,chainSelect) == 1 || $snbVar(panel,chainSelect) == $numConnect } {
		panel_state_buttons "delete draw"
	} else {
		panel_state_buttons "draw"
	}
}

######################################################################
#
# user clicks on add-to-chain button
#
proc chains_add_button { } {
	global snbVar snbConst mySearch myChain

	set cur $myChain(current)
	set numConnect $myChain($cur,nConnect)

	if ![info exists mySearch($snbVar(panel,connectSelect),cix)] {
		set msg "Please click on the connectivity you wish to add to the chain."
		usnb_err $msg
	}

	set cix $mySearch($snbVar(panel,connectSelect),cix)
	set pix $mySearch($snbVar(panel,connectSelect),pix)

	add_connection_check $snbVar(panel,addToChain)
	set snbVar(panel,chainSelect) $snbVar(panel,addToChain) 

	if { $snbVar(panel,addToChain) > $numConnect || $snbVar(panel,addToChain) == 0 } {
		if { $snbVar(panel,addToChain) == 0 } {
			set snbVar(panel,chainSelect) 1
		}
	} else {
		# delete endpoint connection item first 
		delete_refId $snbVar(panel,addToChain)
		delete_connection_item $snbVar(panel,addToChain)
		if { $snbVar(panel,addToChain) == 1 } {
			set snbVar(panel,addToChain) 0
		}
	}

	# now we process the add to connection function 
	add_connection_item $snbVar(panel,addToChain) $cix $pix
	add_refId $snbVar(panel,addToChain) $cix

	update_chains_event 1
	connect_panel_update
}

######################################################################
#
# user clicks on delete-from-chain button
#
proc chains_delete_button { } {
	global snbVar snbConst myChain

	set cur $myChain(current)
	set numConnect $myChain($cur,nConnect)

	set location $snbVar(panel,chainSelect)
	delete_connection_check $location

	# do refId accounting for this connection 
	delete_refId $location

	# if only one connection in the chain, delete the whole chain
	# otherwise, just delete the connection
	if { $numConnect == 1 } { 
		delete_chain $myChain(current)
	} else {
		delete_connection_item $location
		if { $location == 1 } {
			set snbVar(panel,chainSelect) 0
		}
	}

	update_chains_event 1
}

######################################################################
#
# user selects a chain from the chains menu 
#
proc chains_menu_select { cur } {
	global snbVar snbConst myChain

	# new current chain with no connection aItem selected
	set myChain(current) $cur
	set snbVar(panel,chainSelect) -1

	# update panel showing user's current connection choice
	update_chains_event 0

	# make sure buttons on snb screen are set correctly 
	panel_state_buttons ""
}

######################################################################
#
# user selects "delete current chain" from chains menu
#
proc chains_menu_delete { } {
	global snbVar snbConst myChain

	delete_connection_check 0

	# ask user for confirmation to delete the chain	
	set msg "\nPlease confirm that you want to delete the current chain?\n"
	set resp [ usnb_show_prompt $msg ]
	if { $resp == 0 } { 
		delete_refId 0 
		delete_chain $myChain(current)
		update_chains_event 1
	}
}

######################################################################
#
# user clicks on "start a new chain" from chains menu 
#
proc chains_menu_new { } {
	global snbVar snbConst myChain

	# goto the last chain
	set myChain(current) $myChain(chainNum)

	# if no chains, then create a chains record 
	if {$myChain(chainNum) == 0} {
		chains_init
	}
	
	# if the last chain is not empty, create a chains record
	if [info exists myChain($myChain(current),nConnect)] {
		if { $myChain($myChain(current),nConnect) > 0 } {
			chains_init
		}
	}

	# update the screen 
	update_chains_event 0
}

######################################################################
######################################################################
######################################################################

######################################################################
#
# initialize a new chain at the end of our current chain list
#
proc chains_init { } {
	global snbVar snbConst myChain

	incr myChain(chainNum)
	set cur $myChain(chainNum)
	set myChain(current) $cur
	set myChain($cur,nConnect) 0
	set myChain($cur,aaIx) -1
	set myChain($cur,cLabel) "-"
	set snbVar(panel,chainSelect) -1
	set snbVar(panel,connectSelect) -1
}

######################################################################
#
# draw the current chain in the chains panel
#
proc chains_panel_update { } {
	global snbVar snbConst myChain myConnect

	set cur $myChain(current)
	set numConnect $myChain($cur,nConnect)
	set aItem $snbVar(panel,chainSelect)

	# start with empty display list  
	$snbVar(panel,chainList) delete 0 end 

	# if no elements in the chain, nothing to do 
	if { $numConnect == 0 } {
		return
	}

	# add "Start" connection to the chainList
	if { $aItem == 0 } { 
		$snbVar(panel,chainList) insert end "$snbConst(panelSelect) Start"
	} else {
		$snbVar(panel,chainList) insert end "   Start"
	}

	# add each connection to the chainList
	for {set i 1} {$i <= $numConnect} {incr i} {
		set cix $myChain($cur,$i,cix)
		if { $aItem == $i } { 
			set line "$snbConst(panelSelect) $i: $myConnect($cix,ref,0)-$myConnect($cix,ref,1)"
		} else {
			set line "   $i: $myConnect($cix,ref,0)-$myConnect($cix,ref,1)"
		}
		$snbVar(panel,chainList) insert end $line
	}

	# add "End" connection to the chainList
	if { $aItem > $numConnect } { 
		$snbVar(panel,chainList) insert end "$snbConst(panelSelect) End"
	} else {
		$snbVar(panel,chainList) insert end "   End"
	}

	# set display within the panel
	if { $aItem <= 1 } {
		$snbVar(panel,chainList) see 0
	} else {
		if { $aItem >= $numConnect } {
			$snbVar(panel,chainList) see [expr $numConnect + 1]
		} else {
			$snbVar(panel,chainList) see [expr $aItem - 1]
			$snbVar(panel,chainList) see $aItem
			$snbVar(panel,chainList) see [expr $aItem + 1]
		}
	}
}

######################################################################
#
# make a label for the chain based on sequence assignment 
#
proc chains_make_assign_label { cur } {
	global snbVar snbConst mySeq myChain

	set aaIx $myChain($cur,aaIx)
	set numConnect $myChain($cur,nConnect)
	set aaNum $mySeq($aaIx,aaNum)
	set cLabel "$aaNum:"
	for {set i 0} {$i <= $numConnect} {incr i} {
		set cLabel "$cLabel$mySeq($aaIx,name1)"
		incr aaIx 
	}
	set myChain($cur,cLabel) $cLabel
}

######################################################################
#
# make a label for the chain based on reference IDs 
#
proc chains_make_unassign_label { cur } {
	global snbVar snbConst myChain myConnect

	set numConnect $myChain($cur,nConnect)

	# empty chain
	if {$numConnect == 0} { 
		set myChain($cur,cLabel) "-"
		return
	}

	for {set i 1} {$i <= $numConnect} {incr i} {
		set refId $myConnect($myChain($cur,$i,cix),ref,0)
		if [info exists cLabel] {
			set cLabel "$cLabel-$refId"
		} else {
			set cLabel "$refId"
		}
	}
	set refId $myConnect($myChain($cur,$numConnect,cix),ref,1)
	set cLabel "$cLabel-$refId"
	set myChain($cur,cLabel) $cLabel
}

######################################################################
######################################################################
######################################################################

######################################################################
#
# see if we can "add to chain" without any errors. 
#
proc add_connection_check { aItem } {
	global snbVar snbConst myChain

	set cur $myChain(current)
	set numConnect $myChain($cur,nConnect)

	# if chain is empty, there is nothing to check 
	if { $numConnect == 0 } {
		return
	}

	# the current chain is assigned, it must be unassigned for modifications 
	if { $myChain($cur,aaIx) >= 0 } {
		set msg "This chain must be unassigned before adding new connections"
		usnb_err $msg
	}

	# currently we decide not to support the replacement of a
	# connection in the middle of a chain.
	if { $aItem > 1 && $aItem < $numConnect } {
		set msg "You cannot replace connections in the middle of a chain."
		usnb_err $msg
	}
	return
}

######################################################################
#
# add reference peaks for this chain 
#
proc add_refId { aItem cix } {
	global snbVar snbConst myChain myConnect

	set cur $myChain(current)
	set numConnect $myChain($cur,nConnect)
	set ref0 $myConnect($cix,ref,0)
	set ref1 $myConnect($cix,ref,1)

	# for a new chain, record both reference ids
	if { $numConnect == 1 } {
		lappend myChain($ref0,chainId) $cur
		lappend myChain($ref1,chainId) $cur
		return
	}

	# else we just have record the reference id of the current endpoint
	if { $aItem <= 1 } {
		lappend myChain($ref0,chainId) $cur
	}

	if { $aItem >= $numConnect } {
		lappend myChain($ref1,chainId) $cur
	}
}

######################################################################
#
# add a connectivity to our current connect chain
#
proc add_connection_item { location cix pix } {
	global snbVar snbConst myChain

	set cur $myChain(current)
	set numConnect $myChain($cur,nConnect)

	# here is what we do if the current chain is empty
	if { $numConnect == 0 } {
		set myChain($cur,1,cix) $cix
		set myChain($cur,1,pix) $pix
		set myChain($cur,nConnect) 1
		return
	}

	# we will be adding to the beginning of the chain
	if { $location == 0 } {

		# shift all the chain records up 1
		for {set i $numConnect} {$i > 0} {set i [expr $i - 1]} {
			set new [expr $i + 1]
			set myChain($cur,$new,cix) $myChain($cur,$i,cix)
			set myChain($cur,$new,pix) $myChain($cur,$i,pix)
		}
	
		# now create the first chain record 
		set myChain($cur,1,cix) $cix
		set myChain($cur,1,pix) $pix
		incr myChain($cur,nConnect) 

	# adding to the end of the chain is easier
	} else {
		incr myChain($cur,nConnect) 
		set new $myChain($cur,nConnect)
		set myChain($cur,$new,cix) $cix
		set myChain($cur,$new,pix) $pix
	}
}

######################################################################
######################################################################
######################################################################

######################################################################
#
# see if we can delete a connection from chain without any errors. 
#
proc delete_connection_check { aItem } {
	global snbVar snbConst myChain

	set cur $myChain(current)
	set numConnect $myChain($cur,nConnect)

	# chain is already empty
	if { $numConnect <= 0 } { 
		set msg "The current chain is already empty."
		usnb_err $msg
	}

	# don't allow deletion operation of an assigned chain 
	if { $myChain($cur,aaIx) >= 0 } {
		set msg "This assigned chain must be unassigned before modifying"
		usnb_err $msg
	}

	# we can not delete from chain when user selected a middle aItem 
	if { $numConnect >= 3 && $aItem > 1 && $aItem < $numConnect} {
		set msg "Please delete only at the ends of the current chain." 
		usnb_err $msg
	}
}

######################################################################
#
# delete the reference peaks which are no longer associated with this chain
#
proc delete_refId { aItem } {
	global snbVar snbConst myChain myConnect

	set cur $myChain(current)
	set numConnect $myChain($cur,nConnect)

	# delete the first reference peak
	if { $aItem == 1 } {
		set ix $myChain($cur,$aItem,cix)
		set r0 $myConnect($ix,ref,0)
		set myChain($r0,chainId) [u_list_delete $myChain($r0,chainId) $cur]
	}

	# delete the last reference peak
	if { $aItem == $numConnect } {
		set ix $myChain($cur,$aItem,cix)
		set r1 $myConnect($ix,ref,1)
		set myChain($r1,chainId) [u_list_delete $myChain($r1,chainId) $cur]
	}

	# delete all reference peaks of a chain
	if { $aItem == 0 } {
		for {set i 1} {$i <= $numConnect} {incr i} {
			set ix $myChain($cur,$i,cix)
			set r0 $myConnect($ix,ref,0)
			set myChain($r0,chainId) [u_list_delete $myChain($r0,chainId) $cur]
		}
		set r1 $myConnect($ix,ref,1)
		set myChain($r1,chainId) [u_list_delete $myChain($r1,chainId) $cur]
	}
}

######################################################################
#
# user deletes a connection from the current chain 
#
proc delete_connection_item { aItem } {
	global snbVar snbConst myChain

	set cur $myChain(current)
	set numConnect $myChain($cur,nConnect)

	# delete connection from the start of a chain
	if { $aItem == 1 } {
		set nextConnect 1
		for {set i 1} {$i < $numConnect} {incr i} {
			incr nextConnect
			set myChain($cur,$i,cix) $myChain($cur,$nextConnect,cix)
			set myChain($cur,$i,pix) $myChain($cur,$nextConnect,pix)
		}
		set myChain($cur,nConnect) [expr $numConnect - 1]
		return
	}

	# delete connection at the end of a chain is easier
	if { $aItem == $numConnect } {
		set myChain($cur,nConnect) [expr $numConnect - 1]
		return
	}

	set msg "You cannot replace connections in the middle of a chain."
	usnb_err $msg
}

######################################################################
#
# things to do when user deletes an entire chain 
#
proc delete_chain { cur } {
	global snbVar snbConst myChain mySeq

	set numConnect $myChain($cur,nConnect)
	set chainNum $myChain(chainNum)

	# we are deleting a chain which is not at the end of the chain list
	# move chain record variables up one position
	if {$cur < $chainNum} {
		set nextChain [expr $cur + 1]
		for {set j $cur} {$j < $chainNum} {incr j} {
			set myChain($j,cLabel) $myChain($nextChain,cLabel)
			set myChain($j,aaIx) $myChain($nextChain,aaIx)
			for {set k 1} {$k <= $myChain($nextChain,nConnect)} {incr k} {
				set myChain($j,$k,cix) $myChain($nextChain,$k,cix)
				set myChain($j,$k,pix) $myChain($nextChain,$k,pix)
			}
			set myChain($j,nConnect) $myChain($nextChain,nConnect)
			incr nextChain
		}

	# update assignment info so that we still refer to correct chain index
		for {set j 1} {$j < $mySeq(seqLen)} {incr j} {
			if { $mySeq($j,assignChainIx) > $cur } {
				set mySeq($j,assignChainIx) [expr $mySeq($j,assignChainIx) - 1]
			}
		}
	}

	# we now have one less chain
	set myChain(chainNum) [expr $chainNum - 1]

	# place snb in "start new chain" state. I think this makes the
	# most sense after a delete operation. 
	chains_menu_new
}

######################################################################
#
# update event when we have made changes to our current chains 
#
proc update_chains_event { changeFlag } {
	global snbVar snbConst myChain

	if { $changeFlag == 1 } {
		# we have modified the current chain
		# create new label, update the chains menu, update the chains file 
		chains_make_unassign_label $myChain(current)
		panel_menubar_chains
		data_write_chains $snbVar(fn,chains)
	} 

	# both connection and chains panels may need to be redrawn 
	chains_panel_update

	# calculate assignment possibilities and redraw sequence panel
	sequence_panel_update "FALSE"

	# update shifts window if it is active
	if [ winfo exists $snbVar(bestfit,winName) ] {
		sequence_best_assign
	}
}

######################################################################
#
# search for connections, searching is dependent on what connection
# item the user selected in the chains panel
#
proc calc_connections { aItem } {
	global snbVar snbConst myConnect myChain

	set cur $myChain(current)
	set numConnect $myChain($cur,nConnect)

	# user selected "start of chain"
	if { $aItem == 0 } {
		set cix $myChain($cur,1,cix)
		set pix $myChain($cur,1,pix)
		search_ref_ids $snbConst(NO_VAL) $myConnect($cix,ref,0) $snbConst(P_START) 1
		return
	}

	# user selected "end of chain"
	if { $aItem > $numConnect } {
		set cix $myChain($cur,$numConnect,cix)
		set pix $myChain($cur,$numConnect,pix)
		search_ref_ids $myConnect($cix,ref,1) $snbConst(NO_VAL) $snbConst(P_END) 1
		return
	}

	# user selected the only connection 
	if { $aItem == 1 && $numConnect == 1 } {
		set cix $myChain($cur,1,cix)
		set pix $myChain($cur,1,pix)
		search_ref_ids $myConnect($cix,ref,0) $myConnect($cix,ref,1) $snbConst(P_EMPTY) 2
		return
	}

	# The following possibilities involve deleting the current selection,
	# doing the search, then putting the current selection back.
	#
	# This part is complex because I am deleting the connection and then
	# searching as above. I can't think of an easier way to do this.
	# Be careful!

	set cix $myChain($cur,$aItem,cix)
	set pix $myChain($cur,$aItem,pix)

	# user selected the first connection
	if { $aItem == 1 } {
		delete_connection_item $aItem
		sequence_calc_assign "TRUE"
		search_ref_ids $snbConst(NO_VAL) $myConnect($cix,ref,1) $snbConst(P_START) 1
		add_connection_item 0 $cix $pix
		connect_panel_update
		sequence_calc_assign "TRUE"
		return
	}

	# or perhaps the user selected the last connection
	if { $aItem == $numConnect } {
		delete_connection_item $aItem
		sequence_calc_assign "TRUE"
		search_ref_ids $myConnect($cix,ref,0) $snbConst(NO_VAL) $snbConst(P_END) 1
		add_connection_item $aItem $cix $pix
		connect_panel_update
		sequence_calc_assign "TRUE"
		return
	}

	# user must have selected a middle connection
	set cix $myChain($cur,$aItem,cix)
	set pix $myChain($cur,$aItem,pix)
	search_ref_ids $myConnect($cix,ref,0) $myConnect($cix,ref,1) $snbConst(P_MID) 2
}
