
######################################################################
#
# routines to handle folded peaks 
#
######################################################################
	
######################################################################
#
# Reminder message for snb. 
#
proc folded_peaks_check { } {
	global snbVar snbConst varFold

	# remind user about folded peaks
	if { $varFold(pix) == "" } {
		usnb_log file " "
		usnb_log file "Note: Smartnotebook is unaware of any folded peaks in your peaklists."
		usnb_log file "If this is not true, please add folded peaks via the OPTIONS menu. "
		usnb_log file " "
	}
}

######################################################################
#
# user clicks on folded peaks menu item 
#
proc folded_peaks_main { } {
	global snbVar snbConst varFold mySpectra

	set f [usnb_show_win fold1 "User Designated Folded Peaks Window"]
	if { $f == "FALSE" } { return }

	message $f.intro -text \
		"The following window contains all peaks from the peaklists\
		which the user has\
		designated as folded. To add peaklist entries to this list\
		click the Add button. To delete a folded peak you can select\
		it with the mouse and hit DELETE."
	pack $f.intro

	set varFold(panel,foldPeakSelect) -1
	set varFold(saveChanges) 0
	set varFold(mainWin) $f
	folded_read_data $mySpectra(xpk,foldFnames)

	# display current folded peaks list
	pack [frame $f.win] -fill y -expand true
	label $f.win.lab -text "Spectra  PeakId    Original Shifts + Intensity      Folded Shifts + Intensity" -pady 5
	pack $f.win.lab -anchor w
	set varFold(panel,curList) [listbox $f.win.listing \
		-setgrid false -selectmode single \
		-yscrollcommand "$f.win.scroll set"]
	scrollbar $f.win.scroll -command "$f.win.listing yview"
	pack $f.win.scroll -side right -fill y
	pack $f.win.listing -side left -fill y -expand true
	bind $varFold(panel,curList) <ButtonRelease-1> \
		[list do_folded_panel_click $f $varFold(panel,curList) %y]

	# button widgets
	pack [frame $f.buttons] -pady 20
	pack [button $f.buttons.addButton -text "Add" \
		-padx $snbVar(snbwin,padx) \
		-command "do_folded_search_panel"] -side left
	pack [button $f.buttons.deleteButton -text "Delete" \
		-padx $snbVar(snbwin,padx) -state disabled \
		-command "do_folded_delete $f"] -side left
	pack [button $f.buttons.quitButton -padx $snbVar(snbwin,padx)  \
		-command "do_folded_quit $f"] -side left

	display_current_folded
}

######################################################################
#
# The user selected the "quit" button.
# Give user the option to write out folded peaks files, exit out of here.
#
proc do_folded_quit { f } {
	global snbVar snbConst varFold mySpectra

	if { $varFold(saveChanges) == 0 } {
		usnb_kill_win_save $f
		return
	}

	set msg "\nDo you wish to save the changes to your folded peaks?\n\nIf yes, I will re-calculate your connections files."

	# exit without saving the changes
	set resp [ usnb_show_prompt $msg ]
	if { $resp != 0} {
		usnb_log all "Folded peaks list not changed."
		usnb_kill_win_save $f
		return
	}

	usnb_kill_win_save $f

	# save changes to folded files
	folded_write_data $mySpectra(xpk,foldFnames) 

	# tell user this can take awhile
	set msg "\n\nThis message will disappear when the connections \nfiles have been re-calculated, please wait...\n\n"
	usnb_show_msg $msg

	# update data files and screen 
	snb_read_data $snbConst(E_XPK)
	search_from_keyboard

	# kill the wait message
	usnb_kill_msg
}

######################################################################
#
# panel to search for folded peaks in peaklists
#
proc do_folded_search_panel { } {
	global snbVar snbConst varFold myPeak mySpectra

	set wid 30
	set varFold(panel,xpkIxSearch) 0
	set varFold(panel,atom) 0
	set varFold(panel,phase) 0
	set varFold(panel,errTol) 0.50

	set f [usnb_show_win fold2 "Search Folded Peaks"]
	if { $f == "FALSE" } { return }

	label $f.winTitle -text "Folded Peaks Search"
	pack $f.winTitle -pady 20

	pack [frame $f.1]
	message $f.1.intro -text \
		"The user must enter the peakId of the candidate to fold.\
		You can type this reference Id in the entry box provided or you can\
		use the functions in the smartnotebook Reference Id menu.\
		Then Click the SEARCH button to view the candidates for peak folding." 
	pack $f.1.intro

	pack [frame $f.fr] -padx 30 -pady 20 -anchor w

	# enter peak id number
	pack [frame $f.fr.peakId] -anchor w -pady 5
	u_command_entry $f.fr.peakId "Select peakId:" $wid \
		{} -textvariable varFold(panel,refId) -width 5

	# spectra choice radiobutton
	pack [frame $f.fr.spec] -anchor w -pady 5
	label $f.fr.spec.lab -text  "From peaklist:" -width $wid -anchor e
	pack $f.fr.spec.lab -side left -anchor w
	set xpkix 0
	foreach item $mySpectra(xpk,rootNames) {
		radiobutton $f.fr.spec.$xpkix -text "$item " -value $xpkix \
			-variable varFold(panel,xpkIxSearch) \
			-command "set_folded_atoms $f.fr.atom"
		pack $f.fr.spec.$xpkix -side left -anchor w
		incr xpkix 
	}

	# select atom to fold
	pack [frame $f.fr.atom] -anchor w  -pady 5
	label $f.fr.atom.lab -text "Folding on atom:" -width $wid -anchor e
	pack $f.fr.atom.lab -side left -anchor w 
	set_folded_atoms $f.fr.atom

	# enter error tolerance 
	pack [frame $f.fr.errTol] -anchor w  -pady 5
	u_command_entry $f.fr.errTol "With Error Tolerance:" $wid \
		{} -textvariable varFold(panel,errTol) -width 5 

	# enter phase type
	pack [frame $f.fr.phase] -anchor w -pady 5
	label $f.fr.phaseLabel -text "And Phase set to:" -width $wid -anchor e
	pack $f.fr.phaseLabel -side left -anchor w
	radiobutton $f.fr.phaseButton0 -text "opposite" -value 0 \
		-variable varFold(panel,phase) 
	pack $f.fr.phaseButton0 -side left -anchor w
	radiobutton $f.fr.phaseButton1 -text "same" -value 1 \
		-variable varFold(panel,phase) 
	pack $f.fr.phaseButton1 -side left -anchor w

	# button widgets
	pack [frame $f.buttons] -pady 20
	pack [button $f.buttons.searchButton -text "Search" \
		-padx $snbVar(snbwin,padx) \
		-command "do_folded_search"] -side left
	pack [button $f.buttons.quitButton \
		-padx $snbVar(snbwin,padx) \
		-command "usnb_kill_win_save $f"] -side left
}

######################################################################
#
# User selected the SEARCH button,
# show all associated folded peaks for a selected peak id.
#
proc do_folded_search { } {
	global snbVar snbConst varFold 

	calc_folded_search 
	show_folded_choices
	display_choice_folded
}


######################################################################
#
# display atom names of the spectra the user has chosen for folded
#
proc set_folded_atoms { f } {
	global snbVar snbConst varFold myPeak

	# destroy old atom names on the window
	for {set j 0} {$j < $snbConst(MAX_DIM)} {incr j} {
		if [winfo exists $f.$j] { destroy $f.$j }
	}

	# put in the new atom names
	set count 0
	foreach atomName $myPeak($varFold(panel,xpkIxSearch),atomNames) {
		radiobutton $f.$count -text "$atomName " -variable varFold(panel,atom) \
			-value $count
		pack $f.$count -side left
		incr count
	}

	# set default to last atom
	set varFold(panel,atom) [expr $count - 1]
	update idletasks
}

######################################################################
#
# panel to handle selection of folded peaks candidates
#
proc show_folded_choices { } {
	global snbVar snbConst varFold myPeak

	set f [usnb_show_win fold3 "Folded Peaks Search Results"]
	if { $f == "FALSE" } { return }

	label $f.title -text "Folded Peaks Candidates"
	pack $f.title -pady 20

	pack [frame $f.1]
	message $f.1.intro -text \
		"Here are the peaks from all peaklists that most closely\
		match the shift values of the search peak. Your task is to\
		identify each folded peak one at a time and press ADD."
	pack $f.1.intro

	pack [frame $f.win] -pady 5 -padx 10
	# display current folded peaks list
	label $f.win.lab -text "Spectra  PeakId    Original Shifts + Intensity      Folded Shifts + Intensity" -pady 5
	pack $f.win.lab -anchor w
	set varFold(panel,choiceList) [listbox $f.win.listing \
		-setgrid false -selectmode single \
		-yscrollcommand "$f.win.scroll set"]
	scrollbar $f.win.scroll -command "$f.win.listing yview"
	pack $f.win.scroll -side right -fill y
	pack $f.win.listing -side left -fill both -expand true
	bind $varFold(panel,choiceList) <ButtonRelease-1> \
		[list do_folded_choice_select $varFold(panel,choiceList) %y]

	pack [frame $f.msg] -pady 5 -padx 10
	label $f.msg.title -text "Current Number of folded peaks: " -pady 5
	label $f.msg.num -textvariable varFold(curFoldNum) -pady 5
	pack $f.msg.title $f.msg.num -side left

	# button widgets
	pack [frame $f.buttons] -pady 20
	pack [button $f.buttons.add -text "Add" \
		-padx $snbVar(snbwin,padx) \
		-command "do_folded_choice_add"] -side left
	pack [button $f.buttons.quitButton \
		-padx $snbVar(snbwin,padx) \
		-command "usnb_kill_win_save $f"] -side left
}

######################################################################
#
# User selected the SEARCH button,
# show all associated folded peaks for a selected peak id and atom.
#
proc calc_folded_search { } {
	global snbVar snbConst varFold myPeak mySpectra

	# start off with an empty output list
	set varFold(panel,xpkixChoices) "" 
	set varFold(choice,pix) "" 

	# get folded shift value and atomName
	set pix [check_peak_id]
	set xpkix $varFold(panel,xpkIxSearch)
	set refShift [lindex $myPeak($xpkix,origPeak,$pix) $varFold(panel,atom)]
	set refName [lindex $mySpectra(xpk,fieldName,$xpkix) $varFold(panel,atom)]

	# process peaklists
	set i 0
	foreach fn $mySpectra(xpk,foldFnames) {

		set iAtom [get_folded_shift_index $i $refName]
		if { $iAtom < 0 } {
			usnb_log all "Cannot fold '$refName' for '$fn'"
		}

		# find closest peaks using current peaklist.
		find_closest_xpk_peak $refShift $refName $i $iAtom

		# next peaklist
		incr i
	}
}

######################################################################
######################################################################
######################################################################

######################################################################
#
# show list of folded peaks candidates 
#
proc display_choice_folded { } {
	global snbVar snbConst myPeak mySpectra varFold

	$varFold(panel,choiceList) delete 0 end 
	set count 0
	foreach xpkix $varFold(panel,xpkixChoices) {
		set pix [lindex $varFold(choice,pix) $count]
		set origPeak $myPeak($xpkix,origPeak,$pix)
		set line [create_folded_shift $xpkix $pix]
		set foldedPeak [usnb_peak_get $line $mySpectra(xpk,fields,$xpkix)]
		set line [create_folded_line $xpkix $pix $origPeak $foldedPeak]
		$varFold(panel,choiceList) insert end $line
		incr count
	}
}

######################################################################
#
# show list of current folded peaks 
#
proc display_current_folded { } {
	global snbVar snbConst varFold myPeak

	$varFold(panel,curList) delete 0 end 
	set count 0
	foreach xpkix $varFold(xpkix) {
		set pix [lindex $varFold(pix) $count]
		set origPeak $myPeak($xpkix,origPeak,$pix)
		set foldedPeak $myPeak($xpkix,foldedPeak,$pix)
		set line [create_folded_line $xpkix $pix $origPeak $foldedPeak]
		$varFold(panel,curList) insert end $line
		incr count
	}
	if { $count == 0 } {
		set label ""
		$varFold(panel,curList) insert end \
			"  *** No folded peaks in your peaklists have been designated. ***" 
		$varFold(mainWin).buttons.deleteButton configure -state disabled
	} else {
		set label "Spectra  PeakId    Original Shifts + Intensity      Folded Shifts + Intensity"
	}
	$varFold(mainWin).win.lab configure -text $label
	set varFold(curFoldNum) $count
}

######################################################################
#
# create an output line to display a particular folded peak 
#
proc create_folded_line { xpkix pix origPeak foldedPeak } {
	global snbVar snbConst myPeak mySpectra

	set name [lindex $mySpectra(xpk,rootNames) $xpkix]
	set pid $myPeak($xpkix,pkId,$pix)
	set line [ format "%10s %4s" $name $pid ]

	for {set i 0} {$i < $snbConst(MAX_DIM)} {incr i} {
		set item [lindex $origPeak $i]
		if { [string length $item] > 0 } {
			set tmp [format "%7.3f" $item]
		} else { 
			set tmp [format "%7s" $item]
		}
		set line "$line $tmp" 
	}

	set line "$line  "

	for {set i 0} {$i < $snbConst(MAX_DIM)} {incr i} {
		set item [lindex $foldedPeak $i]
		if { [string length $item] > 0 } {
			set tmp [format "%7.3f" $item]
		} else { 
			set tmp [format "%7s" $item]
		}
		set line "$line $tmp" 
	}
	return $line
}

######################################################################
#
# check that the peak id entered by the user is valid 
#
proc check_peak_id { } {
	global snbVar snbConst myPeak varFold

	set xpkix $varFold(panel,xpkIxSearch)
	set pid $varFold(panel,refId)

	# make sure user enters a valid peakid for this spectra
	if [ catch { set pix $myPeak($xpkix,index,$pid) } err] {
		usnb_err "Illegal peakId '$pid' for this spectra"
	}

	return $pix
}

######################################################################
######################################################################
######################################################################

######################################################################
#
# user clicked on a record in the current list of folded peaks
#
proc do_folded_panel_click { f list y } {
	global snbVar snbConst varFold
	set varFold(panel,foldPeakSelect) [$varFold(panel,curList) nearest $y]
	$f.buttons.deleteButton configure -state normal
#	usnb_debug "Click folded panel item [expr 1+$varFold(panel,foldPeakSelect)]."
}

######################################################################
#
# User selected the "delete" button,
# delete a single selected folded peak from the current list
#
proc do_folded_delete { f } {
	global snbVar snbConst varFold myPeak

	set select $varFold(panel,foldPeakSelect)
	if {$select >= 0} {
		set xpkix [lindex $varFold(xpkix) $select]
		set pix [lindex $varFold(pix) $select]
		unset myPeak($xpkix,foldedLine,$pix)
		unset myPeak($xpkix,foldedPeak,$pix)
		set varFold(xpkix) [lreplace $varFold(xpkix) $select $select]
		set varFold(pix) [lreplace $varFold(pix) $select $select]
		set varFold(saveChanges) 1
	} else {
		usnb_log all "Please select a peak for deleting with the mouse"
	}
	display_current_folded
}

######################################################################
#
# User selected the "add" button in the folded choice window,
# add a single folded peak to the current list 
#
proc do_folded_choice_add { } {
	global snbVar snbConst varFold

	set ix $varFold(panel,choiceSelect)
	set xpkix [lindex $varFold(panel,xpkixChoices) $ix]
	set pix [lindex $varFold(choice,pix) $ix]

	set varFold(saveChanges) 1
	add_folded_peak $xpkix $pix
	display_current_folded
}

######################################################################
#
# user clicked on a record in the folded peaks choices display
#
proc do_folded_choice_select { list y } {
	global snbVar snbConst varFold
	set varFold(panel,choiceSelect) [$varFold(panel,choiceList) nearest $y]
#	usnb_debug "Click folded choice panel item [expr 1 + $varFold(panel,choiceSelect)]."
}

######################################################################
######################################################################
######################################################################

######################################################################
#
# add one candidate folded peak to our current list
#
proc add_folded_peak { xpkix pix } {
	global snbVar snbConst varFold myPeak mySpectra

	# see if we already have folded peak in our folded peak list
	set count 0
	foreach i $varFold(xpkix) {
		set j [lindex $varFold(pix) $count]
		if {$i == $xpkix} {
			if {$j == $pix} {
				set name [lindex $mySpectra(xpk,rootNames) $xpkix]
				set px $myPeak($xpkix,pkId,$pix)
				usnb_err "We already have $name:$px in folded peaks list"
			}
		}
		incr count
	}

	# calculate new shift/intensity values
	set line [create_folded_shift $xpkix $pix]

	# this is a new folded peak, make record of it
	lappend varFold(xpkix) $xpkix
	lappend varFold(pix) $pix
	set myPeak($xpkix,foldedLine,$pix) $line 
	set myPeak($xpkix,foldedPeak,$pix) [usnb_peak_get $line $mySpectra(xpk,fields,$xpkix)]
}

######################################################################
#
# do the calculations to figure out folded shift value 
#
proc create_folded_shift { xpkix pix } {
	global snbVar snbConst myPeak varFold mySpectra

	# find which shift we are suppose to fold 
	set refName [lindex $mySpectra(xpk,fieldName,$varFold(panel,xpkIxSearch)) $varFold(panel,atom)]
	set ip [get_folded_shift_index $xpkix $refName]

	# get stats for this shift
	set sw [lindex $myPeak($xpkix,sw) $ip]
	set aix [lindex $mySpectra(xpk,fields,$xpkix) $ip]

	# we either add or subtract the sweep width to/from the old shift value
	set origPeak [lindex $myPeak($xpkix,origPeak,$pix) $ip]
	if { $myPeak($xpkix,ave,$aix) > $origPeak } {
		set foldedPeak [expr $origPeak + $sw]
	} else {
		set foldedPeak [expr $origPeak - $sw]
	}
	set foldedPeak [format "%1.3f" $foldedPeak]

	# create a new line which contains this folded value
	set line $myPeak($xpkix,origLine,$pix)
	set foldIx [expr [lindex $mySpectra(xpk,fields,$xpkix) $ip] - 1]
	set line [lreplace $line $foldIx $foldIx $foldedPeak]

	# invert the sign of the intensity if user selected "opposite" phase 
	if { $varFold(panel,phase) == 0 } {
		set intIx [expr [lrange $mySpectra(xpk,fields,$xpkix) end end] - 1]
		set phase [ expr 0 - [lindex $line $intIx] ]
		set line [lreplace $line $intIx $intIx $phase]
	}
	return $line
}

######################################################################
#
# return index of atom to fold in the given xpk file 
#
proc get_folded_shift_index { xpkix refName } {
	global snbVar snbConst myPeak mySpectra

	set count 0
	set iAtom -1
	foreach sName $mySpectra(xpk,fieldName,$xpkix) {
		if { 0 == [string compare $refName $sName] } {
			set iAtom $count
			break
		}
		incr count
	}
	return $iAtom
}

######################################################################
#
# return closest peak record in peakpick file to a given peak. 'fields' tell
# me an index for each element in peak for searching the peakpick file.
#
proc find_closest_xpk_peak { refShift refName xpkix iAtom } {
	global snbVar snbConst myPeak mySpectra varFold

	# test each peak in the xpk file, record difference between shifts 
	set num $myPeak($xpkix,nPeak)
	for {set iPeak 1} {$iPeak <= $num} {incr iPeak} {
		set xpkVal [lindex $myPeak($xpkix,origPeak,$iPeak) $iAtom]
		set diff [expr abs($refShift - $xpkVal)]
		if {$diff <= $varFold(panel,errTol)} {
			lappend stats "$diff $iPeak" 
		}
	}

	# Return sorted difference scores 
	if [info exists stats] {
		set sortedDiffs [lsort -command u_list_sort $stats]
		set num [llength $sortedDiffs]
		for {set i 0} {$i < $num} {incr i} {
			lappend varFold(panel,xpkixChoices) $xpkix
			lappend varFold(choice,pix) [lindex [lindex $sortedDiffs $i] 1]
		}
	}
}

######################################################################
######################################################################
######################################################################

######################################################################
#
# read all the folded-in peaks files
#
proc folded_read_data { fileNames } {
	global snbVar snbConst varFold myPeak

	set foldNum 0
    set xpkix 0
	set varFold(pix) ""
	set varFold(xpkix) ""

    set count 0
	set fileList ""
	foreach fn $fileNames {
		if { [lsearch -exact $fileList $fn] < 0 } {
			set num [read_fold_file $count $fn]
			lappend fileList $fn
			incr count
			set foldNum [expr $foldNum + $num]
		}
	}
	if {$foldNum == 0} {
		usnb_log all "Number of folded peaks found: $foldNum"
	}
	set varFold(foldNum) $foldNum
}

######################################################################
#
# read a peaklist file of folded peaks. 
#
proc read_fold_file { xpkix filename } {
	global snbVar snbConst myPeak mySpectra varFold

	if { [ catch { set foldFile [open $filename r ] } err] } {
		return 0
	}
	usnb_log all "  Reading folded peaklist: '$filename'"
	set num 0

	while {[gets $foldFile line] >= 0} {
		if { [string index $line 0] != $snbConst(COMMENT) } {
			set pkId [lindex $line 0]
			if {($pkId >= 0) && ($pkId <= 99999) && ([llength $line] > 8)} {

				if [ catch { set pix $myPeak($xpkix,index,$pkId) } err] {
					usnb_log file "Illegal peakId '$pkId' in $filename."
					continue
				}

				# save folded peak info
				set varFold($xpkix,$pix) [llength $varFold(pix)] 
				lappend varFold(pix) $pix
				lappend varFold(xpkix) $xpkix

				# save folded line and peak values
				set myPeak($xpkix,foldedLine,$pix) $line
				set myPeak($xpkix,foldedPeak,$pix) [usnb_peak_get $line $mySpectra(xpk,fields,$xpkix)]
				incr num
			}
		}
	}
	close $foldFile
	return $num
}

######################################################################
#
# write out all folded files. 
#
proc folded_write_data { fileNames } {
	global snbVar snbConst

	set count 0
	foreach xpk $fileNames {
		write_fold_file $count $xpk
		incr count
	}
}

######################################################################
#
# write out one folded peaks file (a peaklist file). 
#
proc write_fold_file { xpkix filename } {
	global snbVar snbConst myPeak varFold mySpectra

	if { [ catch { set fp [open $filename w ] } err] } {
		usnb_err "Cannot write folded peaks file '$filename'"
	}

	set name [lindex $mySpectra(xpk,rootNames) $xpkix]
	puts $fp "#"
	puts $fp "# Folded peaks for peakpick file '$name'"
	puts $fp "#"

	set num 0
	set count 0
	foreach i $varFold(xpkix) {
		if { $i == $xpkix } {
			set pix [lindex $varFold(pix) $num]
			puts $fp $myPeak($i,foldedLine,$pix)
			incr count
		}
		incr num
	}
	close $fp
	usnb_log file "Wrote $count folded peaks to '$filename'"
}
