
######################################################################
#
# nmrview draw spectra subroutines
#
# These routines are the interface to nmrview subroutines. If you wish
# to connect snb to a graphics system other than nmrview keep
# in mind that all the following routines need to be re-written AND
# the initialization files in lib/experiments/*/*.nmrview are also
# nmrview specific.
# 
# A daunting task but not beyond possibility... 
#
######################################################################


######################################################################
#
# draw spectra given x,y coordinates from the hsqc spectra 
# perhaps the user clicked on a peak or entered a reference Id for
# searching
#
proc nvdraw_ref { parms } {
	global snbVar snbConst mySpectra

	# save current active window
	# hsqc spectra is treated as special for now, just update crosshairs.
    set base_win [nv_win act]
	draw_ref_crosshairs $parms

	# now we update each spectra as was specified in the configuration file 
	foreach w $mySpectra(winList) {
		for {set i 0} {$i < $mySpectra($w,spectraNum)} {incr i} {
			set win $mySpectra($w,winName).0_$i
			if ![winfo exists $win] {
				continue
			}
			nv_win active $win

			if ![info exists mySpectra($w,$i,iRes)] { continue }
			set cNum [expr 2 * $mySpectra($w,$i,iRes)]
			set dimVals {}
			set dimWids {}

			foreach axis $snbConst(DIM_LABELS) {
				set dv $snbConst(NO_VAL)
				set dw $snbConst(NO_VAL)
				if [ info exists mySpectra($w,$i,$axis,val) ] {
					set dv [lindex $parms $cNum]
					set dw $mySpectra($w,$i,$axis,width)
					if { $dv == $snbConst(NO_VAL) } {
						# put cursor somewhere
						if { $cNum == 0 } {
							set dw 0.02
							set dv [expr [lindex [nv_win ppmMax x] 0] + $dw]
						} else {
							set dw 0
							set dv [lindex [nv_win ppmMax y] 0]
						}
					}
					incr cNum
				}
				lappend dimWids $dw
				lappend dimVals $dv
			}

			# activate the spectral window, draw the spectrum, draw peak
			# labels if necessary
			draw_strip $dimVals $dimWids
			set fn [lindex $mySpectra(xpk,outFnames) $mySpectra($w,xpk,$i)]
			show_peak_labels $fn $w $i
			NvSpectrumDraw
		}
	}
	nv_win active $base_win
}	

######################################################################
#
# draw one spectral strip 
#
proc draw_strip { dimVals dimWids } {
	global snbVar snbConst

	set ix 0
	foreach axis $snbConst(DIM_LABELS) {
		set val [lindex $dimVals $ix]
		set wid [lindex $dimWids $ix]
		incr ix	
		if { $wid == $snbConst(NO_VAL) } { continue }
		set lo [expr $val - 0.5 * $wid]
		set hi [expr $val + 0.5 * $wid]
		nv_win ppm $axis $lo $hi

		set yVal [lindex [nv_win ppmMax y] 0]
		nv_win cross1 $val $yVal 
		nv_win cross2 $val $yVal 
	}
}	

######################################################################
#
# This time update spectra given a connection record. Typically this
# is invoked when the user clicks on a connection and hits DISPLAY 
# SPECTRA button.
#
proc nvdraw_connect { cix pix } {
	global snbVar snbConst varNv myConnect mySpectra

	# Save current active window
	# Get the connection record 
    set base_win [nv_win act]

	# now we update each spectra as was specified in the configuration file 
	foreach w $mySpectra(winList) {
		for {set i 0} {$i < $mySpectra($w,spectraNum)} {incr i} {
			set win $mySpectra($w,winName).0_$i
			if ![winfo exists $win] {
				continue
			}
			nv_win active $win
			set dimVals {}
			set dimWids {}
			foreach axis $snbConst(DIM_LABELS) {
				if [ info exists mySpectra($w,$i,$axis,val) ] {
					set ix $mySpectra($w,$i,$axis,val)
					if [ info exists myConnect($cix,$pix,$ix) ] {
						lappend dimVals $myConnect($cix,$pix,$ix)
						lappend dimWids $mySpectra($w,$i,$axis,width)
					} else {
						lappend dimWids $snbConst(NO_VAL)
						lappend dimVals $snbConst(NO_VAL)
					}
				} else {
					lappend dimWids $snbConst(NO_VAL)
					lappend dimVals $snbConst(NO_VAL)
				}
			}

			# activate the spectral window, draw the spectrum, draw peak
			# labels if necessary
			draw_strip $dimVals $dimWids
			set fn [lindex $mySpectra(xpk,outFnames) $mySpectra($w,xpk,$i)]
			show_peak_labels $fn $w $i
			NvSpectrumDraw
		}
	}

	# Maybe i am paranoid, but I feel better about updating the cursors
	# only after all spectra are drawn.
	draw_crosshairs $cix $pix $snbVar(cursor,loc)

	#Restore active window.
	nv_win active $base_win
}

######################################################################
#
# We are telling nmrview to update its peaklists.
# We have made an assignment in an xpk file using snb and we want the change
# reflected in nmrview. This was a request from nmrview users.
#
proc nvdraw_update_peaklists { flag } {
	global snbVar snbConst mySpectra

	if { $flag != 1 } {
		return
	}

	# re-read each peakpick file to update nmrview
	usnb_log all "\nUpdating nmrview regarding changed peakpick files..."
	foreach w $mySpectra(winList) {
		for {set i 0} {$i < $mySpectra($w,spectraNum)} {incr i} {
			set specId $w$i
			set xpkix $mySpectra($w,xpk,$i)
			set fn [lindex $mySpectra(xpk,outFnames) $xpkix]
			usnb_log file "Calling nmrview to read peakpick file '$fn'..."
			readpks $fn snbVar(panel,flag$specId)
		}
	}
	usnb_log all "Done nmrview update.\n"
}

######################################################################
#
# update crosshairs on spectral windows when I only have x,y
# coordinates. This is only interesting for hsqc window.
#
proc draw_ref_crosshairs { parms } {
	global snbVar snbConst mySpectra

	# make sure the window exists.
	set win $mySpectra(hsqc,winName).0_0
	if ![winfo exists $win] {
		return
	}

	nv_win active $win
	set xVal [lindex $parms 0] 
	set yVal [lindex $parms 1] 
	nv_win cross1 $xVal $yVal
	set xVal [lindex $parms 2] 
	set yVal [lindex $parms 3] 
	nv_win cross2 $xVal $yVal
}

######################################################################
#
# update crosshairs on spectral windows when I have a connection
# record.
#
proc draw_crosshairs { cix pix loc } {
	global snbVar snbConst mySpectra myConnect

	foreach w $mySpectra(winList) {
		set winName $mySpectra($w,winName)
		for {set i 0} {$i < $mySpectra($w,spectraNum)} {incr i} {
			set win $mySpectra($w,winName).0_$i
			if ![winfo exists $win] {
				continue
			}
			nv_win active $win
			if [info exists mySpectra($w,$i,$loc,1,x)] {
				if [info exists mySpectra($w,$i,$loc,1,y)] {
					if { [ catch { set xVal $myConnect($cix,$pix,$mySpectra($w,$i,$loc,1,x)) } err]} { return  }
					if { [ catch { set yVal $myConnect($cix,$pix,$mySpectra($w,$i,$loc,1,y)) } err]} { return  }
					nv_win cross1 $xVal $yVal
				}
			}
			if [info exists mySpectra($w,$i,$loc,2,x)] {
				if [info exists mySpectra($w,$i,$loc,2,y)] {
					if { [ catch { set xVal $myConnect($cix,$pix,$mySpectra($w,$i,$loc,2,x)) } err]} { return  }
					if { [ catch { set yVal $myConnect($cix,$pix,$mySpectra($w,$i,$loc,2,y)) } err]} { return  }
					nv_win cross2 $xVal $yVal
				}
			}
		}
	}
}

######################################################################
#
# Tell nmrview to draw peak labels on a given spectral strip.
# Slow for the first time through reading peak file
#
proc show_peak_labels { xpk spec Id } {
	global snbVar snbConst mySpectra

	if [info exists mySpectra($spec,$Id,label)] {
		if {$mySpectra($spec,$Id,label) == 1} { 
			if ![info exists snbVar(panel,flag$spec$Id)] {
				readpks $xpk snbVar(panel,flag$spec$Id)
				set snbVar(panel,flag$spec$Id) 1
			}
			nv_win dispeaks snbVar(panel,flag$spec$Id)
		} else {
			nv_win dispeaks none
		}
	}
}
