
######################################################################
#
# preferences editing 
# This section of the program written by Leigh Willard in April 2003
#
######################################################################

######################################################################
#
# Define variables which preferences procedures will track 
#
proc prefs_constants { } {
	global snbVar snbConst myParmList mySpectra

	set myParmList($snbConst(P_PEAKLABEL)) \
		[list mySpectra(hsqc,0,label) mySpectra(hnco,0,label) \
	    mySpectra(strips,0,label) mySpectra(strips,1,label) \
	    mySpectra(strips,2,label) mySpectra(strips,3,label) \
	    mySpectra(strips,4,label) mySpectra(strips,5,label) ]
	set myParmList($snbConst(P_VSCALE)) \
		[list mySpectra(hsqc,0,lvl) mySpectra(hnco,0,lvl) \
		mySpectra(hnco,1,lvl) \
	    mySpectra(strips,0,lvl) mySpectra(strips,1,lvl) \
	    mySpectra(strips,2,lvl) mySpectra(strips,3,lvl) \
	    mySpectra(strips,4,lvl) mySpectra(strips,5,lvl) \
		snbVar(vscale,to) ] 
	set myParmList($snbConst(P_SETTINGS)) \
		[list snbVar(window,fontFamily) snbVar(window,fontSize) \
		snbVar(text,fontFamily) snbVar(text,fontSize) \
		snbVar(snbwin,colMax) snbVar(bestfit,sdevTerm) ] 
}

######################################################################
#
# This procedure is called whenever it is necessary to save and
# write the current snb defaults.
#
proc prefs_save_parms { sectiontype } {
	global snbVar snbConst initfile globalvalues myParmList mySpectra xsnb

	if { $sectiontype == $snbConst(P_GEOMETRY) } {
		set parms ""
		foreach w "snbmain snbinit snbmsg xalign orb $snbConst(WINDOWS)" {
			set parms "$parms snbVar($w,geometry)"
		}
		if [info exists mySpectra(winList)] {
			foreach w $mySpectra(winList) {
				set parms "$parms mySpectra($w,geometry)"
			}
		}
	} else {
		set parms $myParmList($sectiontype)
	}

	# load user's snb.defaults into a list.
	load_init
	set header "\tset"

	# only replace those variables that have CHANGED.  this means
	# keeping 2 lists:
	# 1) current .snb.defaults values 
	# 2) default .snb.defaults (to see what has changed)

	foreach item $parms {
		
		upvar $item value
		set glob "\*${item}\*"

		# if the item isn't defined, don't write it out	
		if ![info exists value] {continue}
		if { [llength $value] == 0 } {
			continue
		}

		set indx [lsearch $initfile $glob]
		set indxG [lsearch $globalvalues $glob]

		# sometimes need quotes 
		if { [llength $value] != 1 } { set value "\"$value\"" }

		# determine if current parameter has been changed
		set changed [check_changed $indxG $globalvalues $value ]
		if  { ! $changed  } {
			set changed [check_changed $indx $initfile $value ]
		}
		if { $indx < 0 && $indxG < 0 } { set changed 1 }
		if { ! $changed } { continue }

		if { $indx >= 0 } {
			set line [lindex $initfile $indx]
			set initfile [lreplace $initfile $indx $indx "$header $item $value"]
		} else {
			lappend initfile "$header $item $value"
		}	
	}

	# create new snb.defaults file
	write_init	
}

#............................................................................
# this procedure writes the current snb.defaults file (which is in memory
# right now with the current values)  into the .snb.defaults file.
# if an .snb.defaults file already exists, it gets backed up first.
#
proc write_init {} {
	global snbVar snbConst initfile

	if [catch {open $snbVar(fn,defaults,user) w} fp] {
		usnb_err "Cannot write to snb.defaults file '$snbVar(fn,defaults,user)'"
	}
	foreach line $initfile {
		puts $fp $line
	}
	close $fp
}

#............................................................................
# given a key/value pair (the key is associated with the index), 
# check to see if it is different from the value
# in the global snb.defaults or local snb.defaults file.
proc check_changed { index initsnb value } {
	global snbVar snbConst

	if { $index >= 0 } {

# corresponding line in the snb.defaults file
		set line [lindex $initsnb $index]


# check for 'option add key value' OR 'set key value'
# wish that I had perl here....
		set ret [string first "option" $line]
		if { $ret >= 0 } {
			set ret \
			[regexp {^[ \t]*[^ \t]+[ \t]+[^ \t]+[ \t]+[^ \t]+[ \t]+(.+)$} $line dummy old]
			if { ! $ret } { puts "ERROR 1 in prefs routine. $line" ; return 0 }
		} else {
			set ret \
			[regexp {^[ \t]*[^ \t]+[ \t]+[^ \t]+[ \t]+(.+)$} $line dummy old]
			if { ! $ret } { puts "ERROR 2 in prefs routine. $ret. $line." ; return 0 }
		}
		
# check for the new and the old (possibly surrounded by quotes) matching
		if { $old == $value || $old == "\"$value\"" } {
# debug
#			puts "SAME!  old = $old   now = $value" 
			return 0
		} else {
# debug
#			puts "DIFF!  old = $old   now = $value" 
			return 1
		}
	} else { return 0 }
}



#............................................................................
# this procedure loads into memory the current snb.defaults file.
#
proc load_init {} {
	global snbVar snbConst initfile globalvalues


	set initfile {}
	set globalvalues {}

#	read in the global snb.defaults file
	set fp [open $snbVar(fn,defaults,lib)] 

	while {[gets $fp line] >= 0} {
		lappend globalvalues $line
	}
	close $fp


#	read in the snb.defaults file in the user's directory
	if [file exists $snbVar(fn,defaults,user)] {
		set fp [open $snbVar(fn,defaults,user)]
	}  else { return }


	while {[gets $fp line] >= 0} {
		lappend initfile $line
	}

	close $fp
}
