
######################################################################
#
# sequence panel events
#
######################################################################

######################################################################
#
# user clicks on "best fit" button 
#
proc sequence_best_assign { } {
	global snbVar snbConst 

	pop_shift_analysis
	set snbVar(assignIx) [fit_find_best]
	show_shifts_chain $snbVar(assignIx) 0 
}

######################################################################
#
# User clicks on a residue in the sequence panel 
#
proc sequence_shifts_fit { aaIx } {
	global snbVar snbConst 

	pop_shift_analysis
	set snbVar(assignIx) $aaIx
	show_shifts_chain $snbVar(assignIx) 1
}

######################################################################
#
# user has moved the stddev slider button for shift fitting 
#
proc sequence_stddev_button { } {
	global snbVar snbConst

	usnb_log file "Stddev cutoff changed to $snbVar(bestfit,sdevTerm)"
	show_shifts_chain $snbVar(assignIx) 2
	sequence_panel_update "TRUE"
	prefs_save_parms $snbConst(P_SETTINGS)
}

######################################################################
######################################################################
######################################################################


######################################################################
#
# update sequence panel 
#
proc sequence_panel_update { calcFlag } {
	global snbVar snbConst mySeq 

	# determine possible assignments for the current chain
	sequence_calc_assign $calcFlag

	# show the appropriate colors on sequence panel
	for {set aaIx 1} {$aaIx <= $mySeq(seqLen)} {incr aaIx} {
		panel_residue_color $aaIx
	}
}

######################################################################
#
# find possible assignments for the current chain
#
proc sequence_calc_assign { forceFlag } {
	global snbVar snbConst myChain mySeq

	set cur $myChain(current)

	# the default is no possible assignment
	for {set aaIx 1} {$aaIx <= $mySeq(seqLen)} {incr aaIx} {
		set mySeq($aaIx,testAssign) "NOT_POSSIBLE"
	}

	# if no current chain, then exit here
	if { $myChain($cur,nConnect) <= 0 } { 
		return
	}

	# if chain is already assigned, do we still want to continue?
	if { $myChain($cur,aaIx) >= 0 && $forceFlag == "FALSE" } {
		return
	}

	# evaluate how current chain can be assigned in the sequence
	set numAssign 0
	set numTests [expr $mySeq(seqLen) - $myChain($cur,nConnect)]
	for {set aaIx 1} {$aaIx <= $numTests} {incr aaIx} {
		if { [ fit_test_chain $aaIx 1 ] < $snbVar(bestfit,sdevTerm) } {
			set_possible_assign $aaIx
			incr numAssign 
		}
	}
}

######################################################################
######################################################################
######################################################################

######################################################################
#
# Update best fit shifts window
#
proc show_shifts_chain { aaIx caller } {
	global snbVar snbConst myChain myConnect

	set cur $myChain(current)
	set numConnect $myChain($cur,nConnect)

	# if no chain and all we did was move the stddev slider, then do nothing
	if { $numConnect <= 0 && $caller == 2 } { 
		return
	}

	# no current chain, show shift information for the residue selected 
	if { $numConnect <= 0 && $caller == 1 } { 
		show_shifts_nochain $aaIx
		return
	}

	# if no chain, print message and leave
	$snbVar(panel,shiftList) delete 0 end
	if { $numConnect <= 0 } { 
		$snbVar(panel,shiftList) insert end "No chain to fit"
		return
	}

	# We can compare the shifts in the chain with this part of the sequence
	$snbVar(panel,shiftList) insert end "                  Expected  Expected   Proposed      Test"
	$snbVar(panel,shiftList) insert end " Residue  Atom       Shift    Stddev      Shift      Stat"
	$snbVar(panel,shiftList) insert end " -------  ----    --------  --------   --------  --------"

	for {set i 1} {$i <= $numConnect} {incr i} {
		set cix $myChain($cur,$i,cix)
		set pix $myChain($cur,$i,pix)
		if { $myConnect($cix,$pix,cStatus) == $snbConst(NO_VAL) && $i > 1 } {
			set j [expr $i - 1 ]
			show_shifts_res $aaIx $myChain($cur,$j,cix) $myChain($cur,$j,pix) 1 
		} else {
			show_shifts_res $aaIx $myChain($cur,$i,cix) $myChain($cur,$i,pix) 0 
		}
		incr aaIx 
	}

	# last residue in chain
	show_shifts_res $aaIx $myChain($cur,$numConnect,cix) $myChain($cur,$numConnect,pix) 1
}

######################################################################
#
# Update shifts window when no current chain exists
#
proc show_shifts_nochain { aaIx } {
	global snbVar snbConst

	$snbVar(panel,shiftList) delete 0 end
	$snbVar(panel,shiftList) insert end "                  Expected  Expected  "
	$snbVar(panel,shiftList) insert end " Residue  Atom       Shift    Stddev  "
	$snbVar(panel,shiftList) insert end " -------  ----    --------    ------  "
	show_shifts_res $aaIx -1 -1 0
}

######################################################################
#
# Display the shifts of one residue in a popup window 
#
proc show_shifts_res { aaIx cix pix connectId } {
	global snbVar snbConst mySeq mySpectra

	# process each atom of the residue
	foreach atomName $mySpectra(atoms,names) {

		set name3 $mySeq($aaIx,name3)
		set aaNum $mySeq($aaIx,aaNum)
		set sname [format " %3s %3d  %4s" $name3 $aaNum $atomName]

		# calculate the shift range for this atom
		if [info exists mySeq($aaIx,$atomName,mean)] {
			set expMean $mySeq($aaIx,$atomName,mean)
			set expSdev $mySeq($aaIx,$atomName,sdev)
			set range [format " %10.2f %9.2f" $expMean $expSdev]
		} else {
			set range [format " %10s %9s" "-" "-"]
		}

		# now we can construct a basic output line with mean and sdev only
		set line [format "%s %s" $sname $range]

		# if no chain made, then we save the line. 
		if { $cix < 0 }  {
			if [info exists mySeq($aaIx,$atomName,mean)] {
				$snbVar(panel,shiftList) insert end $line
			}
			continue
		}

		# get proposed shift from connection record
		# analyze proposed chemical shift with chemical shift range values
		set propose [ connect_get_val $cix $pix $mySpectra(connect,$atomName,$connectId) ] 
		set rc [fit_test_shift $aaIx $propose $atomName]

		# proposed shift value does not exist but the sequence expects one 
		if { $rc == $snbConst(MISSING_STAT) } {
			set line [format "%s %10s %9s %s" $line "-" "-" "X"] 
			$snbVar(panel,shiftList) insert end $line 
			continue
		}

		# proposed shift value exists but the sequence is not expecting one
		if { $rc == $snbConst(EXTRA_STAT) } {
			set line [format "%s %10.2f %9s %s" $line $propose "-" "X"]
			$snbVar(panel,shiftList) insert end $line 
			continue
		}

		# a missing shift matches a missing shift
		if { $rc == $snbConst(NA_STAT) } {
			set line [format "%s %10s %9s %s" $line "-" "-" " "]
			$snbVar(panel,shiftList) insert end $line 
			continue
		}

		if { [expr abs($rc)] > $snbVar(bestfit,sdevTerm) } {
			set line [format "%s %10.2f %9.2f %s" $line $propose $rc "X"]
		} else {
			set line [format "%s %10.2f %9.2f %s" $line $propose $rc " "]
		}
		$snbVar(panel,shiftList) insert end $line 
	}
}

######################################################################
#
# mark a chain in the section for possible assignment 
#
proc set_possible_assign { iStart } {
	global snbVar snbConst myChain mySeq

	set cur $myChain(current)
	set iEnd [expr $iStart + $myChain($cur,nConnect) ]

	# mark this section for possible assignment
	for {set aaIx $iStart} {$aaIx <= $iEnd} {incr aaIx} {
		set mySeq($aaIx,testAssign) "POSSIBLE"
	}

	# first residue in the chain is designated differently
	# so that we can tell where the chain begins
	set mySeq($iStart,testAssign) "START_POSSIBLE"
}

######################################################################
#
# show residue shifts 
#
proc pop_shift_analysis { } {
	global snbVar snbConst

	# if window already exists, there is nothing to do
	set f [usnb_show_win bestfit "Fit current chain to shift constraints"]
	if { $f == "FALSE" } { return }

	frame $f.sdev
	frame $f.win
	frame $f.buttons
	pack $f.sdev
	pack $f.win -fill both -expand true -pady 10 -padx 10
	pack $f.buttons -pady 10

	# allow user to change chemical shifts standard deviation
	scale $f.sdev.scale -variable snbVar(bestfit,sdevTerm) \
		-orient horizontal -label "Stddev limit on assignment     "
	pack $f.sdev.scale
	bind $f.sdev.scale <ButtonRelease> "sequence_stddev_button"
	
	# create list with a scrollbar 
	set snbVar(panel,shiftList) [listbox $f.win.listing \
		-setgrid false -selectmode single \
		-yscrollcommand "$f.win.scroll set"]
	scrollbar $f.win.scroll -command "$f.win.listing yview"
	pack $f.win.scroll -side right -fill y
	pack $f.win.listing -side left -fill both -expand true

	# buttons for this window
	button $f.buttons.assign -text "Assign" \
		-padx $snbVar(snbwin,padx) \
		-command "assign_add_item"
	button $f.buttons.unassign -text "Unassign" \
		-padx $snbVar(snbwin,padx) \
		-command "assign_delete_item"
	button $f.buttons.quit -text "Dismiss" \
		-padx $snbVar(snbwin,padx) \
		-command "usnb_kill_win_save $f"
	pack $f.buttons.assign $f.buttons.unassign $f.buttons.quit -side left 
}
