
######################################################################
#
# "Suggest" chains menu routines 
#
######################################################################

######################################################################
#
# The user selected a suggested chain from the "Suggest" menubar
#
proc suggest_menu_select { item } {
	global snbVar snbConst myChain mySuggest myConnect

	# init a new chain
	incr myChain(chainNum)
	set cur $myChain(chainNum)
	set myChain(current) $cur
	set myChain($cur,nConnect) $mySuggest($item,nConnect) 
	set myChain($cur,aaIx) $mySuggest($item,aaIx)
	set myChain($cur,cLabel) $mySuggest($item,cLabel) 

	# copy over suggested chain as our new current chain 
	set numConnect $myChain($cur,nConnect)
	for {set i 1} {$i <= $numConnect} {incr i} {
		set cix $mySuggest($item,$i,cix)
		set pix $mySuggest($item,$i,pix)
		set myChain($cur,$i,cix) $cix
		set myChain($cur,$i,pix) $pix
		set ref0 $myConnect($cix,ref,0)
		lappend myChain($ref0,chainId) $cur
	}
	set ref1 $myConnect($cix,ref,1)
	lappend myChain($ref1,chainId) $cur
	set snbVar(panel,chainSelect) 0
	set snbVar(panel,connectSelect) -1

	# update panels and smartnotebook gui based on this new chain
	chains_panel_update
	sequence_panel_update "TRUE"
	usnb_kill_win_save $snbVar(bestfit,winName)
	panel_state_buttons ""
	panel_menubar_chains
}

######################################################################
#
# This is the popup window when the user clicks "Suggest Chains..." 
#
proc suggest_panel { } {
	global snbVar snbConst varSuggest

	set f [usnb_show_win reinit "suggest_chains ..."]
	if { $f == "FALSE" } { return }

	label $f.title -text "Suggest Chains to Smartnotebook"
	pack $f.title -pady 10

	set varOption(keep) 0
	set varOption(move) 0
	pack [frame $f.init] -pady 10 -padx 20
	message $f.init.intro -text \
		"There are two ways to import suggested chains into smartnotebook.\
		First, you can enter a chains formatted input file. Second, enter\
		an assignment file in ppm.out format and have the software match\
		the assignments to the connectivity data. Form chains based on the\
		matched connectivity records. "
	pack $f.init.intro

	pack [frame $f.type]
	label $f.type.label -text "Format:" -width 10 -anchor w
	pack $f.type.label -side left -pady 5
	foreach t "chains ppm-out" {
		radiobutton $f.type.$t -value $t -text "$t " \
			-variable snbVar(suggest,type)
		pack $f.type.$t -side left -padx 5
	}

	u_file_entry $f.chains    "FileName: " \
	        18 varSuggest(fn) 44 $snbVar(snbwin,padx) { } 0 0
	pack $f.chains -pady 10

	# buttons for this window
	frame $f.buttons
	pack $f.buttons -pady 20
	button $f.buttons.exe -text "Execute" \
		-padx $snbVar(snbwin,padx) \
		-command "do_suggest_chains $f"
	button $f.buttons.quitButton \
		-padx $snbVar(snbwin,padx) \
		-command "usnb_kill_win_save $f"
	pack $f.buttons.exe $f.buttons.quitButton -side left 
}

######################################################################
#
# User hits the execute button of the "Suggest Chains..." panel
#
proc do_suggest_chains { f } {
	global snbVar snbConst varSuggest

	# make sure we can read the input file
	if ![file readable $varSuggest(fn)] { 
		usnb_err "Unable to read file: '$varSuggest(fn)'"
	}

	# process the chains formatted input file 
	# keep a copy in chains.suggest so that we can read it on startup of snb
	if { $snbVar(suggest,type) == "chains" } {
		sys_remove_file $snbVar(fn,suggest)
		sys_copy_file $varSuggest(fn) $snbVar(fn,suggest)
		suggest_read_chains $varSuggest(fn)
		panel_menubar_suggest
		usnb_log all "Suggested chains read from '$varSuggest(fn)'"
		usnb_kill_win_save $f
		return
	}

	# process the ppm.out formatted input file (lots of work here)
	do_ppmout $varSuggest(fn) $snbVar(fn,suggest)
	usnb_kill_win_save $f
}

######################################################################
#
# Build suggested chains from ppm.out assignment file
#
proc do_ppmout { fn outfn } {
	global snbVar snbConst varSuggest mySuggest

	data_read_ppm $fn suggestAssign
	scan_ppmout

	# take assigned reference peaks and see if there are consecutive entries
	# create chains for these. 
	set chainNum [create_suggest_chains]
	set mySuggest(chainNum) $chainNum

	# write any suggested chains to a file
	sys_remove_file $outfn
	if { $chainNum > 0 } {
		write_suggest_chains $outfn
		suggest_read_chains $outfn
		panel_menubar_suggest
		usnb_log all "Suggested chains read from '$outfn'"
	}
}

######################################################################
#
# Determine which amino acids have assignments for chain creation
#
proc scan_ppmout { } {
	global snbVar snbConst varSuggest mySeq mySpectra

	set i 0
	while { $i <= $mySeq(seqLen) } {
		incr i
		set flag 0
		foreach atomName $mySpectra(REF,atomNames) {
			if ![ info exists mySeq($i,$atomName,suggestAssign) ] {
				set flag 1
				break
			}
		}
		if { $flag == 0 } {
			# find possible reference Ids that match the current amino acid
			set varSuggest($i,ids) [ find_ref_ids $i ]
			if { $varSuggest($i,ids) == "" } {
				usnb_log all "Warning, no reference peak picked to match ppm.out shifts at sequence residue id $i."
				set varSuggest($i,found) 1 
			} else {
				set varSuggest($i,found) 0 
			}
		} else {
			set varSuggest($i,found) 1 
		}
	}
}

######################################################################
#
# Create each chain of two or more adjacently assigned residues of
# the ppm.out formatted input file
#
proc create_suggest_chains { } {
	global snbVar snbConst varSuggest mySeq mySpectra myConnect

	set next 1
	set chainTot 0
	set numConnect 0
	for {set i 1} {$i <= $mySeq(seqLen)} {incr i} {

		# Look for adjacent residues with assignments 
		incr next
		if { $varSuggest($i,found) == 0 && $varSuggest($next,found) == 0 } {
			set bestErr $snbConst(MAX_STAT) 
			foreach id1 $varSuggest($i,ids) {
				foreach id2 $varSuggest($next,ids) {
					set ans [find_connect_fit $i $id1 $id2]
					set err [lindex $ans 0]
					if { $err < $bestErr } {
						set bestErr $err
						set cix [lindex $ans 1]
						set pix [lindex $ans 2]
					}
				}
			}

			# is this the start of a new chain?
			if { $numConnect == 0 } {
				incr chainTot 
				set seqIx $i
			}

			set varSuggest($chainTot,$numConnect,pkid) ""
			if { $bestErr >= $snbConst(MAX_STAT) } {
				set varSuggest($chainTot,$numConnect,ref1) [lindex $id1 0]
				set varSuggest($chainTot,$numConnect,ref2) [lindex $id2 0]
				set varSuggest($chainTot,$numConnect,rule) $snbConst(NO_RULE)
				set varSuggest($chainTot,$numConnect,prob) 0.0
				set varSuggest($chainTot,$numConnect,pix) $snbConst(NO_VAL)
				foreach resInc "0 1" {
					foreach field $mySpectra(connect,PKID,$resInc) {
						lappend varSuggest($chainTot,$numConnect,pkid) $snbConst(NO_VAL)
					}
				}
			} else {
				set varSuggest($chainTot,$numConnect,ref1) $myConnect($cix,ref,0) 
				set varSuggest($chainTot,$numConnect,ref2) $myConnect($cix,ref,1)
				set varSuggest($chainTot,$numConnect,rule) $myConnect($cix,rule)
				set varSuggest($chainTot,$numConnect,prob) 1.0
				set varSuggest($chainTot,$numConnect,pix) $pix
				foreach resInc "0 1" {
					foreach field $mySpectra(connect,PKID,$resInc) {
						lappend varSuggest($chainTot,$numConnect,pkid) $myConnect($cix,$pix,$field)
					}
				}
			}

			# set chain label
			if { $numConnect == 0 } {
				set chainLabel "$i:$varSuggest($chainTot,$numConnect,ref1)"
			}
			# add id2 to the end of current chain
			set chainLabel "$chainLabel-$varSuggest($chainTot,$numConnect,ref2)"
			incr numConnect	

		# these two adjacent residues have not been assigned 
		# this marks the end of the current chain
		} else {
			if {$numConnect > 0} {
			 	set head [format "%-3d %3d   %s" $seqIx $numConnect $chainLabel]
				set varSuggest($chainTot,header) $head
				set varSuggest($chainTot,numConnect) $numConnect
				set numConnect 0
			}
		}
	}
	return $chainTot
}

######################################################################
#
# Given reference Ids ref0 and ref1 and the location of where we are
# in the sequence, return the best fitting connectivity record. 
#
proc find_connect_fit { aaIx ref0 ref1 } {
	global snbVar snbConst myConnect mySpectra mySeq

	set bestcix 0
	set bestpix 0
	set besterr $snbConst(MAX_STAT)

	foreach rule $myConnect(c_rules) {

		if ![info exists myConnect(index,$rule,$ref0,$ref1)] {
			continue
		}

		# we have a possible connection to check
		set cix $myConnect(index,$rule,$ref0,$ref1)
		if { $bestpix == 0 } {
			set bestcix $cix
		}

		for {set pix 1} {$pix <= $myConnect($cix,pNum)} {incr pix} {
			set totErr 0.0
			foreach resInc "0 1" {
				set ix [expr $resInc + $aaIx]
				foreach atomName $mySpectra(connect,PKID,atom) {
					set iShift $mySpectra(connect,$atomName,$resInc)
					set shift $myConnect($cix,$pix,$iShift)
					set errIx [lsearch $mySpectra(atoms,names) $atomName]
					set err [lindex $mySpectra(atoms,errTol) $errIx]
					if [info exists mySeq($ix,$atomName,suggestAssign)] {
						set ppmShift $mySeq($ix,$atomName,suggestAssign)
					} else {
						set ppmShift $snbConst(NO_VAL) 
					}
					set errTerm [expr abs(($ppmShift - $shift) / $err)]
					if {$errTerm > 1.0} {
						set totErr [expr $totErr + $snbConst(MAX_STAT)]
					} else {
						set totErr [expr $errTerm + $totErr]
					}
				}
			}
			if { $totErr < $besterr } {
				set besterr $totErr
				set bestpix $pix
			}
		}
	}
	return "$besterr $bestcix $bestpix"
}

######################################################################
#
# Compare the shift values from our input file from those in our
# reference peaklist. Return all those reference Ids whose shifts match close
# enough (eg, errTol) to the input file shifts.
#
proc find_ref_ids { aaIx } {
	global snbVar snbConst varSuggest mySeq mySpectra myPeak

	foreach atomName $mySpectra(REF,atomNames) {
		lappend ppmShift $mySeq($aaIx,$atomName,suggestAssign)
	}

	set ans ""
	set xpkIx $mySpectra(REF,xpkIx)
	for {set j 1} {$j <= $myPeak($xpkIx,nPeak)} {incr j} {
		set peak $myPeak($xpkIx,origPeak,$j)
		set flag 0
		set ix 0
		foreach atomName $mySpectra(REF,atomNames) {
			set s1 [ lindex $peak $ix ]
			set s2 [ lindex $ppmShift $ix ]
			set iErr [ lsearch $mySpectra(atoms,names) $atomName ]
			set err [ lindex $mySpectra(atoms,errTol) $iErr ]
			if { $s2 < [expr $s1 - $err] || $s2 > [expr $s1 + $err] } {
				set flag 1
				break
			}
			incr ix
		}
		if { $flag == 0 } {
			set ans "$ans $myPeak($xpkIx,pkId,$j)"
		}
	}
	return $ans
}

######################################################################
#
# write out suggested chains from assigned atoms in reference input peaklist 
#
proc write_suggest_chains { results } {
	global snbVar snbConst varSuggest mySuggest

	if {[ catch { set fp [open $results w ] } err]} {
		usnb_err "Cannot write to file '$results'"
	}

	puts $fp "#"
	puts $fp "# This file keeps track of suggested chains."
	puts $fp "# It was made from the assignments made in the input"
	puts $fp "# ppm.out and peakpick files."
	puts $fp "#"
	set num $mySuggest(chainNum)
	if { $num >= 1 } {
		puts $fp "1   (only for compatibility with chains file)" 
	}

	for {set i 1} {$i <= $num} {incr i} {
		puts $fp "$varSuggest($i,header)"
		for {set j 0} {$j < $varSuggest($i,numConnect)} {incr j} {
			set line [format " %3d %3d %s %4.2f   " \
				$varSuggest($i,$j,ref1) $varSuggest($i,$j,ref2) \
				$varSuggest($i,$j,rule) $varSuggest($i,$j,prob)]
			set line "$line $varSuggest($i,$j,pkid)"
			puts $fp $line
		}
	}

	close $fp
	usnb_log file "Wrote $num chain suggestions to file '$results'"
}

######################################################################
#
# read a chains suggestion file 
#
proc suggest_read_chains { fn } {
	global snbVar snbConst mySuggest myConnect mySeq

	set mySuggest(chainNum) 0

	# if file doesn't exist, then nothing to do
	if ![file exists $fn] { return }

	# file can not be open for reading
	if {[ catch { set fp [open $fn r ] } err]} { 
		usnb_err "Cannot open chains file '$fn', check permissions"
	}

	# read first line to find out the user's current display chain
	while {[gets $fp line] >= 0} {
		if { [string index $line 0] != $snbConst(COMMENT)} {
			set mySuggest(current) [lindex $line 0]
			break
		}
	}

	# read chains we have previously made
	set cur 0
	while {[gets $fp line] >= 0} {
		if { [string index $line 0] == $snbConst(COMMENT)} {
			continue
		}
		incr cur

		# suggested chains cannot be assigned, user must assign them
		set mySuggest($cur,aaIx) -1
		set mySuggest($cur,nConnect) [lindex $line 1 ] 
		set mySuggest($cur,cLabel) [lindex $line 2] 

		# now read each connection record within a single chain
		set aaIx $mySuggest($cur,aaIx)
		set count 1
		while { $count <= $mySuggest($cur,nConnect) } {
			if {[gets $fp line] < 0} {
				usnb_err "  Unexpected EOF at chains file '$fn', chain $count."
			}
			if { [string index $line 0] == $snbConst(COMMENT)} {
				continue
			}
			set ref0 [lindex $line 0 ]
			set ref1 [lindex $line 1 ]
			set rule [lindex $line 2 ]

			# find and save the index into connections data structure 
			if [info exists myConnect(index,$rule,$ref0,$ref1)] {
				set cix $myConnect(index,$rule,$ref0,$ref1)
			} else {
				incr myConnect(connectNum)
				set cix $myConnect(connectNum)
				connect_create_ref $cix $rule $ref0 $ref1
			}

			set pix [usnb_get_pix $line $cix ]
			if { $pix == $snbConst(NO_VAL) } {
				usnb_log all "Warning, no connectivity record for\n $rule $ref0 $ref1\n"
				connect_create_item $cix $line $snbConst(NO_VAL)
				set pix $myConnect($cix,pNum)
			}
			set mySuggest($cur,$count,cix) $cix
			set mySuggest($cur,$count,pix) $pix
			incr count
		}
	}
	close $fp

	# this is how many suggested chains we have currently 
	set mySuggest(chainNum) $cur
}
